// ---------------------------------------------------------------------------------
// Quick Battle 2 editor
//
// by Uwe Brinkschulte
//
// Handle Configuration Header File
// ----------------------------------------------------------------------------------

#ifndef __QB2CONFIG_H
#define __QB2CONFIG_H

// ------------------------------ defines -------------------------------------------

// configuration file constants

// path length
#define PATH_LENGTH  512

// line length
#define LINE_LENGTH  512

// the maximum qb2 line length
#define MAX_QB2_LINELENGTH  255

// maximum title length
#define MAX_TITLE       50

// maximum title line length
#define MAX_TITLELINE  200

// maximum dialog speaker length
#define MAX_SPEAKER     40

// maximum dialog line length
#define MAX_DIALINE    200

// maximum self destruct code length
#define MAX_SELFDESTRUCTCODE    10

// path to qb2.txt file
#define QB2DESC_PATH "scripts\\qb2.txt"

// path to battles
#define QB2_PATH "scripts\\"

// name of the file containing the available ships
#define QB2_SHIPFILE "qbships.txt"

// name of the file containing the available systems and planets
#define QB2_SYSTEMPLANETFILE "qb2sys.txt"

// qb2.txt file magic and version
#define QB2DESC_MAGIC   "QB2 Available Battles"
#define QB2DESC_VERSION "File Format Version 1.0"

// .qb2 file magic and version
#define QB2_MAGIC    "QB2 Battle"
#define QB2_VERSION1 "File Format Version 1.0"
#define QB2_VERSION2 "File Format Version 1.1"
#define QB2_VERSION3 "File Format Version 1.2"
#define QB2_VERSION4 "File Format Version 1.3"

// the qb2editor brand to identify a configuration created by the editor
#define QB2EDIT_BRAND "// Created by Quick Battle 2 Editor"

// maximum number of allowed systems
#define MAX_SYSTEMS  6

// configuration file syntax

// the remark sign
#define REMARK        "//"

// the keywords
#define KEY_SYSTEM              "System"
#define KEY_STARDATE            "Stardate"
#define KEY_ESCORT              "Escort"
#define KEY_FRIEND              "Friend"
#define KEY_ENEMY               "Enemy"
#define KEY_NEUTRAL             "Neutral"
#define KEY_HULK                "Hulk"
#define KEY_PLAYER              "Player"
#define KEY_FRIENDPOS           "FriendPosition"
#define KEY_ENEMYPOS            "EnemyPosition"
#define KEY_PLAYERPOS           "PlayerPosition"
#define KEY_NEUTRALPOS          "NeutralPosition"
#define KEY_HULKPOS             "HulkPosition"
#define KEY_NEUTRALBEHAVIOR     "NeutralBehavior"
#define KEY_BATTLEMODE          "BattleMode"
#define KEY_SHIPCHANGE          "ShipChange"
#define KEY_WARPOUT             "WarpOut"
#define KEY_NOWARP              "WarpDisable"
#define KEY_COLLISION           "Collision"
#define KEY_FRIENDACTION        "FriendAction"
#define KEY_ENEMYACTION         "EnemyAction"
#define KEY_NEUTRALACTION       "NeutralAction"
#define KEY_PERFECTCLOAK        "PerfectCloak"
#define KEY_TITLE               "Title"
#define KEY_TITLELINE           "TitleLine"
#define KEY_EPILOGLINE          "EpilogLine"
#define KEY_STARTDIALOG         "StartingDialog"
#define KEY_ENDDIALOG           "EndingDialog"
#define KEY_FRIENDWARPINDIALOG  "FriendWarpInDialog"
#define KEY_ENEMYWARPINDIALOG   "EnemyWarpInDialog"
#define KEY_NEUTRALWARPINDIALOG "NeutralWarpInDialog"
#define KEY_FRIENDWARPOUTDIALOG "FriendWarpOutDialog"
#define KEY_ENEMYWARPOUTDIALOG  "EnemyWarpOutDialog"
#define KEY_NEUTRALWARPOUTDIALOG "NeutralWarpOutDialog"
#define KEY_JOINESCORTDIALOG    "FriendJoinEscortDialog"
#define KEY_NEUTRALTOFRIENDDIALOG "NeutralToFriendDialog"
#define KEY_NEUTRALTOENEMYDIALOG "NeutralToEnemyDialog"
#define KEY_JUMPGATE            "JumpGate"
#define KEY_JUMPPOS             "JumpPosition"
#define KEY_JUMPTIME            "JumpTime"
#define KEY_JUMPDEST            "JumpDestination"
#define KEY_JUMPAPPEARSDIALOG   "JumpGateAppearsDialog"
#define KEY_JUMPDIALOG          "JumpDialog"
#define KEY_JUMPOPTION          "JumpOption"
#define KEY_SELFDESTRUCT        "SelfDestruct"

// sub keywords
#define SUBKEY_PLANET           "Planet"
#define SUBKEY_STATBATTLE       "Stat"
#define SUBKEY_AIBATTLE         "AI"
#define SUBKEY_ANYTIME          "Anytime"
#define SUBKEY_CLEARED          "WhenCleared"
#define SUBKEY_SEQUENTIAL       "Sequential"
#define SUBKEY_OFF              "Off"
#define SUBKEY_OFFALL           "OffForAll"
#define SUBKEY_ON               "On"
#define SUBKEY_AUTO             "Auto"
#define SUBKEY_ENEMYWHENATTACK  "EnemyWhenAttacked"
#define SUBKEY_ENEMYAFTER       "EnemyAfter"
#define SUBKEY_FRIENDAFTER      "FriendAfter"
#define SUBKEY_RANDOM           "Random"
#define SUBKEY_ENEMY            "Enemy"
#define SUBKEY_FRIEND           "Friend"
#define SUBKEY_NEUTRAL          "Neutral"
#define SUBKEY_PLAYER           "Player"
#define SUBKEY_WARPIN           "WarpInAfter"
#define SUBKEY_WARPOUT          "WarpOutAfter"
#define SUBKEY_JOINESCORT       "JoinEscortAfter"
#define SUBKEY_TCLEARED         "Cleared"
#define SUBKEY_JUMPONLY         "JumpOnly"
#define SUBKEY_CLEARJUMP        "ClearOnJump"
#define SUBKEY_ACTDISTANCE      "ActivationDistance"
#define SUBKEY_SHOWACT          "ShowActivation"
#define SUBKEY_ACTONHAIL        "ActivateOnHail"
#define SUBKEY_NOSPIN           "NoSpin"
#define SUBKEY_GATEHIDDEN       "HiddenGate"
#define SUBKEY_STANDARD         "Standard"
#define SUBKEY_OVERKILL         "Overkill"
#define SUBKEY_SELOVERKILL      "SelectiveOverkill"

// --------------------------- simple types -----------------------------------------

// the possible neutral behaviors
typedef enum ENeutBehavior      {EnemyWhenAttacked, EnemyAfter, FriendAfter, RandomBehavior};

// the possible warp out modes
typedef enum EWarpOutMode       {Anytime, WhenCleared, Sequential};

// the possible ship change modes
typedef enum EShipChangeMode    {NoShipChange, ManualShipChange, AutoShipChange};

// the possible battle modes
typedef enum EBattleMode        {AIBattle, StatisticBattle};

// the possible collision modes
typedef enum ECollisionMode     {CollisionsOn, CollisionsOff, CollisionsOffForAll};

// the possible perfect cloak modes
typedef enum EPerfectCloakMode  {PerfectCloakOff, PerfectCloakOn};

// the possible self destruct modes
typedef enum ESelfDestructMode  {StandardDestruct, OverkillDestruct, SelectiveOverkillDestruct};

// ------------------------------  function prototypes ---------------------------------

// special string handling functions needed for parsing a config file
BOOL IsNumber(char *src);
void GetSubString(char *dest, const char *src, unsigned iStart, unsigned iEnd);
void GetNthWordInString(char *dest, char *src, unsigned n);
void Tokenize(char *dest, char *src, char tokendelimiter);
void StripString(char *dest, char *src);

// --------------------------- classes ----------------------------------------------

// the configuration classes

// ---------------  the config file class -----
class CCfgFile 
{
  FILE *pConfigFile;
  char strFName[PATH_LENGTH];

  public:
  // the constructor
  CCfgFile();

  // open the config file
  BOOL Open(const char *fname);

  // create the config file
  BOOL Create(const char *fname);

  // close the config file
  void Close();

  // resetthe config file
  BOOL Reset();

  // read a config file line
  BOOL ReadLine(char *line);

  // write a config file line
  BOOL WriteLine(char *line);
};


// ---------------  the string list class, contains a list of strings ------
class CStrList : public CObList {};


// ---------------  the ship data class, contains all info of a specific ship ------
class CShipData : public CObject
{
  public:
  // override constructor
  CShipData(void);

  // set data functions
  // the class of the ship
  void SetShipClass(const char *ShipClass)   {m_ShipClass = ShipClass; m_ShipClassSet = TRUE;}
  // the name of the ship
  void SetShipName(const char *ShipName)     {m_ShipName = ShipName; m_ShipNameSet = TRUE;}

  // get data functions
  // the class of the ship
  CString& GetShipClass(void)          {return m_ShipClass;}
  // the name of the ship
  CString& GetShipName(void)           {return m_ShipName;}

  // test data set functions
  // the class of the ship
  BOOL IsSetShipClass(void)            {return m_ShipClassSet;}
  // the name of the ship  
  BOOL IsSetShipName(void)             {return m_ShipNameSet;}

  // unset data functions
  // the class of the ship
  void UnsetShipClass(void)            {m_ShipClassSet = FALSE;}
  // the name of the ship  
  void UnsetShipName(void)             {m_ShipNameSet = FALSE;}

  protected:

  // the data elements
  CString       m_ShipClass;
  CString       m_ShipName;

  // the data set markers
  BOOL          m_ShipClassSet;
  BOOL          m_ShipNameSet;
};


// ---------------  the ship list class, contains a list of ships ------
class CShipList : public CObList
{
  public:
// look if a ship exists
// (if pDefault is not NULL, we check as well for default shipnames. To construct
//  the name the type ('F' = friend, 'E' = escort, 'O' = enemy, 'H' = hulk, 'N' = neutral)
//  and system number must be given. pDefault is set to true if the shipname matches a 
//  default name)
  BOOL ExistsShip(const char *strShipName, BOOL bDefault, char cType, unsigned uSysNum);
};


// ---------------  the dialog data class, contains all info of a specific dialog ------
class CDialogData : public CObject
{
  public:
  // override constructor
  CDialogData(void);

  // set data functions
  // the speaker
  void SetSpeaker(char *Speaker)         {m_Speaker = Speaker; m_SpeakerSet = TRUE;}
  // the dialog line
  void SetDialogLine(char *DialogLine)   {m_DialogLine = DialogLine; m_DialogLineSet = TRUE;}
  // the voice file
  void SetVoiceFile(char *VoiceFile)     {m_VoiceFile = VoiceFile; m_VoiceFileSet = TRUE;}

  // get data functions
  // the speaker
  CString& GetSpeaker(void)              {return m_Speaker;}
  // the dialog line
  CString& GetDialogLine(void)           {return m_DialogLine;}
  // the voice file
  CString& GetVoiceFile(void)            {return m_VoiceFile;}

  // test data set functions
  // the speaker
  BOOL IsSetSpeaker(void)                {return m_SpeakerSet;}
  // the dialog line
  BOOL IsSetDialogLine(void)             {return m_DialogLineSet;}
  // the voice file
  BOOL IsSetVoiceFile(void)              {return m_VoiceFileSet;}

  // unset data functions
  // the speaker
  void UnsetSpeaker(void)                {m_SpeakerSet = FALSE;}
  // the dialog line
  void UnsetDialogLine(void)             {m_DialogLineSet = FALSE;}
  // the voice file
  void UnsetVoiceFile(void)              {m_VoiceFileSet = FALSE;}

  protected:

  // the data elements
  CString       m_Speaker;
  CString       m_DialogLine;
  CString       m_VoiceFile;

  // the data set markers
  BOOL          m_SpeakerSet;
  BOOL          m_DialogLineSet;
  BOOL          m_VoiceFileSet;
};

// ---------------  the dialog list class, contains a list of dialogs ------
class CDialogList : public CObList {};


// ---------------  the title data class, contains all info of a title ------
class CTitleData : public CObject
{
  public:
  // override constructor
  CTitleData(void);

  // set data functions
  // the title
  void SetTitle(char *Title)           {m_Title = Title; m_TitleSet = TRUE;}

  // get data functions
  // the title
  CString& GetTitle(void)              {return m_Title;}

  // the title line list
  // (no set functions needed, lists can be handled directly)
  CStrList *GetTitleLines(void)        {return &m_TitleLines;}

  // the title voice list
  // (no set functions needed, lists can be handled directly)
  CStrList *GetTitleVoices(void)       {return &m_TitleVoices;}

  // test data set functions
  // the title
  BOOL IsSetTitle(void)                {return m_TitleSet;}

  // unset data functions
  // the title
  void UnsetTitle(void)                {m_TitleSet = FALSE;}

  protected:

  // the data elements
  CString       m_Title;
  // the title line list
  CStrList      m_TitleLines;
  // the title voice list
  CStrList      m_TitleVoices;

  // the data set markers
  BOOL          m_TitleSet;
};


// ---------------  the epilog data class, contains all info of an epilog ------
class CEpilogData : public CObject
{
  public:
  // override constructor
  CEpilogData(void);

  // set data functions

  // the epliog line list
  // (no set functions needed, lists can be handled directly)
  CStrList *GetEpilogLines(void)        {return &m_EpilogLines;}

  // the title voice list
  // (no set functions needed, lists can be handled directly)
  CStrList *GetEpilogVoices(void)       {return &m_EpilogVoices;}


  protected:

  // the data elements
  // the epilog line list
  CStrList      m_EpilogLines;
  // the epilog voice list
  CStrList      m_EpilogVoices;

};


// ---------------  the general mode data class, contains all general mode info ------
class CGeneralModeData : public CObject
{
  public:
  // override constructor
  CGeneralModeData(void);

  // set data functions
  // the warp out mode
  void SetWarpOutMode(EWarpOutMode WarpOutMode)                {m_WarpOutMode = WarpOutMode; m_WarpOutModeSet = TRUE;}
  // the collision mode
  void SetCollisionMode(ECollisionMode CollisionMode)          {m_CollisionMode = CollisionMode; m_CollisionModeSet = TRUE;}
  // the ship change mode
  void SetShipChangeMode(EShipChangeMode ShipChangeMode)       {m_ShipChangeMode = ShipChangeMode; m_ShipChangeModeSet = TRUE;}
  // the perfect cloak  mode
  void SetPerfectCloakMode(EPerfectCloakMode PerfectCloakMode) {m_PerfectCloakMode = PerfectCloakMode; m_PerfectCloakModeSet = TRUE;}

  // get data functions
  // the warp out mode
  EWarpOutMode      GetWarpOutMode(void)          {return m_WarpOutMode;}
  // the collision mode
  ECollisionMode    GetCollisionMode(void)        {return m_CollisionMode;}
  // the ship change mode
  EShipChangeMode   GetShipChangeMode(void)       {return m_ShipChangeMode;}
  // the perfect cloak mode
  EPerfectCloakMode GetPerfectCloakMode(void)     {return m_PerfectCloakMode;}

  // test data set functions
  // the warp out mode
  BOOL IsSetWarpOutMode(void)                   {return m_WarpOutModeSet;}
  // the collision mode
  BOOL IsSetCollisionMode(void)                 {return m_CollisionModeSet;}
  // the ship change mode
  BOOL IsSetShipChangeMode(void)                {return m_ShipChangeModeSet;}
  // the perfect cloak mode
  BOOL IsSetPerfectCloakMode(void)              {return m_PerfectCloakModeSet;}

  // unset data functions
  // the warp out mode
  void UnsetWarpOutMode(void)                   {m_WarpOutModeSet = FALSE;}
  // the collision mode
  void UnsetCollisionMode(void)                 {m_CollisionModeSet = FALSE;}
  // the ship change mode
  void UnsetShipChangeMode(void)                {m_ShipChangeModeSet = FALSE;}
  // the perfect cloak mode
  void UnsetPerfectCloakMode(void)              {m_PerfectCloakModeSet = FALSE;}


  protected:

  // the data elements
  EWarpOutMode       m_WarpOutMode;
  ECollisionMode     m_CollisionMode;
  EShipChangeMode    m_ShipChangeMode;
  EPerfectCloakMode  m_PerfectCloakMode;

  // the data set markers
  BOOL             m_WarpOutModeSet;
  BOOL             m_CollisionModeSet;
  BOOL             m_ShipChangeModeSet;
  BOOL             m_PerfectCloakModeSet;
};


// ---------------  the jump option data class, contains all jump gate options ------
class CJumpOptionsData : public CObject
{
  public:
  // override constructor
  CJumpOptionsData(void);

  // set data functions
  // the hidden gate option
  void SetGateHidden()                      {m_GateHiddenSet = TRUE;}
  // the jump only option
  void SetJumpOnly()                        {m_JumpOnlySet = TRUE;}
  // the clear after jump option
  void SetClearJump()                       {m_ClearJumpSet = TRUE;}
  // the no spin after jump option
  void SetNoSpin()                          {m_NoSpinSet = TRUE;}
  // the show gate activation option
  void SetShowActivation()                  {m_ShowActivationSet = TRUE;}
  // the activate on hail option
  void SetActOnHail()                       {m_ActOnHailSet = TRUE;}
  // the new activation distance option
  void SetActivationDistance(long Distance) {m_ActDistance = Distance; m_ActDistanceSet = TRUE;}

  // get data functions
  // the current activation distance
  long GetActivationDistance()              {return m_ActDistance;}

  // test data set functions
  // the hidden gate option
  BOOL IsSetGateHidden()                      {return m_GateHiddenSet;}
  // the jump only option
  BOOL IsSetJumpOnly()                        {return m_JumpOnlySet;}
  // the clear after jump option
  BOOL IsSetClearJump()                       {return m_ClearJumpSet;}
  // the no spin after jump option
  BOOL IsSetNoSpin()                          {return m_NoSpinSet;}
  // the show gate activation option
  BOOL IsSetShowActivation()                  {return m_ShowActivationSet;}
  // the activate on hail option
  BOOL IsSetActOnHail()                       {return m_ActOnHailSet;}
  // the new activation distance option
  BOOL IsSetActivationDistance()              {return m_ActDistanceSet;}

  // unset data functions
  // the hidden gate option
  void UnsetGateHidden()                      {m_GateHiddenSet = FALSE;}
  // the jump only option
  void UnsetJumpOnly()                        {m_JumpOnlySet = FALSE;}
  // the clear after jump option
  void UnsetClearJump()                       {m_ClearJumpSet = FALSE;}
  // the no spin after jump option
  void UnsetNoSpin()                          {m_NoSpinSet = FALSE;}
  // the show gate activation option
  void UnsetShowActivation()                  {m_ShowActivationSet = FALSE;}
  // the activate on hail option
  void UnsetActOnHail()                       {m_ActOnHailSet = FALSE;}
  // the new activation distance option
  void UnsetActivationDistance()              {m_ActDistanceSet = FALSE;}

  protected:

  // the data elements
  long               m_ActDistance;

  // the data set markers
  BOOL               m_GateHiddenSet;
  BOOL               m_JumpOnlySet;
  BOOL               m_ClearJumpSet;
  BOOL               m_NoSpinSet;
  BOOL               m_ShowActivationSet;
  BOOL               m_ActOnHailSet;
  BOOL               m_ActDistanceSet;
};


// ---------------  the self destruct data class, contains all self destruct info ------
class CSelfDestructData : public CObject
{
  public:
  // override constructor
  CSelfDestructData(void);

  // set data functions
  // the self destruct mode
  void SetMode(ESelfDestructMode Mode)                {m_Mode = Mode; m_ModeSet = TRUE;}
  // the self destruct code
  void SetCode(const char *Code)                      {m_Code = Code;}
  // the overkill distance
  void SetOverkillDistance(unsigned long Distance)    {m_OverkillDistance = Distance;}

  // get data functions
  // the self destrcut mode
  ESelfDestructMode   GetMode(void)                   {return m_Mode;}
  // the self destruct code
  CString&            GetCode(void)                   {return m_Code;}
  // the overkill distance
  unsigned long       GetOverkillDistance(void)       {return m_OverkillDistance;}

  // test data set functions
  // the mode
  BOOL IsSetMode(void)                                {return m_ModeSet;}

  // unset data functions
  // the mode
  void UnsetMode(void)                                {m_ModeSet = FALSE;}

  protected:

  // the data elements
  ESelfDestructMode  m_Mode;
  unsigned long      m_OverkillDistance;
  CString            m_Code;

  // the data set markers
  BOOL               m_ModeSet;
};


// ---------------  the system data class, contains all info of a specific solar system ------
class CSystemData : public CObject
{

  public:
  // override constructor
  CSystemData(void);

  // set data functions
  // the name of the solar system
  void SetSystemName(const char *SystemName)       {m_SystemName = SystemName; m_SystemNameSet = TRUE;}
  // the stardate in the solar system
  void SetStardate(long Stardate)                  {m_Stardate = Stardate; m_StardateSet = TRUE;}
  // the enemy position in the solar system
  void SetEnemyPosX(long EnemyPosX)                {m_EnemyPosX = EnemyPosX; m_EnemyPosSet = TRUE;}
  void SetEnemyPosY(long EnemyPosY)                {m_EnemyPosY = EnemyPosY; m_EnemyPosSet = TRUE;}
  void SetEnemyPosZ(long EnemyPosZ)                {m_EnemyPosZ = EnemyPosZ; m_EnemyPosSet = TRUE;}
  void SetEnemyPosPlanet(char *EnemyPosPlanet)     {m_EnemyPosPlanet = EnemyPosPlanet; m_EnemyPosPlanetSet = TRUE;}
  // the friend position in the solar system
  void SetFriendPosX(long FriendPosX)              {m_FriendPosX = FriendPosX; m_FriendPosSet = TRUE;}
  void SetFriendPosY(long FriendPosY)              {m_FriendPosY = FriendPosY; m_FriendPosSet = TRUE;}
  void SetFriendPosZ(long FriendPosZ)              {m_FriendPosZ = FriendPosZ; m_FriendPosSet = TRUE;}
  void SetFriendPosPlanet(char *FriendPosPlanet)   {m_FriendPosPlanet = FriendPosPlanet; m_FriendPosPlanetSet = TRUE;}
  // the player position in the solar system
  void SetPlayerPosX(long PlayerPosX)              {m_PlayerPosX = PlayerPosX; m_PlayerPosSet = TRUE;}
  void SetPlayerPosY(long PlayerPosY)              {m_PlayerPosY = PlayerPosY; m_PlayerPosSet = TRUE;}
  void SetPlayerPosZ(long PlayerPosZ)              {m_PlayerPosZ = PlayerPosZ; m_PlayerPosSet = TRUE;}
  void SetPlayerPosPlanet(char *PlayerPosPlanet)   {m_PlayerPosPlanet = PlayerPosPlanet; m_PlayerPosPlanetSet = TRUE;}
  // the neutral position in the solar system
  void SetNeutralPosX(long NeutralPosX)            {m_NeutralPosX = NeutralPosX; m_NeutralPosSet = TRUE;}
  void SetNeutralPosY(long NeutralPosY)            {m_NeutralPosY = NeutralPosY; m_NeutralPosSet = TRUE;}
  void SetNeutralPosZ(long NeutralPosZ)            {m_NeutralPosZ = NeutralPosZ; m_NeutralPosSet = TRUE;}
  void SetNeutralPosPlanet(char *NeutralPosPlanet) {m_NeutralPosPlanet = NeutralPosPlanet; m_NeutralPosPlanetSet = TRUE;}
  // the hulk position in the solar system
  void SetHulkPosX(long HulkPosX)                  {m_HulkPosX = HulkPosX; m_HulkPosSet = TRUE;}
  void SetHulkPosY(long HulkPosY)                  {m_HulkPosY = HulkPosY; m_HulkPosSet = TRUE;}
  void SetHulkPosZ(long HulkPosZ)                  {m_HulkPosZ = HulkPosZ; m_HulkPosSet = TRUE;}
  void SetHulkPosPlanet(char *HulkPosPlanet)       {m_HulkPosPlanet = HulkPosPlanet; m_HulkPosPlanetSet = TRUE;}
  // the jump gate position in the solar system
  void SetJumpGatePosX(long JumpGatePosX)          {m_JumpGatePosX = JumpGatePosX; m_JumpGatePosSet = TRUE;}
  void SetJumpGatePosY(long JumpGatePosY)          {m_JumpGatePosY = JumpGatePosY; m_JumpGatePosSet = TRUE;}
  void SetJumpGatePosZ(long JumpGatePosZ)          {m_JumpGatePosZ = JumpGatePosZ; m_JumpGatePosSet = TRUE;}
  void SetJumpGatePosPlanet(char *JumpGatePosPlanet) {m_JumpGatePosPlanet = JumpGatePosPlanet; m_JumpGatePosPlanetSet = TRUE;}
  // the neutral behavior in the solar system
  void SetNeutralBehavior(ENeutBehavior NeutralBehavior) {m_NeutralBehavior = NeutralBehavior; m_NeutralBehaviorSet = TRUE;}
  void SetNeutralBehaviorTime(long NeutralBehaviorTime)  {m_NeutralBehaviorTime = NeutralBehaviorTime; m_NeutralBehaviorTimeSet = TRUE;}
  // the warp disables in the solar system
  void SetPlayerWarpDisable(BOOL PlayerWarpDisable)   {m_PlayerWarpDisable = PlayerWarpDisable; m_PlayerWarpDisableSet = TRUE;}
  void SetFriendWarpDisable(BOOL FriendWarpDisable)   {m_FriendWarpDisable = FriendWarpDisable; m_FriendWarpDisableSet = TRUE;}
  void SetNeutralWarpDisable(BOOL NeutralWarpDisable) {m_NeutralWarpDisable = NeutralWarpDisable; m_NeutralWarpDisableSet = TRUE;}
  void SetEnemyWarpDisable(BOOL EnemyWarpDisable)     {m_EnemyWarpDisable = EnemyWarpDisable; m_EnemyWarpDisableSet = TRUE;}
  // the battle mode in the solar system
  void SetBattleMode(EBattleMode BattleMode)          {m_BattleMode = BattleMode; m_BattleModeSet = TRUE;}
  // the warp in actions
  void SetFriendWarpInTime(long FriendWarpInTime)   {m_FriendWarpInTime = FriendWarpInTime; m_FriendWarpInTimeSet = TRUE;}
  void SetEnemyWarpInTime(long EnemyWarpInTime)     {m_EnemyWarpInTime = EnemyWarpInTime; m_EnemyWarpInTimeSet = TRUE;}
  void SetNeutralWarpInTime(long NeutralWarpInTime) {m_NeutralWarpInTime = NeutralWarpInTime; m_NeutralWarpInTimeSet = TRUE;}
  // the warp out actions
  void SetFriendWarpOutTime(long FriendWarpOutTime)   {m_FriendWarpOutTime = FriendWarpOutTime; m_FriendWarpOutTimeSet = TRUE;}
  void SetEnemyWarpOutTime(long EnemyWarpOutTime)     {m_EnemyWarpOutTime = EnemyWarpOutTime; m_EnemyWarpOutTimeSet = TRUE;}
  void SetNeutralWarpOutTime(long NeutralWarpOutTime) {m_NeutralWarpOutTime = NeutralWarpOutTime; m_NeutralWarpOutTimeSet = TRUE;}
  // the join escort actions
  void SetFriendJoinEscortTime(long FriendJoinEscortTime) {m_FriendJoinEscortTime = FriendJoinEscortTime; m_FriendJoinEscortTimeSet = TRUE;}
  // the jump time
  void SetJumpTime(long JumpTime)                     {m_JumpTime = JumpTime; m_JumpTimeSet = TRUE;}
  // the jump destination
  void SetJumpDestination(const char *Destination)    {m_JumpDestination = Destination; m_JumpDestinationSet = TRUE;}


  // get data functions
  // the name of the solar system
  CString& GetSystemName(void)         {return m_SystemName;}
  // the stardate in the solar system
  long     GetStardate(void)           {return m_Stardate;}
  // the enemy position in the solar system
  long     GetEnemyPosX(void)          {return m_EnemyPosX;}
  long     GetEnemyPosY(void)          {return m_EnemyPosY;}
  long     GetEnemyPosZ(void)          {return m_EnemyPosZ;}
  CString& GetEnemyPosPlanet(void)     {return m_EnemyPosPlanet;}
  // the friend position in the solar system
  long     GetFriendPosX(void)         {return m_FriendPosX;}
  long     GetFriendPosY(void)         {return m_FriendPosY;}
  long     GetFriendPosZ(void)         {return m_FriendPosZ;}
  CString& GetFriendPosPlanet(void)    {return m_FriendPosPlanet;}
  // the player position in the solar system
  long     GetPlayerPosX(void)         {return m_PlayerPosX;}
  long     GetPlayerPosY(void)         {return m_PlayerPosY;}
  long     GetPlayerPosZ(void)         {return m_PlayerPosZ;}
  CString& GetPlayerPosPlanet(void)    {return m_PlayerPosPlanet;}
  // the neutral position in the solar system
  long     GetNeutralPosX(void)        {return m_NeutralPosX;}
  long     GetNeutralPosY(void)        {return m_NeutralPosY;}
  long     GetNeutralPosZ(void)        {return m_NeutralPosZ;}
  CString& GetNeutralPosPlanet(void)   {return m_NeutralPosPlanet;}
  // the hulk position in the solar system
  long     GetHulkPosX(void)           {return m_HulkPosX;}
  long     GetHulkPosY(void)           {return m_HulkPosY;}
  long     GetHulkPosZ(void)           {return m_HulkPosZ;}
  CString& GetHulkPosPlanet(void)      {return m_HulkPosPlanet;}
  // the jump gate position in the solar system
  long     GetJumpGatePosX(void)       {return m_JumpGatePosX;}
  long     GetJumpGatePosY(void)       {return m_JumpGatePosY;}
  long     GetJumpGatePosZ(void)       {return m_JumpGatePosZ;}
  CString& GetJumpGatePosPlanet(void)  {return m_JumpGatePosPlanet;}
  // the neutral behavior in the solar system
  ENeutBehavior GetNeutralBehavior(void) {return m_NeutralBehavior;}
  long     GetNeutralBehaviorTime(void)  {return m_NeutralBehaviorTime;}
  // the warp disables in the solar system
  BOOL     GetPlayerWarpDisable(void)  {return m_PlayerWarpDisable;}
  BOOL     GetFriendWarpDisable(void)  {return m_FriendWarpDisable;}
  BOOL     GetNeutralWarpDisable(void) {return m_NeutralWarpDisable;}
  BOOL     GetEnemyWarpDisable(void)   {return m_EnemyWarpDisable;}
  // the battle mode in the solar system
  EBattleMode GetBattleMode(void)      {return m_BattleMode;}
  // the warp in actions
  long     GetFriendWarpInTime(void)   {return m_FriendWarpInTime;}
  long     GetEnemyWarpInTime(void)    {return m_EnemyWarpInTime;}
  long     GetNeutralWarpInTime(void)  {return m_NeutralWarpInTime;}
  // the warp out actions
  long     GetFriendWarpOutTime(void)  {return m_FriendWarpOutTime;}
  long     GetEnemyWarpOutTime(void)   {return m_EnemyWarpOutTime;}
  long     GetNeutralWarpOutTime(void) {return m_NeutralWarpOutTime;}
  // the join escort actions
  long     GetFriendJoinEscortTime(void) {return m_FriendJoinEscortTime;}
  // the jump time
  long     GetJumpTime(void)           {return m_JumpTime;}
  // the jump destination
  CString& GetJumpDestination(void)    {return m_JumpDestination;}

  // get the lists of the enemies, friends, neutrals and hulks in the solar system
  // (no set functions needed, lists can be handled directly)
  CShipList *GetEnemies(void)          {return &m_Enemies;}
  CShipList *GetFriends(void)          {return &m_Friends;}
  CShipList *GetNeutrals(void)         {return &m_Neutrals;}
  CShipList *GetHulks(void)            {return &m_Hulks;}
  // get the jump gate object
  // (no set functions needed, object can be handled directly)
  CShipData *GetJumpGate(void)         {return &m_JumpGate;}
  // get the jump gate options
  // (no set functions needed, object can be handled directly)
  CJumpOptionsData *GetJumpOptions(void)  {return &m_JumpOptions;}
  // get the lists of starting and ending dialogs
  // (no set functions needed, lists can be handled directly)
  CDialogList *GetStartingDialogs(void)  {return &m_StartDialogs;}
  CDialogList *GetEndingDialogs(void)    {return &m_EndDialogs;}
  // get the lists of action dialogs
  // (no set functions needed, lists can be handled directly)
  CDialogList *GetFriendWarpInDialogs(void)     {return &m_FriendWarpInDialogs;}
  CDialogList *GetEnemyWarpInDialogs(void)      {return &m_EnemyWarpInDialogs;}
  CDialogList *GetNeutralWarpInDialogs(void)    {return &m_NeutralWarpInDialogs;}
  CDialogList *GetFriendWarpOutDialogs(void)    {return &m_FriendWarpOutDialogs;}
  CDialogList *GetEnemyWarpOutDialogs(void)     {return &m_EnemyWarpOutDialogs;}
  CDialogList *GetNeutralWarpOutDialogs(void)   {return &m_NeutralWarpOutDialogs;}
  CDialogList *GetFriendJoinEscortDialogs(void) {return &m_FriendJoinEscortDialogs;}
  // get the lists of behavior change dialogs
  // (no set functions needed, lists can be handled directly)
  CDialogList *GetNeutralToFriendDialogs(void)  {return &m_NeutralToFriendDialogs;}
  CDialogList *GetNeutralToEnemyDialogs(void)   {return &m_NeutralToEnemyDialogs;}
  // get the lists of jump gate related dialogs
  // (no set functions needed, lists can be handled directly)
  CDialogList *GetJumpGateAppearsDialogs(void)  {return &m_JumpGateAppearsDialogs;}
  CDialogList *GetJumpDialogs(void)             {return &m_JumpDialogs;}

  // test data set functions
  // the name of the solar system
  BOOL     IsSetSystemName(void)       {return m_SystemNameSet;}
  // the stardate in the solar system
  BOOL     IsSetStardate(void)         {return m_StardateSet;}
  // the enemy position in the solar system
  BOOL     IsSetEnemyPos(void)         {return m_EnemyPosSet;}
  BOOL     IsSetEnemyPosPlanet(void)   {return m_EnemyPosPlanetSet;}
  // the friend position in the solar system
  BOOL     IsSetFriendPos(void)        {return m_FriendPosSet;}
  BOOL     IsSetFriendPosPlanet(void)  {return m_FriendPosPlanetSet;}
  // the player position in the solar system
  BOOL     IsSetPlayerPos(void)        {return m_PlayerPosSet;}
  BOOL     IsSetPlayerPosPlanet(void)  {return m_PlayerPosPlanetSet;}
  // the neutral position in the solar system
  BOOL     IsSetNeutralPos(void)       {return m_NeutralPosSet;}
  BOOL     IsSetNeutralPosPlanet(void) {return m_NeutralPosPlanetSet;}
  // the hulk position in the solar system
  BOOL     IsSetHulkPos(void)          {return m_HulkPosSet;}
  BOOL     IsSetHulkPosPlanet(void)    {return m_HulkPosPlanetSet;}
  // the jump gate position in the solar system
  BOOL     IsSetJumpGatePos(void)       {return m_JumpGatePosSet;}
  BOOL     IsSetJumpGatePosPlanet(void) {return m_JumpGatePosPlanetSet;}
  // the neutral behavior in the solar system
  BOOL     IsSetNeutralBehavior(void)     {return m_NeutralBehaviorSet;}
  BOOL     IsSetNeutralBehaviorTime(void) {return m_NeutralBehaviorTimeSet;}
  // the warp disables in the solar system
  BOOL     IsSetPlayerWarpDisable(void)  {return m_PlayerWarpDisableSet;}
  BOOL     IsSetFriendWarpDisable(void)  {return m_FriendWarpDisableSet;}
  BOOL     IsSetNeutralWarpDisable(void) {return m_NeutralWarpDisableSet;}
  BOOL     IsSetEnemyWarpDisable(void)   {return m_EnemyWarpDisableSet;}
  // the battle mode in the solar system
  BOOL     IsSetBattleMode(void)         {return m_BattleModeSet;}
  // the warp in actions
  BOOL     IsSetFriendWarpInTime(void)   {return m_FriendWarpInTimeSet;}
  BOOL     IsSetEnemyWarpInTime(void)    {return m_EnemyWarpInTimeSet;}
  BOOL     IsSetNeutralWarpInTime(void)  {return m_NeutralWarpInTimeSet;}
  // the warp out actions
  BOOL     IsSetFriendWarpOutTime(void)  {return m_FriendWarpOutTimeSet;}
  BOOL     IsSetEnemyWarpOutTime(void)   {return m_EnemyWarpOutTimeSet;}
  BOOL     IsSetNeutralWarpOutTime(void) {return m_NeutralWarpOutTimeSet;}
  // the join escort actions
  BOOL     IsSetFriendJoinEscortTime(void) {return m_FriendJoinEscortTimeSet;}
  // the jump time
  BOOL     IsSetJumpTime(void)           {return m_JumpTimeSet;}
  // the jump destination
  BOOL     IsSetJumpDestination(void)    {return m_JumpDestinationSet;}

  // unset data functions
  // the name of the solar system
  void     UnsetSystemName(void)       {m_SystemNameSet = FALSE;}
  // the stardate in the solar system
  void     UnsetStardate(void)         {m_StardateSet = FALSE;}
  // the enemy position in the solar system
  void     UnsetEnemyPos(void)         {m_EnemyPosSet = FALSE;}
  void     UnsetEnemyPosPlanet(void)   {m_EnemyPosPlanetSet = FALSE;}
  // the friend position in the solar system
  void     UnsetFriendPos(void)        {m_FriendPosSet = FALSE;}
  void     UnsetFriendPosPlanet(void)  {m_FriendPosPlanetSet = FALSE;}
  // the player position in the solar system
  void     UnsetPlayerPos(void)        {m_PlayerPosSet = FALSE;}
  void     UnsetPlayerPosPlanet(void)  {m_PlayerPosPlanetSet = FALSE;}
  // the neutral position in the solar system
  void     UnsetNeutralPos(void)       {m_NeutralPosSet = FALSE;}
  void     UnsetNeutralPosPlanet(void) {m_NeutralPosPlanetSet = FALSE;}
  // the hulk position in the solar system
  void     UnsetHulkPos(void)          {m_HulkPosSet = FALSE;}
  void     UnsetHulkPosPlanet(void)    {m_HulkPosPlanetSet = FALSE;}
  // the jump gate position in the solar system
  void     UnsetJumpGatePos(void)       {m_JumpGatePosSet = FALSE;}
  void     UnsetJumpGatePosPlanet(void) {m_JumpGatePosPlanetSet = FALSE;}
  // the neutral behavior in the solar system
  void     UnsetNeutralBehavior(void)     {m_NeutralBehaviorSet = FALSE;}
  void     UnsetNeutralBehaviorTime(void) {m_NeutralBehaviorTimeSet = FALSE;}
  // the warp disables in the solar system
  void     UnsetPlayerWarpDisable(void)  {m_PlayerWarpDisableSet = FALSE;}
  void     UnsetFriendWarpDisable(void)  {m_FriendWarpDisableSet = FALSE;}
  void     UnsetNeutralWarpDisable(void) {m_NeutralWarpDisableSet = FALSE;}
  void     UnsetEnemyWarpDisable(void)   {m_EnemyWarpDisableSet = FALSE;}
  // the battle mode in the solar system
  void     UnsetBattleMode(void)         {m_BattleModeSet = FALSE;}
  // the warp in actions
  void     UnsetFriendWarpInTime(void)   {m_FriendWarpInTimeSet = FALSE;}
  void     UnsetEnemyWarpInTime(void)    {m_EnemyWarpInTimeSet = FALSE;}
  void     UnsetNeutralWarpInTime(void)  {m_NeutralWarpInTimeSet = FALSE;}
  // the warp out actions
  void     UnsetFriendWarpOutTime(void)  {m_FriendWarpOutTimeSet = FALSE;}
  void     UnsetEnemyWarpOutTime(void)   {m_EnemyWarpOutTimeSet = FALSE;}
  void     UnsetNeutralWarpOutTime(void) {m_NeutralWarpOutTimeSet = FALSE;}
  // the join escort actions
  void     UnsetFriendJoinEscortTime(void) {m_FriendJoinEscortTimeSet = FALSE;}
  // the jump time
  void     UnsetJumpTime(void)           {m_JumpTimeSet = FALSE;}
  // the jump destination
  void     UnsetJumpDestination(void)    {m_JumpDestinationSet = FALSE;}

  protected:

  // the data elements
  CString       m_SystemName;
  long          m_Stardate;
  long          m_EnemyPosX;
  long          m_EnemyPosY;
  long          m_EnemyPosZ;
  CString       m_EnemyPosPlanet;
  long          m_FriendPosX;
  long          m_FriendPosY;
  long          m_FriendPosZ;
  CString       m_FriendPosPlanet;
  long          m_PlayerPosX;
  long          m_PlayerPosY;
  long          m_PlayerPosZ;
  CString       m_PlayerPosPlanet;
  long          m_NeutralPosX;
  long          m_NeutralPosY;
  long          m_NeutralPosZ;
  CString       m_NeutralPosPlanet;
  long          m_HulkPosX;
  long          m_HulkPosY;
  long          m_HulkPosZ;
  CString       m_HulkPosPlanet;
  long          m_JumpGatePosX;
  long          m_JumpGatePosY;
  long          m_JumpGatePosZ;
  CString       m_JumpGatePosPlanet;
  ENeutBehavior m_NeutralBehavior;
  long          m_NeutralBehaviorTime;
  BOOL          m_PlayerWarpDisable;
  BOOL          m_FriendWarpDisable;
  BOOL          m_EnemyWarpDisable;
  BOOL          m_NeutralWarpDisable;
  EBattleMode   m_BattleMode;
  long          m_FriendWarpInTime;
  long          m_EnemyWarpInTime;
  long          m_NeutralWarpInTime;
  long          m_FriendWarpOutTime;
  long          m_EnemyWarpOutTime;
  long          m_NeutralWarpOutTime;
  long          m_FriendJoinEscortTime;
  long          m_JumpTime;
  CString       m_JumpDestination;
  CJumpOptionsData m_JumpOptions;
  CShipData     m_JumpGate;
  // lists
  CShipList     m_Enemies;
  CShipList     m_Friends;
  CShipList     m_Neutrals;
  CShipList     m_Hulks;
  CDialogList   m_StartDialogs;
  CDialogList   m_EndDialogs;
  CDialogList   m_FriendWarpInDialogs;
  CDialogList   m_EnemyWarpInDialogs;
  CDialogList   m_NeutralWarpInDialogs;
  CDialogList   m_FriendWarpOutDialogs;
  CDialogList   m_EnemyWarpOutDialogs;
  CDialogList   m_NeutralWarpOutDialogs;
  CDialogList   m_FriendJoinEscortDialogs;
  CDialogList   m_NeutralToFriendDialogs;
  CDialogList   m_NeutralToEnemyDialogs;
  CDialogList   m_JumpGateAppearsDialogs;
  CDialogList   m_JumpDialogs;

  // the data set markers
  BOOL          m_SystemNameSet;
  BOOL          m_StardateSet;
  BOOL          m_JumpTimeSet;
  BOOL          m_JumpDestinationSet;
  BOOL          m_EnemyPosSet;
  BOOL          m_EnemyPosPlanetSet;
  BOOL          m_FriendPosSet;
  BOOL          m_FriendPosPlanetSet;
  BOOL          m_PlayerPosSet;
  BOOL          m_PlayerPosPlanetSet;
  BOOL          m_NeutralPosSet;
  BOOL          m_NeutralPosPlanetSet;
  BOOL          m_HulkPosSet;
  BOOL          m_HulkPosPlanetSet;
  BOOL          m_JumpGatePosSet;
  BOOL          m_JumpGatePosPlanetSet;
  BOOL          m_NeutralBehaviorSet;
  BOOL          m_NeutralBehaviorTimeSet;
  BOOL          m_PlayerWarpDisableSet;
  BOOL          m_FriendWarpDisableSet;
  BOOL          m_EnemyWarpDisableSet;
  BOOL          m_NeutralWarpDisableSet;
  BOOL          m_BattleModeSet;
  BOOL          m_FriendWarpInTimeSet;
  BOOL          m_EnemyWarpInTimeSet;
  BOOL          m_NeutralWarpInTimeSet;
  BOOL          m_FriendWarpOutTimeSet;
  BOOL          m_EnemyWarpOutTimeSet;
  BOOL          m_NeutralWarpOutTimeSet;
  BOOL          m_FriendJoinEscortTimeSet;
};


// ---------------  the system list class, contains all info of all solar systems ------
class CSystemList : public CObList {
  public:

  // look if a system exists
  BOOL ExistsSystem(const char *strSystemName);

};

// ---------------  the main configuration class --------------------------------------
class CConfiguration
{

  public:
  // the constructor
  CConfiguration();

  // the destructor
  ~CConfiguration();

  // setup a new configuration (only needed when not parsed)
  BOOL Setup();

  // parse a configuration
  BOOL Parse(const char *ConfigName, const char *Path, BOOL CheckOnly);

  // write a configuration
  BOOL Write(char *ConfigName);

  // set the configuration name
  void SetConfigName(const char *ConfigName) {strConfigName = ConfigName;}  

  // get the configuration
  // get the playership
  CShipData *GetPlayership() {return &PlayerShip;}
  // get the escort
  CShipList *GetEscort()     {return &Escort;}
  // get the systems
  CSystemList *GetSystems()    {return &Systems;}
  // get the general mode
  CGeneralModeData *GetGeneralMode() {return &GeneralMode;}
  // get the self destruct mode
  CSelfDestructData *GetSelfDestruct() {return &SelfDestruct;}
  // get the title
  CTitleData *GetTitle()     {return &Title;}
  // get the epilog
  CEpilogData *GetEpilog()   {return &Epilog;}
  // get the remarks
  CStrList *GetRemarks()   {return &Remarks;}
  // get the configuration name
  CString   *GetConfigName() {return &strConfigName;}  

  // check if a shipname is unique
  BOOL CheckShipName(const char *strShipName, BOOL bCheckDefault, BOOL bReportError, BOOL bShowLine,
                     unsigned *uWhere, unsigned *uGroup);

  // check all user given ship names against all default ship names
  BOOL CheckAllGivenAgainstDefaultShipNames(unsigned *uWhereGiven, unsigned *uGroupGiven,
                                            unsigned *uWhereDefault, unsigned *uGroupDefault,
                                            CString *strShipName);
  // check some critical parameters for stat battle mode
  BOOL CheckStatBattleMode(CSystemData *pSystem, BOOL bReportError);

  // check jump destination system
  BOOL CheckJumpDestination(CString DestSystemName);

  // check some other critical parameters for a system
  BOOL CheckOthers(CSystemData *pSystem);

  // a final configuration check
  BOOL FinalConfigCheck(BOOL ExtendedErrorMessage);

  protected:

  // internal methods

  // special string handling functions needed for parsing a config file
  // remove remarks
  void RemoveRemarks(char *dest, char *src);
  // sample remarks
  BOOL SampleRemarks(char *src);

  // internal functions needed for parsing
  // report a configuration error
  void ReportConfigError(const char *ErrorMessage, BOOL printLine);
  // do some basic parameter test
  BOOL CheckParam(char *strParam);

  // the functions to analyze the configuration
  // define a player
  BOOL DefinePlayer(char *strParam);
  // define an escort
  BOOL DefineEscort(char *strParam);
  // define a friend
  BOOL DefineFriend(char *strParam);
  // define an enemy
  BOOL DefineEnemy(char *strParam);
  // define a neutral
  BOOL DefineNeutral(char *strParam);
  // define a hulk
  BOOL DefineHulk(char *strParam);
  // define a jump gate
  BOOL DefineJumpGate(char *strParam);
  // define a ship
  BOOL DefineShip(char *strParam, unsigned iWho);
  // define a system
  BOOL DefineSystem(char *strParam);
  // define player position
  BOOL DefinePlayerPosition(char *strParam);
  // define friend position
  BOOL DefineFriendPosition(char *strParam);
  // define enemy position
  BOOL DefineEnemyPosition(char *strParam);
  // define hulk position
  BOOL DefineHulkPosition(char *strParam);
  // define neutral position
  BOOL DefineNeutralPosition(char *strParam);
  // define jump gate position
  BOOL DefineJumpGatePosition(char *strParam);
  // define a position
  BOOL DefinePosition(char *strParam, unsigned iWho);
  // define a stardate for the current system
  BOOL DefineStardate(char *strParam);
  // define the jump time for a solar system
  BOOL DefineJumpTime(char *strParam);
  // define the jump destination for a solar system
  BOOL DefineJumpDestination(char *strParam);
  // define the jump options for a solar system
  BOOL DefineJumpOptions(char *strParam);
  // define the battle mode in a solar system
  BOOL DefineBattleMode(char *strParam);
  // define the posibility of a ship change
  BOOL DefineShipChange(char *strParam);
  // define when the player is allowed to leave the system
  BOOL DefineWarpOut(char *strParam);
  // define collision checking for the playership and the rest
  BOOL DefineCollision(char *strParam);
  // define the self destruct options for the playership
  BOOL DefineSelfDestruct(char *strParam);
  // define perfect cloak for the playership
  BOOL DefinePerfectCloak(char *strParam);
  // define a battle title
  BOOL DefineTitle(char *strParam);
  // define some title lines for a battle title
  BOOL DefineTitleLine(char *strParam);
  // define some epilog lines for a battle epilog
  BOOL DefineEpilogLine(char *strParam);
  // define some title or epilog lines 
  BOOL DefineTitleEpilogLine(char *strParam, unsigned iWhat);
  // define a starting dialog for a system
  BOOL DefineStartDialog(char *strParam);
  // define an ending dialog for a system
  BOOL DefineEndDialog(char *strParam);
  // define a friend warp in dialog for a system
  BOOL DefineFriendWarpInDialog(char *strParam);
  // define an enemy warp in dialog for a system
  BOOL DefineEnemyWarpInDialog(char *strParam);
  // define a neutral warp in dialog for a system
  BOOL DefineNeutralWarpInDialog(char *strParam);
  // define a friend warp out dialog for a system
  BOOL DefineFriendWarpOutDialog(char *strParam);
  // define an enemy warp out dialog for a system
  BOOL DefineEnemyWarpOutDialog(char *strParam);
  // define a neutral warp out dialog for a system
  BOOL DefineNeutralWarpOutDialog(char *strParam);
  // define a friend join escort dialog for a system
  BOOL DefineFriendJoinEscortDialog(char *strParam);
  // define a neutral to friend dialog for a system
  BOOL DefineNeutralToFriendDialog(char *strParam);
  // define a neutral to enemy dialog for a system
  BOOL DefineNeutralToEnemyDialog(char *strParam);
  // define a jump gate appears dialog for a system
  BOOL DefineJumpGateAppearsDialog(char *strParam);
  // define a jump dialog for a system
  BOOL DefineJumpDialog(char *strParam);
  // define a dialog for a system
  BOOL DefineDialog(char *strParam, unsigned iWhat);
  // define the behavior of the neutrals in a system
  BOOL DefineNeutralBehavior(char *strParam);
  // disable warp for some groups
  BOOL DefineWarpDisable(char *strParam);
  // define an action for friends
  BOOL DefineFriendAction(char *strParam);
  // define an action for enemies
  BOOL DefineEnemyAction(char *strParam);
  // define an action for neutrals
  BOOL DefineNeutralAction(char *strParam);
  // define an action for some groups
  BOOL DefineAction(char *strParam, unsigned iWho);

  // internal functions needed for writing
  // writes a line to the config file, limits line length
  BOOL WriteCfgLine(const char *strLine);
  // sets up an output line containing a ship class and name
  void OutputShipClassAndName(CString *line, CShipData *ship);
  // write the header of the config file
  BOOL WriteHeader();
  // write the title
  BOOL WriteTitleAndTitleLines();
  // write the epilog
  BOOL WriteEpilogLines();
  // write the player and his escort
  BOOL WritePlayerAndEscort();
  // write general parameters
  BOOL WriteGeneral();
  // write self destruct
  BOOL WriteSelfDestruct();
  // write the system name
  BOOL WriteSystemName(CSystemData *pSystem);
  // write the friends in the given system
  BOOL WriteFriends(CSystemData *pSystem);
  // write the enemies in the given system
  BOOL WriteEnemies(CSystemData *pSystem);
  // write the neutrals in the given system
  BOOL WriteNeutrals(CSystemData *pSystem);
  // write the hulks in the given system
  BOOL WriteHulks(CSystemData *pSystem);
  // write the jump gate in the given system
  BOOL WriteJumpGate(CSystemData *pSystem);
  // write the jump time in the given system
  BOOL WriteJumpTime(CSystemData *pSystem);
  // write the jump destination in the given system
  BOOL WriteJumpDestination(CSystemData *pSystem);
  // write the jump options in the given system
  BOOL WriteJumpOptions(CSystemData *pSystem);
  // write the positions in the given system
  BOOL WritePositions(CSystemData *pSystem);
  // write the dialogs in the given system
  BOOL WriteDialogs(CSystemData *pSystem);
  // write the warp disables in the given system
  BOOL WriteWarpDisable(CSystemData *pSystem);
  // write the stardate in the given system
  BOOL WriteStardate(CSystemData *pSystem);
  // write the neutral behavior in the given system
  BOOL WriteNeutralBehavior(CSystemData *pSystem);
  // write the battle mode in the given system
  BOOL WriteBattleMode(CSystemData *pSystem);
  // write the actions in the given system
  BOOL WriteActions(CSystemData *pSystem);


  // the configuration data
  
  // the initial remarks
  CStrList          Remarks;
  // the title
  CTitleData        Title;
  // the epilog
  CEpilogData       Epilog;
  // the general mode
  CGeneralModeData  GeneralMode;
  // the self destruct data
  CSelfDestructData SelfDestruct;
  // the player ship
  CShipData         PlayerShip;
  // the escort
  CShipList         Escort;
  // the systems and all related data
  CSystemList       Systems;

  // general data

  // the configuration file
  CCfgFile          ConfigFile;
  // the name of the configuration
  CString           strConfigName;
  // pointer to the current solar system
  CSystemData       *pCurrentSystem;
  // the currently parsed line
  unsigned          uParsedLine;
  // the sample remarks flag
  BOOL              bSampleRemarks;
};


// ---------------  the available ships class --------------------------------------
class CAvailableShips
{

  // the available ships file
  CCfgFile         AvailableShipFile;
  // the list of avaliable ships
  CShipList        AvailableShipList;

  public:
  // the constructor
  CAvailableShips();

  // the destructor
  ~CAvailableShips();

  // get the list of available ships
  BOOL Get();

  // fill a listbox with the ships
  void FillListBox(CModalDialog *pDialog, LONG ListBoxId);

  // find a ship class
  BOOL FindShipClass(LONG Index, CString *Class);

  // find a ship
  CShipData *FindShip(CString Class);
};


// ---------------  the system planet data class, contains system name and planet names for a specific system ----
class CSystemPlanetData : public CObject
{
  CString     m_SystemName;
  CStrList    m_Planets;

  public:
  // override constructor
  CSystemPlanetData(void);

  // set the system name
  void SetSystemName(const char *SystemName) {m_SystemName = SystemName;}

  // get the system name
  CString &GetSystemName(void)               {return m_SystemName;}

  // get the planet list
  CStrList *GetPlanets(void)                 {return &m_Planets;}
};

// ---------------  the system planet list class, contains system name and planet names for some systems ----
class CSystemPlanetList : public CObList {
  public:

  // find a system
  CSystemPlanetData *FindSystem(const char *strSystemName);
};

// ---------------  the available systems and planets class -----------------------------------
class CAvailableSystemsAndPlanets
{

  // the available system and planets file
  CCfgFile            AvailableSystemsPlanetsFile;
  // the list of avaliable systems and planets
  CSystemPlanetList   AvailableSystemsPlanetsList;

  public:
  // the constructor
  CAvailableSystemsAndPlanets();

  // the destructor
  ~CAvailableSystemsAndPlanets();

  // get the list of available systems and planets
  BOOL Get();

  // fill a listbox with the systems
  void FillListBoxWithSystems(CModalDialog *pDialog, LONG ListBoxId);

  // fill a listbox with the planet names of a given system
  void FillListBoxWithSystemPlanets(CSystemData *pSystem, CModalDialog *pDialog, LONG ListBoxId);

  // find a system name
  BOOL FindSystemName(LONG Index, CString *SystemName);

  // find a system
  CSystemPlanetData *FindSystem(CString SystemName);

  // find a planet
  BOOL FindPlanet(CSystemPlanetData *pSystemPlanet, CString PlanetName);

};

#endif
