// ---------------------------------------------------------------------------------
// Quick Battle 2 editor
//
// by Uwe Brinkschulte
//
// Main Header File
// ----------------------------------------------------------------------------------

#ifndef __QB2EDIT_H
#define __QB2EDIT_H

// ------------------------------ defines -------------------------------------------

// the windows nt hlocalmachine registry key for qb2edit
#define QB2EDIT_REGKEY "SOFTWARE\\QB2Edit\\KAPath\\"

// --------------------------- simple types -----------------------------------------

// the results of a dialog
typedef enum EDialogResultType   {CancelDialog, FinishDialog, NextDialog, PreviousDialog};

// position definition
typedef enum EPosDefType         {NoPositionDefined, PlanetPositionDefined, CoordPositionDefined};

// ------------------------------  function prototypes ---------------------------------

void FormatShipClassAndName(CString *out, CShipData *ship);
void AnotherFormatShipClassAndName(CString *out, CShipData *ship);
void FormatSystemName(CString *out, CSystemData *system);
void FormatPosition(CString *out, EPosDefType *posdefined, CSystemData *system, unsigned iWho);
BOOL FormatDialog(char *str, char *strSpeaker, char *strVoice, BOOL *bTruncate, BOOL *bDefaultSpeaker,
                  BOOL *bEmptyLine, BOOL *bFisRemoved);
BOOL FormatTitleEpilogLine(char *str, char *strVoice, BOOL *bTruncate, BOOL *bEmptyLine, BOOL *bFisRemoved);
BOOL AskForShipName(CShipData *ship, CConfiguration *pConfiguration, char *grouptext);
BOOL TestSelfDestruct(char *strDestructCode);
BOOL FinishTest(CConfiguration *pConfiguration);
BOOL ExactFindInListBox(CListBox *Listbox, const char *str, LONG *iItemFound);
void DrawBitmap(CPaintDC *dc, LONG BitmapID, int x, int y);

// --------------------------- classes ----------------------------------------------

// the application class
class CTheApp : public CWinApp
{
  public:
    // override InitInstance
    BOOL InitInstance();
};


// the main window class
class CMainWindow : public CFrameWnd
{
  public:
    // override Constructor
    CMainWindow();

    // handler function on painting
    afx_msg void OnPaint();

    // handler function on about dialog
    afx_msg void OnAbout();

    // handler function on start
    afx_msg void OnStart();

    // handler function on configure
    afx_msg void OnConfigure();

    // handler function on exit
    afx_msg void OnExit();

    // macro to declare a message map
    DECLARE_MESSAGE_MAP()
};


// the start dialog class
class CStartDialog : public CModalDialog
{
  CCfgFile ConfigDescFile;
  CCfgFile ConfigFile;
  BOOL     bCfgDescFileChanged;
  long     lastSpinPosition;

  public:
  // override Constructor
  CStartDialog(CWnd* pParentWnd = NULL); 
  
  // overriding OnInitDialog function
  virtual BOOL OnInitDialog();

  // new UpBattle method
  virtual void UpBattle();

  // new DownBattle method
  virtual void DownBattle();

  // overriding OnCancel function
  virtual void OnCancel();

  // new OnCreateButton method
  virtual void OnCreateButton();

  // new OnEditButton method
  virtual void OnEditButton();

  // new OnDeleteButton method
  virtual void OnDeleteButton();

  // new OnRenameButton method
  virtual void OnRenameButton();

  // new OnCheckButton method
  virtual void OnCheckButton();

  // new OnImportButton method
  virtual void OnImportButton();

  // new OnUpDownButton method
  virtual void OnUpDownButton();

  // new OnSelChange method
  virtual void OnSelChange();

  // new SaveAndCloseDescFile method
  void SaveAndCloseDescFile();

  // macro to declare a message map
  DECLARE_MESSAGE_MAP()
};  



// the general dialog class
class CGeneralDialog : public CModalDialog
{
  CConfiguration    *pConfiguration;
  EDialogResultType eDialogResult;

  public:
  // override Constructor
  CGeneralDialog(CWnd* pParentWnd = NULL); 
  
  // overriding OnInitDialog function
  virtual BOOL OnInitDialog();

  // overriding OnCancel function
  virtual void OnCancel();

  // new OnFinishButton method
  virtual void OnFinishButton();

  // new OnNextButton method
  virtual void OnNextButton();

  // new OnSelectButton method
  virtual void OnSelectButton();

  // new OnAddButton method
  virtual void OnAddButton();

  // new OnRemoveButton method
  virtual void OnRemoveButton();

  // new OnOptionButton method
  virtual void OnOptionButton();

  // new OnSelGavChange method
  virtual void OnGavSelChange();

  // new OnEscortSelChange method
  virtual void OnEscortSelChange();

  // new OnEscortDoubleClick method
  virtual void OnEscortDoubleClick();

  // new OnPlayerDoubleClick method
  virtual void OnPlayerDoubleClick();

  // setup data to edit
  void Setup(CConfiguration *Configuration);

  // do the dialog
  EDialogResultType DoDialog();

  // macro to declare a message map
  DECLARE_MESSAGE_MAP()
};  




// the system dialog class
class CSystemDialog : public CModalDialog
{
  CConfiguration    *pConfiguration;
  unsigned          uSystemToEdit;
  EDialogResultType eDialogResult;
  long              lastSpinPosition;

  public:
  // override Constructor
  CSystemDialog(CWnd* pParentWnd = NULL); 
  
  // overriding OnInitDialog function
  virtual BOOL OnInitDialog();

  // new UpSystem method
  virtual void UpSystem();

  // new DownSystem method
  virtual void DownSystem();

  // overriding OnCancel function
  virtual void OnCancel();

  // new OnFinishButton method
  virtual void OnFinishButton();

  // new OnPreviousButton method
  virtual void OnPreviousButton();

  // new OnAddButton method
  virtual void OnAddButton();

  // new OnReplaceButton method
  virtual void OnReplaceButton();

  // new OnRemoveButton method
  virtual void OnRemoveButton();

  // new OnEditButton method
  virtual void OnEditButton();

  // new OnUpDownButton method
  virtual void OnUpDownButton();

  // new OnCheckButton method
  virtual void OnCheckButton();
 
  // new OnSavSelChange method
  virtual void OnSavSelChange();

  // new OnSystemSelChange method
  virtual void OnSystemSelChange();

  // setup data to edit
  void Setup(CConfiguration *Configuration);

  // do the dialog
  EDialogResultType DoDialog(unsigned *uSystem);

  // macro to declare a message map
  DECLARE_MESSAGE_MAP()
};  


// the system friend enemy dialog class
class CSystemFriendEnemyDialog : public CModalDialog
{
  CConfiguration    *pConfiguration;
  CSystemData       *pSystem;
  unsigned          uSystemNum;
  EDialogResultType eDialogResult;

  public:
  // override Constructor
  CSystemFriendEnemyDialog(CWnd* pParentWnd = NULL); 
  
  // overriding OnInitDialog function
  virtual BOOL OnInitDialog();

  // overriding OnCancel function
  virtual void OnCancel();

  // new OnFinishButton method
  virtual void OnFinishButton();

  // new OnNextButton method
  virtual void OnNextButton();

  // new OnPreviousButton method
  virtual void OnPreviousButton();

  // new OnAddFriendButton method
  virtual void OnAddFriendButton();

  // new OnRemoveFriendButton method
  virtual void OnRemoveFriendButton();

  // new OnAddEnemyButton method
  virtual void OnAddEnemyButton();

  // new OnRemoveBEnemyutton method
  virtual void OnRemoveEnemyButton();

  // new OnPositionButton method
  virtual void OnPositionButton();

  // new OnOptionButton method
  virtual void OnOptionButton();

  // new OnActionButton method
  virtual void OnActionButton();

  // new OnS1AvSelChange method
  virtual void OnS1AvSelChange();

  // new OnFriendsSelChange method
  virtual void OnFriendsSelChange();

  // new OnEnemiesSelChange method
  virtual void OnEnemiesSelChange();

  // new OnFriendsDoubleClick method
  virtual void OnFriendsDoubleClick();

  // new OnEnemiesDoubleClick method
  virtual void OnEnemiesDoubleClick();

  // setup data to edit
  void Setup(CConfiguration *Configuration, unsigned SystemNum);

  // do the dialog
  EDialogResultType DoDialog();

  // macro to declare a message map
  DECLARE_MESSAGE_MAP()
};  


// the system neutral hulk dialog class
class CSystemNeutralHulkDialog : public CModalDialog
{
  CConfiguration    *pConfiguration;
  CSystemData       *pSystem;
  unsigned          uSystemNum;
  EDialogResultType eDialogResult;

  public:
  // override Constructor
  CSystemNeutralHulkDialog(CWnd* pParentWnd = NULL); 
  
  // overriding OnInitDialog function
  virtual BOOL OnInitDialog();

  // overriding OnCancel function
  virtual void OnCancel();

  // new OnFinishButton method
  virtual void OnFinishButton();

  // new OnNextButton method
  virtual void OnNextButton();

  // new OnPreviousButton method
  virtual void OnPreviousButton();

  // new OnAddNeutralButton method
  virtual void OnAddNeutralButton();

  // new OnRemoveNeutralButton method
  virtual void OnRemoveNeutralButton();

  // new OnAddHulkButton method
  virtual void OnAddHulkButton();

  // new OnRemoveHulkButton method
  virtual void OnRemoveHulkButton();

  // new OnPositionButton method
  virtual void OnPositionButton();

  // new OnBehaviorButton method
  virtual void OnBehaviorButton();

  // new OnS2AvSelChange method
  virtual void OnS2AvSelChange();

  // new OnNeutralsSelChange method
  virtual void OnNeutralsSelChange();

  // new OnHulksSelChange method
  virtual void OnHulksSelChange();

  // new OnNeutralsDoubleClick method
  virtual void OnNeutralsDoubleClick();

  // new OnHulksDoubleClick method
  virtual void OnHulksDoubleClick();

  // setup data to edit
  void Setup(CConfiguration *Configuration, unsigned SystemNum);

  // do the dialog
  EDialogResultType DoDialog();

  // macro to declare a message map
  DECLARE_MESSAGE_MAP()
};  


// the system jump gate dialog class
class CSystemJumpGateDialog : public CModalDialog
{
  CConfiguration    *pConfiguration;
  CSystemData       *pSystem;
  unsigned          uSystemNum;
  EDialogResultType eDialogResult;

  public:
  // override Constructor
  CSystemJumpGateDialog(CWnd* pParentWnd = NULL); 
  
  // overriding OnInitDialog function
  virtual BOOL OnInitDialog();

  // overriding OnCancel function
  virtual void OnCancel();

  // new OnFinishButton method
  virtual void OnFinishButton();

  // new OnPreviousButton method
  virtual void OnPreviousButton();

  // new OnSelectButton method
  virtual void OnSelectButton();

  // new OnRemoveButton method
  virtual void OnRemoveButton();

  // new OnPositionButton method
  virtual void OnPositionButton();

  // new OnOptionButton method
  virtual void OnOptionButton();

  // new OnDestinationButton method
  virtual void OnDestinationButton();

  // new OnS3AvSelChange method
  virtual void OnS3AvSelChange();

  // new OnJumpGateSelChange method
  virtual void OnJumpGateSelChange();

  // new OnJumpGateDoubleClick method
  virtual void OnJumpGateDoubleClick();

  // setup data to edit
  void Setup(CConfiguration *Configuration, unsigned SystemNum);

  // display jump gate state
  void ShowState();

  // do the dialog
  EDialogResultType DoDialog();

  // macro to declare a message map
  DECLARE_MESSAGE_MAP()
};  


// the position dialog class
class CPositionDialog : public CModalDialog
{
  CConfiguration    *pConfiguration;
  CSystemData       *pSystem;
  unsigned          uSystemNum;
  EPosDefType       ePlayerPosDef, eFriendPosDef, eEnemyPosDef, eNeutralPosDef, eHulkPosDef, eJumpPosDef;

  public:
  // override Constructor
  CPositionDialog(CWnd* pParentWnd = NULL); 
  
  // overriding OnInitDialog function
  virtual BOOL OnInitDialog();

  // overriding OnOk function
  virtual void OnOK();

  // new OnPlayerPlanetButton method
  virtual void OnPlayerPlanetButton();

  // new OnFriendsPlanetButton method
  virtual void OnFriendsPlanetButton();

  // new OnEnemiesPlanetButton method
  virtual void OnEnemiesPlanetButton();

  // new OnNeutralsPlanetButton method
  virtual void OnNeutralsPlanetButton();

  // new OnHulksPlanetButton method
  virtual void OnHulksPlanetButton();

  // new OnJumpGatePlanetButton method
  virtual void OnJumpGatePlanetButton();

  // new OnPlayerCoordButton method
  virtual void OnPlayerCoordButton();

  // new OnFriendsCoordButton method
  virtual void OnFriendsCoordButton();

  // new OnEnemiesCoordButton method
  virtual void OnEnemiesCoordButton();

  // new OnNeutralsCoordButton method
  virtual void OnNeutralsCoordButton();

  // new OnHulksCoordButton method
  virtual void OnHulksCoordButton();

  // new OnJumpGateCoordButton method
  virtual void OnJumpGateCoordButton();

  // new OnPlayerClearButton method
  virtual void OnPlayerClearButton();

  // new OnFriendsClearButton method
  virtual void OnFriendsClearButton();

  // new OnEnemiesClearButton method
  virtual void OnEnemiesClearButton();

  // new OnNeutralsClearButton method
  virtual void OnNeutralsClearButton();

  // new OnHulksClearButton method
  virtual void OnHulksClearButton();

  // new OnJumpGateClearButton method
  virtual void OnJumpGateClearButton();

  // setup data to edit
  void Setup(CConfiguration *Configuration, unsigned SystemNum);

  // do the dialog
  void DoDialog();

  // macro to declare a message map
  DECLARE_MESSAGE_MAP()
};  


// the position coordinates dialog class
class CPosCoordDialog : public CModalDialog
{
  CString strXPos, strYPos, strZPos;
  BOOL    bPosSet;

  public:
  // override Constructor
  CPosCoordDialog(CWnd* pParentWnd = NULL); 
  
  // overriding OnInitDialog function
  virtual BOOL OnInitDialog();

  // overriding OnOk function
  virtual void OnOK();

  // setup data to edit
  void Setup(const char *strX, const char *strY, const char *strZ);

  // do the dialog
  BOOL DoDialog(CString *strX, CString *strY, CString *strZ);
};  



// the general options dialog class
class CGeneralOptionsDialog : public CModalDialog
{
  CConfiguration    *pConfiguration;

  public:
  // override Constructor
  CGeneralOptionsDialog(CWnd* pParentWnd = NULL); 
  
  // overriding OnInitDialog function
  virtual BOOL OnInitDialog();

  // overriding OnOk function
  virtual void OnOK();

  // new OnSetupSelfDestructButton function
  virtual void OnSetupSelfDestructButton();

  // setup data to edit
  void Setup(CConfiguration *Configuration);

  // do the dialog
  void DoDialog();

  // macro to declare a message map
  DECLARE_MESSAGE_MAP()
};  



// the self destruct dialog class
class CSelfDestructDialog : public CModalDialog
{
  CConfiguration    *pConfiguration;

  public:
  // overriding Constructor
  CSelfDestructDialog(CWnd* pParentWnd = NULL);

  // overriding OnInitDialog function
  virtual BOOL OnInitDialog();

  // overriding OnOk function
  virtual void OnOK();

  // new OnNoneButton function
  virtual void OnNoneButton();

  // new OnStandardButton function
  virtual void OnStandardButton();

  // new OnOverkillButton function
  virtual void OnOverkillButton();

  // new OnSelectiveOverkillButton function
  virtual void OnSelectiveOverkillButton();

  // setup data to edit
  void Setup(CConfiguration *Configuration);

  // do the dialog
  void DoDialog();

  // macro to declare a message map
  DECLARE_MESSAGE_MAP()
};  


// the system options dialog class
class CSystemOptionsDialog : public CModalDialog
{
  CConfiguration    *pConfiguration;
  CSystemData       *pSystem;
  unsigned          uSystemNum;

  public:
  // override Constructor
  CSystemOptionsDialog(CWnd* pParentWnd = NULL); 
  
  // overriding OnInitDialog function
  virtual BOOL OnInitDialog();

  // overriding OnOk function
  virtual void OnOK();

  // setup data to edit
  void Setup(CConfiguration *Configuration, unsigned SystemNum);

  // do the dialog
  void DoDialog();
};  



// the system jump gate options dialog class
class CSystemJumpOptionsDialog : public CModalDialog
{
  CConfiguration    *pConfiguration;
  CSystemData       *pSystem;
  unsigned          uSystemNum;

  public:
  // override Constructor
  CSystemJumpOptionsDialog(CWnd* pParentWnd = NULL); 
  
  // overriding OnInitDialog function
  virtual BOOL OnInitDialog();

  // overriding OnOk function
  virtual void OnOK();

  // new OnAppearsButton method
  void OnAppearsButton();

  // new OnActDistButton method
  void OnActDistButton();

  // setup data to edit
  void Setup(CConfiguration *Configuration, unsigned SystemNum);

  // do the dialog
  void DoDialog();

  // macro to declare a message map
  DECLARE_MESSAGE_MAP()
};  



// the system jump gate options dialog class
class CSystemJumpDestinationDialog : public CModalDialog
{
  CConfiguration    *pConfiguration;
  CSystemData       *pSystem;
  unsigned          uSystemNum;
  CString           DestSystemName;
  BOOL              bShowAllSystems;

  public:
  // override Constructor
  CSystemJumpDestinationDialog(CWnd* pParentWnd = NULL); 
  
  // overriding OnInitDialog function
  virtual BOOL OnInitDialog();

  // overriding OnOk function
  virtual void OnOK();

  // setup data to edit
  void Setup(CConfiguration *Configuration, unsigned SystemNum);

  // show systems
  void ShowSystems();

  // new OnSelectButton method
  void OnSelectButton();

  // new OnRemoveButton method
  void OnRemoveButton();

  // new OnSelectSysButton method
  void OnSelectSysButton();

  // new OnAvSelChange method
  void OnAvSelChange();

  // new OnDestinationSelChange method
  void OnDestinationSelChange();

  // do the dialog
  void DoDialog();

  // macro to declare a message map
  DECLARE_MESSAGE_MAP()
};  



// the neutral behavior and action dialog class
class CNeutralBehaviorDialog : public CModalDialog
{
  CConfiguration    *pConfiguration;
  CSystemData       *pSystem;
  unsigned          uSystemNum;

  public:
  // override Constructor
  CNeutralBehaviorDialog(CWnd* pParentWnd = NULL); 
  
  // overriding OnInitDialog function
  virtual BOOL OnInitDialog();

  // overriding OnOk function
  virtual void OnOK();

  // new OnBehaviorDialogButton method
  virtual void OnBehaviorDialogButton();

  // new OnWarpInDialogButton method
  virtual void OnWarpInDialogButton();

  // new OnWarpOutDialogButton method
  virtual void OnWarpOutDialogButton();

  // new OnEnemyFriendAfterButton method
  virtual void OnEnemyFriendAfterButton();

  // new OnEnemyRandomButton method
  virtual void OnEnemyRandomButton();

  // new OnWarpInNeverButton method
  virtual void OnWarpInNeverButton();

  // new OnWarpInAfterButton method
  virtual void OnWarpInAfterButton();

  // new OnWarpOutClearedNeverButton method
  virtual void OnWarpOutClearedNeverButton();

  // new OnWarpOutAfterButton method
  virtual void OnWarpOutAfterButton();

  // setup data to edit
  void Setup(CConfiguration *Configuration, unsigned SystemNum);

  // do the dialog
  void DoDialog();

  // macro to declare a message map
  DECLARE_MESSAGE_MAP()
};  


// the friend enemy action dialog class
class CFriendEnemyActionDialog : public CModalDialog
{
  CConfiguration    *pConfiguration;
  CSystemData       *pSystem;
  unsigned          uSystemNum;

  public:
  // override Constructor
  CFriendEnemyActionDialog(CWnd* pParentWnd = NULL); 
  
  // overriding OnInitDialog function
  virtual BOOL OnInitDialog();

  // overriding OnOk function
  virtual void OnOK();

  // new OnFriendWarpInDialogButton method
  virtual void OnFriendWarpInDialogButton();

  // new OnEnemyWarpInDialogButton method
  virtual void OnEnemyWarpInDialogButton();

  // new OnFriendWarpOutDialogButton method
  virtual void OnFriendWarpOutDialogButton();

  // new OnEnemyWarpOutDialogButton method
  virtual void OnEnemyWarpOutDialogButton();

  // new OnFriendWarpInNeverButton method
  virtual void OnFriendWarpInNeverButton();

  // new OnFriendWarpInAfterButton method
  virtual void OnFriendWarpInAfterButton();

  // new OnEnemyWarpInNeverButton method
  virtual void OnEnemyWarpInNeverButton();

  // new OnEnemyWarpInAfterButton method
  virtual void OnEnemyWarpInAfterButton();

  // new OnEnemyWarpOutClearedNeverButton method
  virtual void OnEnemyWarpOutClearedNeverButton();

  // new OnEnemyWarpOutAfterButton method
  virtual void OnEnemyWarpOutAfterButton();

  // new OnFriendWarpOutClearedNeverButton method
  virtual void OnFriendWarpOutClearedNeverButton();

  // new OnFriendWarpOutAfterButton method
  virtual void OnFriendWarpOutAfterButton();

  // new OnFriendJoinEscortAfterButton method
  virtual void OnFriendJoinEscortAfterButton();

  // setup data to edit
  void Setup(CConfiguration *Configuration, unsigned SystemNum);

  // do the dialog
  void DoDialog();

  // macro to declare a message map
  DECLARE_MESSAGE_MAP()
};  


// the single-dialog dialog class
class CSingleDialogDialog : public CModalDialog
{
  CDialogList       *pDialogList;
  char              *pCaption, *pSubCaption;

  public:
  // override Constructor
  CSingleDialogDialog(CWnd* pParentWnd = NULL); 
  
  // overriding OnInitDialog function
  virtual BOOL OnInitDialog();

  // overriding OnOk function
  virtual void OnOK();

  // setup data to edit
  void Setup(CDialogList *DialogList, const char *Caption, const char *SubCaption);

  // do the dialog
  void DoDialog();

};  


// the double-dialog dialog class
class CDoubleDialogDialog : public CModalDialog
{
  CDialogList       *pDialogList1, *pDialogList2;
  char              *pCaption, *pSubCaption1, *pSubCaption2;

  public:
  // override Constructor
  CDoubleDialogDialog(CWnd* pParentWnd = NULL); 
  
  // overriding OnInitDialog function
  virtual BOOL OnInitDialog();

  // overriding OnOk function
  virtual void OnOK();

  // setup data to edit
  void Setup(CDialogList *DialogList1, CDialogList *DialogList2,
             const char *Caption, const char *SubCaption1, const char *SubCaption2);

  // do the dialog
  void DoDialog();

};  


// the check configuration dialog class
class CCheckConfigDialog : public CModalDialog
{
  CConfiguration               *pConfiguration;
  CAvailableShips              *pAvailableShips;
  CAvailableSystemsAndPlanets  *pAvailableSystemsAndPlanets;
  BOOL                         bMissingObjectFound;  
  char                         *strCaption;

  public:
  // override Constructor
  CCheckConfigDialog(CWnd* pParentWnd = NULL);

  // overriding the OnInitDialog method
  virtual BOOL OnInitDialog();

  // setup data 
  void Setup(CConfiguration *pConfig, CAvailableShips *pAvShips,
             CAvailableSystemsAndPlanets *pAvSystemsAndPlanets,
             const char *Caption);

  // do the dialog
  BOOL DoDialog();

};



// the ka setup dialog class
class CKASetupDialog
{
  CWnd     *pParentWindow;

  public:
  // override Constructor
  CKASetupDialog(CWnd* pParentWnd = NULL); 
  
  // do the dialog
  void DoDialog(void);

};  



// the import battle dialog class
class CImportBattleDialog
{
  // the current path and selection
  CString  strCurrentSel;
  CString  strCurrentPath;
  CListBox *pBattleListBox;
  BOOL     bBattleImported;
  CWnd     *pParentWindow;

  public:
  // override Constructor
  CImportBattleDialog(CWnd* pParentWnd = NULL); 
  
  // add the do import function
  BOOL DoImport();

  // do the dialog
  BOOL DoDialog(CListBox *BattleListBox);

};  



// the save dialog
class CSaveDialog : public CModalDialog
{
  char    *strConfigname;
  BOOL    bDoSave;

  public:
  // override Constructor
  CSaveDialog(CWnd* pParentWnd = NULL); 
  
  // overriding OnInitDialog function
  virtual BOOL OnInitDialog();

  // overriding OnOK function
  virtual void OnOK();

  // new OnSaveUnder function
  virtual void OnSaveUnder();

  // get the name
  BOOL DoDialog(char *configname);

  // macro to declare a message map
  DECLARE_MESSAGE_MAP()
};  


// the save under dialog
class CSaveUnderDialog : public CModalDialog
{
  char    strNewName[LINE_LENGTH];
  char    *strOldname;
  char    *strCaption;
  BOOL    bNameSet;

  public:
  // override Constructor
  CSaveUnderDialog(CWnd* pParentWnd = NULL); 
  
  // overriding OnInitDialog function
  virtual BOOL OnInitDialog();

  // overriding OnOK function
  virtual void OnOK();

  // get the name
  BOOL GetName(const char *oldname, char *newname, const char *caption);
};  


// the ship name dialog class
class CShipNameDialog : public CModalDialog
{
  char    *strCaption;
  char    *strOldname;
  char    strNewName[MAX_QB2_LINELENGTH];
  BOOL    bNameSet;

  public:
  // override Constructor
  CShipNameDialog(CWnd* pParentWnd = NULL); 
  
  // overriding OnInitDialog function
  virtual BOOL OnInitDialog();

  // overriding OnOK function
  virtual void OnOK();

  // get the name
  BOOL GetName(const char *caption, const char *oldname, char *newname);
};  


// the create new config dialog
class CCreateNewDialog : public CModalDialog
{
  char    strNewName[LINE_LENGTH];
  char    *strOldname;
  BOOL    bNameSet;

  public:
  // override Constructor
  CCreateNewDialog(CWnd* pParentWnd = NULL); 
  
  // overriding OnInitDialog function
  virtual BOOL OnInitDialog();

  // overriding OnOK function
  virtual void OnOK();

  // get the name
  BOOL GetName(const char *oldname, char *newname);
};  


// the delete dialog class
class CDeleteDialog : public CModalDialog
{
  BOOL           bDeleteIt;
  const char    *strDeleteObject;

  public:
  // override Constructor
  CDeleteDialog(CWnd* pParentWnd = NULL, const char *strDelObject = NULL); 
  
  // overriding OnInitDialog function
  virtual BOOL OnInitDialog();

  // overriding OnOK function
  virtual void OnOK();

  // new GetResult method
  BOOL GetResult();
};  


// the general confirm dialog class
class CConfirmDialog : public CModalDialog
{
  BOOL           bConfirmed;
  const char    *strConfirmObject;

  public:
  // override Constructor
  CConfirmDialog(CWnd* pParentWnd = NULL, const char *strConfObject = NULL); 
  
  // overriding OnInitDialog function
  virtual BOOL OnInitDialog();

  // overriding OnOK function
  virtual void OnOK();

  // new GetResult method
  BOOL GetResult();
};  


// the replace dialog class
class CReplaceDialog : public CModalDialog
{
  BOOL           bReplaceIt;
  const char    *strReplaceObject1;
  const char    *strReplaceObject2;

  public:
  // override Constructor
  CReplaceDialog(CWnd* pParentWnd = NULL, const char *strReplaceObject1 = NULL, 
                                          const char *strReplaceObject2 = NULL); 
  
  // overriding OnInitDialog function
  virtual BOOL OnInitDialog();

  // overriding OnOK function
  virtual void OnOK();

  // new GetResult method
  BOOL GetResult();
};  


// the overwrite dialog class
class COverwriteDialog : public CModalDialog
{
  BOOL     bOverwriteIt;
  char*    strOverwriteObject;

  public:
  // override Constructor
  COverwriteDialog(CWnd* pParentWnd = NULL, char *strOverwriteObject = NULL); 
  
  // overriding OnInitDialog function
  virtual BOOL OnInitDialog();

  // overriding OnOK function
  virtual void OnOK();

  // new GetResult method
  BOOL GetResult();
};  


// the cancel dialog class
class CCancelDialog : public CModalDialog
{
  BOOL     bCancel;

  public:
  // override Constructor
  CCancelDialog(CWnd* pParentWnd = NULL); 
  
  // overriding OnOK function
  virtual void OnOK();

  // new GetResult method
  BOOL GetResult();
};  


// the error dialog class
class CErrorDialog : public CModalDialog
{
  char *strCaption;
  char *strMessage;

  public:
  // override Constructor
  CErrorDialog(CWnd* pParentWnd = NULL); 

  // overriding OnInitDialog function
  virtual BOOL OnInitDialog();

  // report an error
  void Report(const char *message, const char *caption);

};


// the message dialog class
class CMessageDialog : public CModalDialog
{
  char *strCaption;
  char *strMessage;

  public:
  // override Constructor
  CMessageDialog(CWnd* pParentWnd = NULL); 

  // overriding OnInitDialog function
  virtual BOOL OnInitDialog();

  // report an error
  void Report(const char *message, const char *caption);

};



// the main editor class
class CMainQB2Edit
{
  // the configuration object
  CConfiguration Configuration;

  // the edit dialog objects
  CGeneralDialog           GeneralDialog;
  CSystemDialog            SystemDialog;
  CSystemFriendEnemyDialog SystemFriendEnemyDialog;
  CSystemNeutralHulkDialog SystemNeutralHulkDialog;
  CSystemJumpGateDialog    SystemJumpGateDialog;

  public:

  // The constructor
  CMainQB2Edit();

  // The destructor
  ~CMainQB2Edit();

  // do edit
  BOOL DoEdit(char *ConfigName, BOOL CreateNewConfig);

  // do check
  BOOL DoCheck(char *ConfigName);
};


#endif
