// ---------------------------------------------------------------------------------
// Quick Battle 2 editor
//
// by Uwe Brinkschulte
//
// Main File
// ----------------------------------------------------------------------------------

// ------------------------------ includes ---------------------------------------------

#include <afxwin.h>
#include <commdlg.h>
#include <shlobj.h>
#include "resrc1.h"
#include "qb2text.h"
#include "qb2config.h"
#include "qb2edit.h"

// ------------------------------ defines ----------------------------------------------


// ------------------------------ variables --------------------------------------------

// the main object
CTheApp         theApp;

// the available ships
CAvailableShips AvailableShips;

// the available systems and planets
CAvailableSystemsAndPlanets AvailableSystemsAndPlanets;

// global path to KA main folder (this path is set by registry info)
CString         strPathToKA = "c:\\";

// global path to the folder for importing battles
CString         strImportPath = "";

// a flag to indicate path to import folder is not set yet
BOOL            bImportPathNotSet = TRUE;

// ------------------------------  functions -------------------------------------------


//-----------------------------------
// formats ship class and name in a string
//-----------------------------------
void FormatShipClassAndName(CString *out, CShipData *ship)
{
  // first set the class
  *out = ship->GetShipClass();
  // do we have a name for the ship
  if (ship->IsSetShipName())
  {
    // yes, add it
    *out = *out + " --> ";
    *out = *out + ship->GetShipName();
  }

}

//-----------------------------------
// formats ship class and name in a string in another way
//-----------------------------------
void AnotherFormatShipClassAndName(CString *out, CShipData *ship)
{
  // first set the class
  *out = ship->GetShipClass();
  // do we have a name for the ship
  if (ship->IsSetShipName())
  {
    // yes, add it
    *out = *out + "   (";
    *out = *out + ship->GetShipName();
    *out = *out + ")";
  }

}

//-----------------------------------
// formats system name together with friends, enemies, neutrals, hulks and gates in a string
//-----------------------------------
void FormatSystemName(CString *out, CSystemData *system)
{
  char         str[MAX_QB2_LINELENGTH];
  unsigned     nGates;

  // the system name
  *out = system->GetSystemName();
  // do we have an active gate ?
  nGates = 0;
  if (system->GetJumpGate()->IsSetShipClass()) nGates = 1;
  // get system quick info
  sprintf(str,SYS_QUICKINFO, system->GetFriends()->GetCount(),  system->GetEnemies()->GetCount(), 
                             system->GetNeutrals()->GetCount(), system->GetHulks()->GetCount(),
                             nGates);
  *out = *out + str;
}

//-----------------------------------
// formats positions in a string, returns the string and info (posdefined)
// (iWho = 0:player, 1:friend, 2:enemy, 3:neutral, 4:hulk)
//-----------------------------------
void FormatPosition(CString *out, EPosDefType *posdefined, CSystemData *system, unsigned iWho)
{
  char str[20];

  *posdefined = NoPositionDefined;

  if (iWho == 0)
  {
    // show the player position
    if (system->IsSetPlayerPosPlanet())
    {
      *out = system->GetPlayerPosPlanet();
      *posdefined = PlanetPositionDefined;
    } else
    if (system->IsSetPlayerPos())
    {
      sprintf(str,"%li",system->GetPlayerPosX());
      *out = str;
      *out = *out + "   ";
      sprintf(str,"%li",system->GetPlayerPosY());
      *out = *out + str;
      *out = *out + "   ";
      sprintf(str,"%li",system->GetPlayerPosZ());
      *out = *out + str;
      *posdefined = CoordPositionDefined;
    } else
    *out = "";
  } else

  if (iWho == 1)
  {
    // show the friend position
    if (system->IsSetFriendPosPlanet())
    {
      *out = system->GetFriendPosPlanet();
      *posdefined = PlanetPositionDefined;
    } else
    if (system->IsSetFriendPos())
    {
      sprintf(str,"%li",system->GetFriendPosX());
      *out = str;
      *out = *out + "   ";
      sprintf(str,"%li",system->GetFriendPosY());
      *out = *out + str;
      *out = *out + "   ";
      sprintf(str,"%li",system->GetFriendPosZ());
      *out = *out + str;
      *posdefined = CoordPositionDefined;
    } else
    *out = "";
  } else

  if (iWho == 2)
  {
    // show the enemy position
    if (system->IsSetEnemyPosPlanet())
    {
      *out = system->GetEnemyPosPlanet();
      *posdefined = PlanetPositionDefined;
    } else
    if (system->IsSetEnemyPos())
    {
      sprintf(str,"%li",system->GetEnemyPosX());
      *out = str;
      *out = *out + "   ";
      sprintf(str,"%li",system->GetEnemyPosY());
      *out = *out + str;
      *out = *out + "   ";
      sprintf(str,"%li",system->GetEnemyPosZ());
      *out = *out + str;
      *posdefined = CoordPositionDefined;
    } else
    *out = "";
  } else

  if (iWho == 3)
  {
    // show the neutral position
    if (system->IsSetNeutralPosPlanet())
    {
      *out = system->GetNeutralPosPlanet();
      *posdefined = PlanetPositionDefined;
    } else
    if (system->IsSetNeutralPos())
    {
      sprintf(str,"%li",system->GetNeutralPosX());
      *out = str;
      *out = *out + "   ";
      sprintf(str,"%li",system->GetNeutralPosY());
      *out = *out + str;
      *out = *out + "   ";
      sprintf(str,"%li",system->GetNeutralPosZ());
      *out = *out + str;
      *posdefined = CoordPositionDefined;
    } else
    *out = "";
  } else

  if (iWho == 4)
  {
    // show the hulk position
    if (system->IsSetHulkPosPlanet())
    {
      *out = system->GetHulkPosPlanet();
      *posdefined = PlanetPositionDefined;
    } else
    if (system->IsSetHulkPos())
    {
      sprintf(str,"%li",system->GetHulkPosX());
      *out = str;
      *out = *out + "   ";
      sprintf(str,"%li",system->GetHulkPosY());
      *out = *out + str;
      *out = *out + "   ";
      sprintf(str,"%li",system->GetHulkPosZ());
      *out = *out + str;
      *posdefined = CoordPositionDefined;
    } else
    *out = "";
  } else

  if (iWho == 5)
  {
    // show the jump gate position
    if (system->IsSetJumpGatePosPlanet())
    {
      *out = system->GetJumpGatePosPlanet();
      *posdefined = PlanetPositionDefined;
    } else
    if (system->IsSetJumpGatePos())
    {
      sprintf(str,"%li",system->GetJumpGatePosX());
      *out = str;
      *out = *out + "   ";
      sprintf(str,"%li",system->GetJumpGatePosY());
      *out = *out + str;
      *out = *out + "   ";
      sprintf(str,"%li",system->GetJumpGatePosZ());
      *out = *out + str;
      *posdefined = CoordPositionDefined;
    } else
    *out = "";
  }
}


//-----------------------------------
// formats dialogs, str is the unformated line
// returns the speaker (strSpeaker), the voice file (strVoice), the line (str) and if some 
// modifications were necessary. Returns TRUE if the dialog can be added
//-----------------------------------
BOOL FormatDialog(char *str, char *strSpeaker, char *strVoice, BOOL *bTruncate, BOOL *bDefaultSpeaker,
                  BOOL *bEmptyLine, BOOL *bFisRemoved)
{
  unsigned j;

  // first strip the line
  StripString(str,str);
  // a totally empty line ?
  if (str[0] == '\0') return FALSE;

  // isolate the optional voice file
  strVoice[0] = '\0';
  if (str[0] == '[')
  {
    // look for closing bracket
    Tokenize(strVoice, str, ']');
    // is there a voice file ?
    if (strlen(strVoice) == strlen(str))
    {
      // no
      strVoice[0] = '\0';
    } else
    {
      // yes we have a voice file, remove it from the input string
      GetSubString(str, str, strlen(strVoice)+1, strlen(str)-1);
      StripString(str, str);
      // remove the opening bracket from the voice file
      GetSubString(strVoice, strVoice, 1, strlen(strVoice)-1);
      StripString(strVoice, strVoice);
    }
  }

  // get the speaker
  Tokenize(strSpeaker, str, ':');
  // is there a speaker ?
  if (strlen(strSpeaker) == strlen(str))
  {
    // no, empty speaker
    strSpeaker[0] = '\0';
  } else
  {
    // get the dialog line
    GetSubString(str, str, strlen(strSpeaker)+1, strlen(str)-1);
  }
  // strip the line
  StripString(str, str);
  // strip the speaker
  StripString(strSpeaker, strSpeaker);

  // dialog line empty ?
  if (str[0] == '\0')
  {
    *bEmptyLine = TRUE;
    return FALSE;
  }

  // speaker empty ?
  if (strSpeaker[0] == '\0')
  {
    // set default speaker
    strcpy(strSpeaker, DEFAULT_SPEAKER);
    *bDefaultSpeaker = TRUE;
  } 

  // speaker too long ?
  if (strlen(strSpeaker) > MAX_SPEAKER)
  {
    // truncate it
    strSpeaker[MAX_SPEAKER] = '\0';
    *bTruncate = TRUE;
  }

  // dialog line too long ?
  if (strlen(str) > MAX_DIALINE)
  { 
    // truncate it
    str[MAX_DIALINE] = '\0';
    *bTruncate = TRUE;
  }

  // remove # characters from speaker
  for (j = 0; j < strlen(strSpeaker); j++) if (strSpeaker[j] == '#')
  {
    strSpeaker[j] = '*';
    *bFisRemoved = TRUE;
  }

  // remove # characters from dialog line
  for (j = 0; j < strlen(str); j++) if (str[j] == '#')
  {
    str[j] = '*';
    *bFisRemoved = TRUE;
  }

  // remove # characters from voice file
  for (j = 0; j < strlen(strVoice); j++) if (strVoice[j] == '#')
  {
    strVoice[j] = '*';
    *bFisRemoved = TRUE;
  }

  // ok, we can use the line
  return TRUE;
}

//-----------------------------------
// formats title or epilog lines, str is the unformated line
// returns the voice file (strVoice), the line (str) and if some modifications were necessary
// returns TRUE if the line can be added
//-----------------------------------
BOOL FormatTitleEpilogLine(char *str, char *strVoice, BOOL *bTruncate, BOOL *bEmptyLine, BOOL *bFisRemoved)
{
  unsigned j;

  // first strip the line
  StripString(str,str);
  // a totally empty line ?
  if (str[0] == '\0') return FALSE;

  // isolate the optional voice file
  strVoice[0] = '\0';
  if (str[0] == '[')
  {
    // look for closing bracket
    Tokenize(strVoice, str, ']');
    // is there a voice file ?
    if (strlen(strVoice) == strlen(str))
    {
      // no
      strVoice[0] = '\0';
    } else
    {
      // yes we have a voice file, remove it from the input string
      GetSubString(str, str, strlen(strVoice)+1, strlen(str)-1);
      StripString(str, str);
      // remove the opening bracket from the voice file
      GetSubString(strVoice, strVoice, 1, strlen(strVoice)-1);
      StripString(strVoice, strVoice);
    }
  }

  // line too long ?
  if (strlen(str) > MAX_TITLELINE)
  {
    // truncate
    str[MAX_TITLELINE] = '\0';
    *bTruncate = TRUE;
  }

  // line empty ?
  if (str[0] == '\0')
  {
    *bEmptyLine = TRUE;
    return FALSE;
  }

  // remove # characters from line
  for (j = 0; j < strlen(str); j++) if (str[j] == '#')
  {
    str[j] = '*';
    *bFisRemoved = TRUE;
  }

  // remove # characters from voice file
  for (j = 0; j < strlen(strVoice); j++) if (strVoice[j] == '#')
  {
    strVoice[j] = '*';
    *bFisRemoved = TRUE;
  }

  // ok, we can use the line
  return TRUE;
}



//-----------------------------------
// asks for a new shipname, check it against the configuration
//-----------------------------------
BOOL AskForShipName(CShipData *pShip, CConfiguration *pConfiguration, char *grouptext)
{

  CShipNameDialog ShipNameDialog(NULL);
  CErrorDialog    ErrorDialog(NULL);
  char            str[MAX_QB2_LINELENGTH + 1];
  CString         CStr, OldName;
  BOOL            res;
  unsigned        where, who;

  CStr = SHIPNAME_TEXT;
  CStr = CStr + pShip->GetShipClass();
  CStr = CStr + grouptext;
  if (pShip->IsSetShipName())
  {
    res = ShipNameDialog.GetName(CStr, pShip->GetShipName(), str);
    // store old name, we may need it later
    OldName = pShip->GetShipName();
  }
  else
  {
    res = ShipNameDialog.GetName(CStr, NULL, str);
    OldName.Empty();
  }

  // user pressed cancel ?
  if (!res) return FALSE;

  // user pressed ok, so wipe out the old ship name
  pShip->UnsetShipName();

  // strip the new ship name
  StripString(str, str);

  // a empty string given ? If so, so we are finished here
  if (str[0] == '\0') return TRUE;

  // check the new name (old name must be wiped out here because we don't want to check
  // the old name against the new name. If both are the same this would cause an error)
  if (!pConfiguration->CheckShipName(str, FALSE, FALSE, FALSE, &where, &who))
  {
    // new name already in use, build error message
    CStr = ERR_NAMEREDEF1;
    CStr = CStr + str;
    CStr = CStr + ERR_NAMEREDEF2;
    if (who == 0) CStr = CStr + ERR_NAMEREDEF3a; else
    if (who == 1) CStr = CStr + ERR_NAMEREDEF3b; else
    if (who == 2) CStr = CStr + ERR_NAMEREDEF3c; else
    if (who == 3) CStr = CStr + ERR_NAMEREDEF3d; else
    if (who == 4) CStr = CStr + ERR_NAMEREDEF3e; else
    if (who == 5) CStr = CStr + ERR_NAMEREDEF3f; else
    if (who == 6) CStr = CStr + ERR_NAMEREDEF3g; else
    if (where > 0)
    {
      CStr = CStr + ERR_NAMEREDEF4;
      sprintf(str,"%u",where);
      CStr = CStr + str;
    }
    ErrorDialog.Report(CStr, ERR_HEADER);
    // restore the old name
    if (!OldName.IsEmpty()) pShip->SetShipName(OldName);

    return FALSE;
  }

  // set the new name
  pShip->SetShipName(str);

  return TRUE;
}

//-----------------------------------
// test the self destruct code
//-----------------------------------
BOOL TestSelfDestruct(char *strDestructCode)
{
  unsigned       i;
  BOOL           bToUpper, bInvalid;
  CErrorDialog   ErrorDialog;
  CMessageDialog Message;

  bToUpper = FALSE;
  bInvalid = FALSE;

  // the edit box ensures the code is not too long, but it may contain invalid characters
  for (i=0; i < strlen(strDestructCode); i++)
  {
    // check for lower case
    if ((strDestructCode[i] >= 'a') && (strDestructCode[i] <= 'z'))
    {
      // make it upper case
      strDestructCode[i] = toupper(strDestructCode[i]);
      bToUpper = TRUE;
    } else
    // check for invalid chars
    if (!(((strDestructCode[i] >= 'A') && (strDestructCode[i] <= 'Z')) ||
         ((strDestructCode[i] >= '0') && (strDestructCode[i] <= '9'))))
    {
      // error
      bInvalid = TRUE;
    }
  }

  // if invalid, create an error
  if (bInvalid) ErrorDialog.Report(ERR_DESTRUCTINVALID, ERR_HEADER);

  // if lower to upper conversion, create a warning
  if (bToUpper) Message.Report(WARN_DESTRUCTLOWER, WARN_HEADER);

  // allow the code, if not invalid
  return !bInvalid;
}


//-----------------------------------
// to some tests before finishing the configuration
//-----------------------------------
BOOL FinishTest(CConfiguration *pConfiguration)
{
  CErrorDialog    ErrorDialog(NULL);
  CMessageDialog  Message;
  CString         CStr, shipName;
  CSystemData     *pSystem;
  POSITION        pos;
  unsigned        uWhereGiven, uGroupGiven, uWhereDefault, uGroupDefault;
  char            str[20];

  // check the default ship names for conflicts with given names
  if (!pConfiguration->CheckAllGivenAgainstDefaultShipNames(&uWhereGiven, &uGroupGiven, &uWhereDefault, &uGroupDefault, &shipName))
  {
    // error, create the error message
    CStr = ERR_DEFAULTREDEF1;
    if (uGroupGiven == 0) CStr = CStr + ERR_DEFAULTREDEF2a; else
    if (uGroupGiven == 1) CStr = CStr + ERR_DEFAULTREDEF2b; else
    if (uGroupGiven == 2) CStr = CStr + ERR_DEFAULTREDEF2c; else
    if (uGroupGiven == 3) CStr = CStr + ERR_DEFAULTREDEF2d; else
    if (uGroupGiven == 4) CStr = CStr + ERR_DEFAULTREDEF2e; else
    if (uGroupGiven == 5) CStr = CStr + ERR_DEFAULTREDEF2f; else
    if (uGroupGiven == 6) CStr = CStr + ERR_DEFAULTREDEF2g;
    CStr = CStr + shipName;
    if (uWhereGiven > 0)
    {
      CStr = CStr + ERR_DEFAULTREDEF3;
      sprintf(str,"%u",uWhereGiven);
      CStr = CStr + str;
    }
    CStr = CStr + ERR_DEFAULTREDEF4;
    if (uGroupDefault == 0) CStr = CStr + ERR_DEFAULTREDEF2a; else
    if (uGroupDefault == 1) CStr = CStr + ERR_DEFAULTREDEF2b; else
    if (uGroupDefault == 2) CStr = CStr + ERR_DEFAULTREDEF2c; else
    if (uGroupDefault == 3) CStr = CStr + ERR_DEFAULTREDEF2d; else
    if (uGroupDefault == 4) CStr = CStr + ERR_DEFAULTREDEF2e; else
    if (uGroupDefault == 5) CStr = CStr + ERR_DEFAULTREDEF2f; else
    if (uGroupDefault == 6) CStr = CStr + ERR_DEFAULTREDEF2g;
    if (uWhereDefault > 0)
    {
      CStr = CStr + ERR_DEFAULTREDEF5;
      sprintf(str,"%u",uWhereDefault);
      CStr = CStr + str;
    }
    CStr = CStr + ERR_DEFAULTREDEF6;
    // report it
    ErrorDialog.Report(CStr, ERR_HEADER);

    return FALSE;
  }

  // check for some more errors
  if (!(pConfiguration->FinalConfigCheck(TRUE))) return FALSE;

  // check for incomplete jump gates
  CStr = "";
  // loop through all systems
  for (pos = pConfiguration->GetSystems()->GetHeadPosition(); pos != NULL; )
  {
    // iterate through systems
    pSystem = (CSystemData *)pConfiguration->GetSystems()->GetNext(pos);
    // do the check
    if (pSystem->GetJumpGate()->IsSetShipClass() &&
        ((!pSystem->IsSetJumpGatePos() && !pSystem->IsSetJumpGatePosPlanet() && !pSystem->IsSetJumpTime()) ||
         (!pSystem->IsSetJumpDestination())))
    {
      // we have an inclomplete gate, put the system in the output list
      if (CStr.GetLength() > 0) CStr = CStr + ", ";
      CStr = CStr + pSystem->GetSystemName();
    }
  }
  // print an error and return if we have found an incomplete gate
  if (CStr.GetLength() > 0)
  {
    CStr = ERR_INCOMPLETEGATE + CStr;
    CStr = CStr + ERR_INCOMPLETEGATE1;
    Message.Report(CStr, ERR_HEADER);
    return FALSE;
  }

  // check for conflicts with statistic battle mode in the systems
  CStr = "";
  // loop through all systems
  for (pos = pConfiguration->GetSystems()->GetHeadPosition(); pos != NULL; )
  {
    // iterate through systems
    pSystem = (CSystemData *)pConfiguration->GetSystems()->GetNext(pos);
    // do the check
    if (!pConfiguration->CheckStatBattleMode(pSystem, FALSE))
    {
      // put the system in the output list
      if (CStr.GetLength() > 0) CStr = CStr + ", ";
      CStr = CStr + pSystem->GetSystemName();
      // set battle mode to AI
      pSystem->SetBattleMode(AIBattle);
    }
  }
  // print the warning if we have modified the battle mode
  // (but we can continue because we have corrected the problem)
  if (CStr.GetLength() > 0)
  {
    CStr = WARN_BATTLEMODE + CStr;
    CStr = CStr + WARN_BATTLEMODE1;
    Message.Report(CStr, WARN_HEADER);
  }    

  // check if self destruct code is too long
  if ((pConfiguration->GetSelfDestruct()->GetCode()).GetLength() > MAX_SELFDESTRUCTCODE)
  {
    // truncate it
    CStr = (pConfiguration->GetSelfDestruct()->GetCode()).Left(MAX_SELFDESTRUCTCODE);
    pConfiguration->GetSelfDestruct()->SetCode(CStr);
    // print a warning
    Message.Report(WARN_DESTRUCTTRUNC, WARN_HEADER);
  }

  return TRUE;
}

//-----------------------------------
// find a string in a list box (exact find, the list box FindString looks for prefix only)
//-----------------------------------
BOOL ExactFindInListBox(CListBox *Listbox, const char *str, LONG *iItemFound)

{
  LONG    iItem, iNumberOfItems;
  CString CStr;

  // get the number of items in the box
  iNumberOfItems = Listbox->GetCount();
  // search the string
  for (iItem = 0; iItem < iNumberOfItems; iItem++)
  {
    // get the string
    Listbox->GetText(iItem, CStr);
    // compare it
    if (CStr.CompareNoCase(str) == 0)
    {
      *iItemFound = iItem;
      return TRUE;
    }
  }
  
  // not found
  return FALSE;
}


//-----------------------------------
// draw a bitmap
//-----------------------------------
void DrawBitmap(CPaintDC *dc, LONG BitmapID, int x, int y)
{
  CBitmap Bitmap;
  BITMAP bm;
  
  // load the bitmap
  if (Bitmap.LoadBitmap(BitmapID))
  {

    // create the compatible drawing context
    HDC hdcMem = ::CreateCompatibleDC(dc->m_hDC);
    if (hdcMem != NULL)
    {
      // select the bitmap
      ::SelectObject(hdcMem, Bitmap.m_hObject);
      // get dimensions
      ::GetObject(Bitmap.m_hObject, sizeof(bm), (LPSTR)&bm);
      // output
      ::BitBlt(dc->m_hDC, x - (bm.bmWidth / 2), y - (bm.bmHeight / 2), bm.bmWidth, bm.bmHeight, hdcMem, 0, 0, SRCCOPY);
      // delete the context
      ::DeleteDC(hdcMem);
    }
  }
}

// ------------------------------  classes  --------------------------------------------


// ----------- the main application class -----------------------------------

//-----------------------------------
// overriding the InitInstance method
//-----------------------------------
BOOL CTheApp::InitInstance()
{
  // handle to registry entry
  HKEY      hRegHandle;
  // some variables to get the registry value
  char      strName[10];
  DWORD     lNameLen = 10;
  char      strValue[PATH_LENGTH];
  DWORD     lValueLen = PATH_LENGTH;

  // Construct a window object in the heap
  m_pMainWnd = new CMainWindow();

  // show the window
  m_pMainWnd->ShowWindow(m_nCmdShow);

  // paint the window
  m_pMainWnd->UpdateWindow();

  // check if already registered
  if (RegOpenKey(HKEY_LOCAL_MACHINE,QB2EDIT_REGKEY,&hRegHandle) != ERROR_SUCCESS)
  {
     // create the key
     RegCreateKey(HKEY_LOCAL_MACHINE,QB2EDIT_REGKEY,&hRegHandle);

     // force configuration
     ((CMainWindow *)m_pMainWnd)->OnConfigure();
    
  }

  // get the KA main directory
  RegEnumValue(hRegHandle,0,strName,&lNameLen,NULL,NULL,(LPBYTE)strValue,&lValueLen);
  strPathToKA = strValue;  
  
  return TRUE;
}



// ----------- the main window class ----------------------------------------

//-----------------------------------
// overriding the Constructor
//-----------------------------------
CMainWindow::CMainWindow()
{
   // load the shortcut table
   LoadAccelTable("MainAccelTable");
   // create the window
   Create(NULL,MAIN_CAPTION, WS_OVERLAPPEDWINDOW, rectDefault, NULL, "MainMenu");
}

//-----------------------------------
// overriding the OnPaint method
//-----------------------------------
void CMainWindow::OnPaint()
{
   CString  s = WELCOME_TEXT;
   CString  s1 = VERSION_TEXT;
   CPaintDC dc(this);
   CRect    rect;

   // get the window size
   GetClientRect(rect);
   // allign the text
   dc.SetTextAlign(TA_BASELINE | TA_CENTER);
   dc.SetBkMode(TRANSPARENT);
   // write the welcome text
   dc.TextOut(rect.right / 2, (rect.bottom / 2) - 100, s, s.GetLength());
   dc.TextOut(rect.right / 2, (rect.bottom / 2) + 100, s1, s1.GetLength());

   // draw a bitmap
   DrawBitmap(&dc, IDB_KLINGON_BITMAP, rect.right / 2, rect.bottom / 2);
}

//-----------------------------------
// adding the OnAbout method
//-----------------------------------
void CMainWindow::OnAbout()
{
   CModalDialog about("ABOUTBOX", this);

   // show the about box
   about.DoModal();
}

//-----------------------------------
// adding the OnStart method
//-----------------------------------
void CMainWindow::OnStart()
{
   CStartDialog startDialog(this);

   // show the start dialog
   startDialog.DoModal();
}

//-----------------------------------
// adding the OnConfigure method
//-----------------------------------
void CMainWindow::OnConfigure()
{
   CKASetupDialog configureDialog(this);

   // show the configure dialog
   configureDialog.DoDialog();
}

//-----------------------------------
// adding the OnExit method
//-----------------------------------
void CMainWindow::OnExit()
{
   // exit the application
   PostQuitMessage(0);
}

//-----------------------------------
// the message map
//-----------------------------------
BEGIN_MESSAGE_MAP(CMainWindow, CFrameWnd)
  ON_WM_PAINT()
  ON_COMMAND(ID_HELP_ABOUT, OnAbout)
  ON_COMMAND(ID_FILE_START, OnStart)
  ON_COMMAND(ID_FILE_EXIT,  OnExit)
  ON_COMMAND(ID_CONFIGURE_PATH,  OnConfigure)
END_MESSAGE_MAP()



// ----------- the start dialog class ---------------------------------------

//-----------------------------------
// overriding the Constructor
//-----------------------------------
CStartDialog::CStartDialog(CWnd* pParentWnd) : CModalDialog("STARTBOX", pParentWnd)
{
  // init the spin button position
  lastSpinPosition = -1;
}

//-----------------------------------
// overriding the OnInitDialog method
//-----------------------------------
BOOL CStartDialog::OnInitDialog()
{
  CString      strCfgDescName;
  CString      strConfigFileName;
  CErrorDialog ErrorDialog(this);
  LONG         iItem;
  char         line[LINE_LENGTH];
  char         cline[LINE_LENGTH];
  char         *str;

  // flag to see if description changed
  bCfgDescFileChanged = FALSE;

  // try to open the configuration description
  strCfgDescName = strPathToKA + QB2DESC_PATH;
  str = strCfgDescName.GetBuffer(LINE_LENGTH);
  // ok ?
  if (!ConfigDescFile.Open(str))
  {
    // no, stop dialog
    ErrorDialog.Report(ERR_DESCNOTFOUND, ERR_HEADER);
    EndDialog(IDOK);
    return TRUE;
  }
  strCfgDescName.ReleaseBuffer();

  // check the configuration file
  if (!ConfigDescFile.ReadLine(line))
  {
    // empty file is invalid
    ErrorDialog.Report(ERR_DESCINVALID, ERR_HEADER);
    EndDialog(IDOK);
    return TRUE;
  }
  if (strcmp(line,QB2DESC_MAGIC) != 0)
  {
    // not the proper magic
    ErrorDialog.Report(ERR_DESCINVALID, ERR_HEADER);
    EndDialog(IDOK);
    return TRUE;
  }
  if (!ConfigDescFile.ReadLine(line))
  {
    // no version line
    ErrorDialog.Report(ERR_DESCINVALID, ERR_HEADER);
    EndDialog(IDOK);
    return TRUE;
  }
  if (strcmp(line,QB2DESC_VERSION) != 0)
  {
    // not the proper version
    ErrorDialog.Report(ERR_DESCVERSION, ERR_HEADER);
    EndDialog(IDOK);
    return TRUE;
  }

  // copy the lines to the message box
  while (ConfigDescFile.ReadLine(line))
  {
    // first do a little checking, try to open the battle file ?
    StripString(line,line);
    strConfigFileName = strPathToKA + QB2_PATH + line + ".qb2";
    str = strConfigFileName.GetBuffer(PATH_LENGTH);
    if (ConfigFile.Open(str))
    {
      // header ok ?
      if (ConfigFile.ReadLine(cline))
      {
        if (strcmp(cline,QB2_MAGIC) == 0)
        {
          // all is ok, copy the line to the message box
          SendDlgItemMessage(IDC_LISTBATTLE, LB_ADDSTRING,0,(LONG)(LPSTR)line);
        }
      }
      ConfigFile.Close();
    }
    strConfigFileName.ReleaseBuffer(); 
  }

  // check if we have a selection, enable edit, delete, up and down button if so
  iItem = SendDlgItemMessage(IDC_LISTBATTLE, LB_GETCURSEL,0,0);
  if (iItem != LB_ERR)
  {
    GetDlgItem(IDC_EDITBATTLE)->EnableWindow(TRUE);
    GetDlgItem(IDC_DELETEBATTLE)->EnableWindow(TRUE);
    GetDlgItem(IDC_CHECKBATTLE)->EnableWindow(TRUE);
    GetDlgItem(IDC_RENAMEBATTLE)->EnableWindow(TRUE);
    GetDlgItem(IDC_UPDOWNBATTLE)->EnableWindow(TRUE);
    GetDlgItem(IDC_UPDOWNSPIN)->EnableWindow(TRUE);
  }


  return TRUE;
}

//-----------------------------------
// new UpBattle method
//-----------------------------------
void CStartDialog::UpBattle()
{
  // move the selected battle one position up
  LONG iItem;
  char line[LINE_LENGTH];

  // get the selected battle
  iItem = SendDlgItemMessage(IDC_LISTBATTLE, LB_GETCURSEL,0,0);
  if ((iItem != LB_ERR) && (iItem > 0))
  {
    // get the battle
    SendDlgItemMessage(IDC_LISTBATTLE, LB_GETTEXT,(UINT)iItem,(LONG)(LPSTR)line);
    // delete the battle on its old position
    SendDlgItemMessage(IDC_LISTBATTLE, LB_DELETESTRING,(UINT)iItem,0);
    // put it to his new position
    iItem--;
    SendDlgItemMessage(IDC_LISTBATTLE, LB_INSERTSTRING,(UINT)iItem,(LONG)(LPSTR)line);
    // and select it again
    SendDlgItemMessage(IDC_LISTBATTLE, LB_SETCURSEL,(UINT)iItem,0);
    // config desc file changed
    bCfgDescFileChanged = TRUE;

  }
}

//-----------------------------------
// new DownBattle method
//-----------------------------------
void CStartDialog::DownBattle()
{
  // move the selected battle one position down
  LONG iItem, iNumberOfItems;
  char line[LINE_LENGTH];

  // get the selected battle
  iItem = SendDlgItemMessage(IDC_LISTBATTLE, LB_GETCURSEL,0,0);
  // get the number of items in the box
  iNumberOfItems = SendDlgItemMessage(IDC_LISTBATTLE, LB_GETCOUNT,0,0);
  if ((iItem != LB_ERR) && ((iItem + 1) < iNumberOfItems))
  {
    // get the battle
    SendDlgItemMessage(IDC_LISTBATTLE, LB_GETTEXT,(UINT)iItem,(LONG)(LPSTR)line);
    // delete the battle on its old position
    SendDlgItemMessage(IDC_LISTBATTLE, LB_DELETESTRING,(UINT)iItem,0);
    // put it to his new position
    iItem++;
    SendDlgItemMessage(IDC_LISTBATTLE, LB_INSERTSTRING,(UINT)iItem,(LONG)(LPSTR)line);
    // and select it again
    SendDlgItemMessage(IDC_LISTBATTLE, LB_SETCURSEL,(UINT)iItem,0);
    // config desc file changed
    bCfgDescFileChanged = TRUE;
  }
}

//-----------------------------------
// overriding the OnCancel method
//-----------------------------------
void CStartDialog::OnCancel()
{
  // save the changes in the desc file
  SaveAndCloseDescFile();
  EndDialog(IDOK);
}
 

//-----------------------------------
// new OnCreateButton method
//-----------------------------------
void CStartDialog::OnCreateButton()
{
  CMainQB2Edit     MainQB2Edit;
  CCreateNewDialog CreateNewDialog;
  CMessageDialog   Message;
  CCfgFile         ConfigFile;
  CString          CStr;
  char             line[LINE_LENGTH];
  LONG             iItem;

  // see if a battle is selected
  iItem = SendDlgItemMessage(IDC_LISTBATTLE, LB_GETCURSEL,0,0);
  if (iItem != LB_ERR)
    // get the name
    SendDlgItemMessage(IDC_LISTBATTLE, LB_GETTEXT,(UINT)iItem,(LONG)(LPSTR)line);
  else 
    line[0] = '\0';
  // ask for the name of the new battle, take the selected one as default
  if (CreateNewDialog.GetName(line, line))
  {
    // check if a valid name
    StripString(line,line);
    if (line[0]!='\0')
    {
      // check if configuration exists already
      CStr = strPathToKA + QB2_PATH + line + ".qb2";
      if (ConfigFile.Open(CStr))
      {
        // print a warning
        ConfigFile.Close();
        Message.Report(WARN_OVERWRITE, WARN_HEADER);
      }
      // edit it
      if (MainQB2Edit.DoEdit(line, TRUE))
      {
        // look if we have to update the configuration descriptor
        StripString(line,line);
        // look if we have to update the configuration descriptor
        if (!ExactFindInListBox((CListBox *)GetDlgItem(IDC_LISTBATTLE), line, &iItem))
        {
          // we have to add the new battle
          SendDlgItemMessage(IDC_LISTBATTLE, LB_ADDSTRING,0,(LONG)(LPSTR)line);
          // config desc file changed
          bCfgDescFileChanged = TRUE;
        }
      }
    }
  }
}


//-----------------------------------
// new OnEditButton method
//-----------------------------------
void CStartDialog::OnEditButton()
{
  CMainQB2Edit MainQB2Edit;
  char         line[LINE_LENGTH];
  LONG         iItem;

  // get the selected battle
  iItem = SendDlgItemMessage(IDC_LISTBATTLE, LB_GETCURSEL,0,0);
  if (iItem != LB_ERR)
  {
    // get the battle
    SendDlgItemMessage(IDC_LISTBATTLE, LB_GETTEXT,(UINT)iItem,(LONG)(LPSTR)line);
    // edit it
    if (MainQB2Edit.DoEdit(line, FALSE))
    {
      // look if we have to update the configuration descriptor
      StripString(line,line);
      // look if we have to update the configuration descriptor
      if (!ExactFindInListBox((CListBox *)GetDlgItem(IDC_LISTBATTLE), line, &iItem))
      {
        // we have to add the new battle
        SendDlgItemMessage(IDC_LISTBATTLE, LB_ADDSTRING,0,(LONG)(LPSTR)line);
        // config desc file changed
        bCfgDescFileChanged = TRUE;
      }
    }
  }
}


//-----------------------------------
// new OnDeleteButton method
//-----------------------------------
void CStartDialog::OnDeleteButton()
{
  LONG         iItem;
  CString      strConfigFileName;
  CErrorDialog ErrorDialog(this);
  char         line[LINE_LENGTH];
  char         *str;

  // get the selected battle
  iItem = SendDlgItemMessage(IDC_LISTBATTLE, LB_GETCURSEL,0,0);
  if (iItem != LB_ERR)
  {
    // get the battle
    SendDlgItemMessage(IDC_LISTBATTLE, LB_GETTEXT,(UINT)iItem,(LONG)(LPSTR)line);
    // confirm deletion    
    CDeleteDialog deleteDialog(NULL/*this*/, line);
    deleteDialog.DoModal();
    if (deleteDialog.GetResult())
    {
      // we delete it
      // delete the file
      strConfigFileName = strPathToKA + QB2_PATH + line + ".qb2";
      str = strConfigFileName.GetBuffer(PATH_LENGTH);
      if (remove(str) != 0)
      {
        ErrorDialog.Report(ERR_CANNOTDELETE, ERR_HEADER);
        return;
      }
      // delete the listbox entry     
      SendDlgItemMessage(IDC_LISTBATTLE, LB_DELETESTRING,(UINT)iItem,0);
      // set the new selection
      if (SendDlgItemMessage(IDC_LISTBATTLE, LB_SETCURSEL,(UINT)iItem,0) == LB_ERR)
      {
        // we cannot select this one, maybe end of the list. So lets try the previous one
        if (iItem > 0) iItem--;
        if (SendDlgItemMessage(IDC_LISTBATTLE, LB_SETCURSEL,(UINT)iItem,0) == LB_ERR)
        {
          // no selection possible, so disable the edit, delete, up and down button (spin & edit field) 
         GetDlgItem(IDC_DELETEBATTLE)->EnableWindow(FALSE);
         GetDlgItem(IDC_EDITBATTLE)->EnableWindow(FALSE);
         GetDlgItem(IDC_CHECKBATTLE)->EnableWindow(FALSE);
         GetDlgItem(IDC_RENAMEBATTLE)->EnableWindow(FALSE);
         GetDlgItem(IDC_UPDOWNBATTLE)->EnableWindow(FALSE);
         GetDlgItem(IDC_UPDOWNSPIN)->EnableWindow(FALSE);
        }
      }

      // config desc file changed
      bCfgDescFileChanged = TRUE;
    }
  }
}


//-----------------------------------
// new OnRenameButton method
//-----------------------------------
void CStartDialog::OnRenameButton()
{
  LONG             iItem, iItem1;
  CSaveUnderDialog SaveUnderDialog;
  CCfgFile         ConfigFile;
  CString          CStr, CStr1;
  CErrorDialog     ErrorDialog(this);
  char             line[LINE_LENGTH], strNewname[LINE_LENGTH];

  // get the selected battle
  iItem = SendDlgItemMessage(IDC_LISTBATTLE, LB_GETCURSEL,0,0);
  if (iItem != LB_ERR)
  {
    // get the battle
    SendDlgItemMessage(IDC_LISTBATTLE, LB_GETTEXT,(UINT)iItem,(LONG)(LPSTR)line);
    StripString(line, line);
    // ask for the new name
    if (SaveUnderDialog.GetName(line, strNewname, RENAME_CAPTION))
    {
      // check if a valid name and old and new name are different
      StripString(strNewname, strNewname);
      CStr = strNewname;
      if ((CStr.GetLength() != 0) && (CStr.CompareNoCase(line) != 0))
      {
        // check if configuration exists already
        CStr = strPathToKA + QB2_PATH + strNewname + ".qb2";
        if (ConfigFile.Open(CStr))
        {
          // print a warning
          ConfigFile.Close();
          // confirm overwrite
          COverwriteDialog overwriteDialog(NULL, strNewname);
          overwriteDialog.DoModal();
          if (!overwriteDialog.GetResult()) return;
          // delete the battle to overwrite
          if (remove(CStr) != 0)
          {
            ErrorDialog.Report(ERR_CANNOTDELETE, ERR_HEADER);
            return;
          }
        }

        // rename the battle
        CStr = strPathToKA + QB2_PATH + line + ".qb2";
        CStr1 = strPathToKA + QB2_PATH + strNewname + ".qb2";
        if (rename(CStr, CStr1) != 0)
        {
          ErrorDialog.Report(ERR_CANNOTRENAME, ERR_HEADER);
          return;
        }
 
        // delete the listbox entry     
        SendDlgItemMessage(IDC_LISTBATTLE, LB_DELETESTRING,(UINT)iItem,0);

        // look if we have to update the configuration descriptor
        if (!ExactFindInListBox((CListBox *)GetDlgItem(IDC_LISTBATTLE), strNewname, &iItem1))
        {
          // we have to add the new battle
          SendDlgItemMessage(IDC_LISTBATTLE, LB_INSERTSTRING,(UINT)iItem,(LONG)(LPSTR)strNewname);
          // and select it again
          SendDlgItemMessage(IDC_LISTBATTLE, LB_SETCURSEL,(UINT)iItem,0);
        } else
        {
          // and select the new item
          SendDlgItemMessage(IDC_LISTBATTLE, LB_SETCURSEL,(UINT)iItem1,0);
        }

        // config desc file changed
        bCfgDescFileChanged = TRUE;

      }
    }
  }
}


//-----------------------------------
// new OnCheckButton method
//-----------------------------------
void CStartDialog::OnCheckButton()
{
  CMainQB2Edit MainQB2Edit;
  char         line[LINE_LENGTH];
  LONG         iItem;

  // get the selected battle
  iItem = SendDlgItemMessage(IDC_LISTBATTLE, LB_GETCURSEL,0,0);
  if (iItem != LB_ERR)
  {
    // get the battle
    SendDlgItemMessage(IDC_LISTBATTLE, LB_GETTEXT,(UINT)iItem,(LONG)(LPSTR)line);
    // check it
    MainQB2Edit.DoCheck(line);
  }
}


//-----------------------------------
// new OnImportButton method
//-----------------------------------
void CStartDialog::OnImportButton()
{
  CImportBattleDialog ImportBattleDialog(this);

  // import a battle
  if (ImportBattleDialog.DoDialog((CListBox *)GetDlgItem(IDC_LISTBATTLE)))
  {
    // config desc file changed
    bCfgDescFileChanged = TRUE;
  }
}


//-----------------------------------
// new OnUpDownButton method
//-----------------------------------
void CStartDialog::OnUpDownButton()
{
  char str[MAX_QB2_LINELENGTH+1];
  long actSpinPos;

  // Moves the selected battle one position up or down in the list box
  // Uses UpBattle and DownBattle for this purpose
  //
  // This is a tricky solution: A spin button is used to implement the up and down in the list box.
  // Normaly, a spin button increments and decrements an edit box. It has the nice feature of
  // autorepeat when you keep the button pressed. 
  // To have this feature here as well, a hidden edit box is introduced. The spin button increments
  // and decrements the value of this box. This routine here is called whenever the content of
  // this hidden edit boc changes. The it evaluates if the value has increased or decreased and moves
  // the battle in the list box in the corresponding direction. The value of the edit box is then set back
  // to its iniial value => The spin button indirectly moves the battle in the list box with auto-repeat possible!

  // init the last spin position ? (first event when setting the initial value in the hidden edit box)
  if (lastSpinPosition == -1) 
  {
    lastSpinPosition = 1;
  } else
  {
    // we have an event in the hidden edit box related to the spin up/down buttons
    // find out if the value has increased or decreased, that's all we need to know
    SendDlgItemMessage(IDC_UPDOWNBATTLE, WM_GETTEXT,(WORD)MAX_QB2_LINELENGTH,(LONG)(LPSTR)str);
    actSpinPos = atol(str);
    // increased means DOWN
    if (actSpinPos > lastSpinPosition) DownBattle(); else
    // decreased means UP
    if (actSpinPos < lastSpinPosition) UpBattle();
    // since we are only interested in relative position, set the the hidden edit box back to its initial value
    if (atol(str) != lastSpinPosition)
    {
      // note: we only do this if the value has changed. Otherwise we would cause an endless event series, because
      //       changing the value causes again an event a.s.o. So for the second event the value has not changed an
      //       we can stop this series. 
      lastSpinPosition = 1;
      sprintf(str,"%lu",lastSpinPosition);
      SendDlgItemMessage(IDC_UPDOWNBATTLE, WM_SETTEXT,(WORD)0,(LONG)(LPSTR)str);
    }
  }
}


//-----------------------------------
// new OnSelChange method
//-----------------------------------
void CStartDialog::OnSelChange()
{
  LONG iItem;

  // selection in the listbox has changed

  // check if we have a selection
  iItem = SendDlgItemMessage(IDC_LISTBATTLE, LB_GETCURSEL,0,0);
  if (iItem != LB_ERR)
  {
    // enable edit, delete, up and down button if so
    GetDlgItem(IDC_EDITBATTLE)->EnableWindow(TRUE);
    GetDlgItem(IDC_DELETEBATTLE)->EnableWindow(TRUE);
    GetDlgItem(IDC_CHECKBATTLE)->EnableWindow(TRUE);
    GetDlgItem(IDC_RENAMEBATTLE)->EnableWindow(TRUE);
    GetDlgItem(IDC_UPDOWNBATTLE)->EnableWindow(TRUE);
    GetDlgItem(IDC_UPDOWNSPIN)->EnableWindow(TRUE);
  } else
  {
    // else disable the buttons
    GetDlgItem(IDC_EDITBATTLE)->EnableWindow(FALSE);
    GetDlgItem(IDC_DELETEBATTLE)->EnableWindow(FALSE);
    GetDlgItem(IDC_CHECKBATTLE)->EnableWindow(FALSE);
    GetDlgItem(IDC_RENAMEBATTLE)->EnableWindow(FALSE);
    GetDlgItem(IDC_UPDOWNBATTLE)->EnableWindow(FALSE);
    GetDlgItem(IDC_UPDOWNSPIN)->EnableWindow(FALSE);
  }

}

//-----------------------------------
// new SaveAndCloseDescFile method
//-----------------------------------
void CStartDialog::SaveAndCloseDescFile()
{
  LONG iItem, iNumberOfItems;
  char line[LINE_LENGTH];

  // changes in config description
  if (bCfgDescFileChanged)
  {
    // yes, reset the file
    ConfigDescFile.Reset();
    // write header
    ConfigDescFile.WriteLine(QB2DESC_MAGIC);
    ConfigDescFile.WriteLine(QB2DESC_VERSION);
    // write data
    iNumberOfItems = SendDlgItemMessage(IDC_LISTBATTLE, LB_GETCOUNT,0,0);
    for (iItem = 0; iItem < iNumberOfItems; iItem ++)
    {
      // get the battle
      SendDlgItemMessage(IDC_LISTBATTLE, LB_GETTEXT,(UINT)iItem,(LONG)(LPSTR)line);
      // write it to file
      ConfigDescFile.WriteLine(line);
    }
  }

  // close config description file
  ConfigDescFile.Close();
}

//-----------------------------------
// the message map
//-----------------------------------
BEGIN_MESSAGE_MAP(CStartDialog, CModalDialog)
  ON_BN_CLICKED(IDC_CREATEBATTLE,    OnCreateButton)
  ON_BN_CLICKED(IDC_EDITBATTLE,      OnEditButton)
  ON_BN_CLICKED(IDC_DELETEBATTLE,    OnDeleteButton)
  ON_BN_CLICKED(IDC_RENAMEBATTLE,    OnRenameButton)
  ON_BN_CLICKED(IDC_CHECKBATTLE,     OnCheckButton)
  ON_BN_CLICKED(IDC_IMPORTBATTLE,    OnImportButton)
  ON_EN_UPDATE(IDC_UPDOWNBATTLE,     OnUpDownButton)
  ON_LBN_DBLCLK(IDC_LISTBATTLE,      OnEditButton)
  ON_LBN_SELCHANGE(IDC_LISTBATTLE,   OnSelChange)
END_MESSAGE_MAP()


// ----------- the general dialog class ---------------------------------------

//-----------------------------------
// overriding the Constructor
//-----------------------------------
CGeneralDialog::CGeneralDialog(CWnd* pParentWnd) : CModalDialog("GENERALBOX", pParentWnd)
{
  // init
  pConfiguration = NULL;
  eDialogResult = CancelDialog;
}

//-----------------------------------
// overriding the OnInitDialog method
//-----------------------------------
BOOL CGeneralDialog::OnInitDialog()
{

  CString      CStr;
  CShipData    *pShip;
  POSITION     pos;
  LONG         iItem;

  // some basic checks
  if (pConfiguration == NULL) return TRUE;

  // set up the caption
  CStr = EDIT_CAPTION;
  CStr = CStr + *(pConfiguration->GetConfigName());
  CStr = CStr + GENERAL_CAPTION;
  SendMessage(WM_SETTEXT,0,(LONG)(const char *)CStr);

  // copy the playership to the list box
  if ((pConfiguration->GetPlayership())->IsSetShipClass())
  {
    // format the class and optional name
    FormatShipClassAndName(&CStr, pConfiguration->GetPlayership());
    // put it to the box
    SendDlgItemMessage(IDC_PLAYERSHIP, LB_ADDSTRING,0,(LONG)(const char *)CStr);
    // enable the next button
    GetDlgItem(IDC_GNEXT)->EnableWindow(TRUE);
  }

  // copy the escort to the list box
  if ((pConfiguration->GetEscort())->GetCount() > 0)
  {
    // iterate through all escorts
    for (pos = (pConfiguration->GetEscort())->GetHeadPosition(); pos != NULL; )
    {
      // get ship class and name
      pShip = (CShipData *)(pConfiguration->GetEscort())->GetNext(pos);
      // format the class and optional name
      FormatShipClassAndName(&CStr, pShip);
      SendDlgItemMessage(IDC_ESCORTS, LB_ADDSTRING,0,(LONG)(const char *)CStr);
    }
  }

  // create the available ship list
  AvailableShips.FillListBox(this, IDC_GAVSHIPS);

  // check if we have a selection in the general avaibale ship list box,
  // enable select and add button if so
  iItem = SendDlgItemMessage(IDC_GAVSHIPS, LB_GETCURSEL,0,0);
  if (iItem != LB_ERR)
  {
    GetDlgItem(IDC_GSELECT)->EnableWindow(TRUE);
    GetDlgItem(IDC_GADD)->EnableWindow(TRUE);
  }

  // check if we have a selection in the escort list box,
  // enable remove button if so
  iItem = SendDlgItemMessage(IDC_ESCORTS, LB_GETCURSEL,0,0);
  if (iItem != LB_ERR)
  {
    GetDlgItem(IDC_GREMOVE)->EnableWindow(TRUE);
  }

  // check if we have at least one system,
  // enable the finish button if so
  if ((pConfiguration->GetSystems())->GetCount()>0) GetDlgItem(IDC_GFINISH)->EnableWindow(TRUE);

  return TRUE;
}

//-----------------------------------
// setup data to edit
//-----------------------------------
void CGeneralDialog::Setup(CConfiguration *Configuration)
{
  // assign the pointer to the configuration
  pConfiguration = Configuration;
}

//-----------------------------------
// overriding the OnCancel method
//-----------------------------------
void CGeneralDialog::OnCancel()
{
  CCancelDialog CancelDia(this);

  // make sure the user wants to cancel
  CancelDia.DoModal();
  if (CancelDia.GetResult())
  {  
    // set the result
    eDialogResult = CancelDialog;
    // quit
    EndDialog(IDOK);
  }
}
 

//-----------------------------------
// new OnFinishButton method
//-----------------------------------
void CGeneralDialog::OnFinishButton()
{
  // do some tests before finishing
  if (!FinishTest(pConfiguration)) return;
  // set the result
  eDialogResult = FinishDialog;
  // quit
  EndDialog(IDOK);
}

//-----------------------------------
// new OnNextButton method
//-----------------------------------
void CGeneralDialog::OnNextButton()
{
  // set the result
  eDialogResult = NextDialog;
  // quit
  EndDialog(IDOK);
}

//-----------------------------------
// new OnSelectButton method
//-----------------------------------
void CGeneralDialog::OnSelectButton()
{
  LONG    iItem;
  CString ShipClass, CStr;

  // select a new playership
  iItem = SendDlgItemMessage(IDC_GAVSHIPS, LB_GETCURSEL,0,0);
  if (iItem != LB_ERR)
  {
    // get the ship
    if (AvailableShips.FindShipClass(iItem, &ShipClass))
    {
      // set the data    
      (pConfiguration->GetPlayership())->SetShipClass(ShipClass);
      // update the list box
      SendDlgItemMessage(IDC_PLAYERSHIP, LB_DELETESTRING,0,(LONG)0);
      FormatShipClassAndName(&CStr, pConfiguration->GetPlayership());
      SendDlgItemMessage(IDC_PLAYERSHIP, LB_ADDSTRING,0,(LONG)(const char *)CStr);
      // enable the next button
      GetDlgItem(IDC_GNEXT)->EnableWindow(TRUE);
    }
  }
}

//-----------------------------------
// new OnAddButton method
//-----------------------------------
void CGeneralDialog::OnAddButton()
{
  LONG         iItem;
  CString      ShipClass, CStr;
  CShipData    *pShip;
  CErrorDialog ErrorDialog(this);

  // add a new escort
  iItem = SendDlgItemMessage(IDC_GAVSHIPS, LB_GETCURSEL,0,0);
  if (iItem != LB_ERR)
  {
    // get the ship
    if (AvailableShips.FindShipClass(iItem, &ShipClass))
    {
      // create the new ship
      pShip = new CShipData;
      if (pShip == NULL)
      {
        ErrorDialog.Report(ERR_OUTOFMEMORY, ERR_HEADER);
        return;
      }
      // set the data    
      pShip->SetShipClass(ShipClass);
      // put it to the escort list
      (pConfiguration->GetEscort())->AddTail(pShip);
      // update the list box
      FormatShipClassAndName(&CStr, pShip);
      SendDlgItemMessage(IDC_ESCORTS, LB_ADDSTRING,0,(LONG)(const char *)CStr);
    }
  }
  
}

//-----------------------------------
// new OnRemoveButton method
//-----------------------------------
void CGeneralDialog::OnRemoveButton()
{
  LONG         iItem;
  CShipData    *pShip;
  POSITION     pos;

  // remove an escort
  iItem = SendDlgItemMessage(IDC_ESCORTS, LB_GETCURSEL,0,0);
  if (iItem != LB_ERR)
  {
    // find the ship
    pos = (pConfiguration->GetEscort())->FindIndex(iItem);
    if (pos != NULL)
    {
      // delete the ship
      pShip = (CShipData *)(pConfiguration->GetEscort())->GetAt(pos);
      delete pShip;
      (pConfiguration->GetEscort())->RemoveAt(pos);
      // update the list box
      SendDlgItemMessage(IDC_ESCORTS, LB_DELETESTRING, (WORD)iItem,(LONG)0);
      // set the new selection
      if (SendDlgItemMessage(IDC_ESCORTS, LB_SETCURSEL,(UINT)iItem,0) == LB_ERR)
      {
        // we cannot select this one, maybe end of the list. So lets try the previous one
        if (iItem > 0) iItem--;
        if (SendDlgItemMessage(IDC_ESCORTS, LB_SETCURSEL,(UINT)iItem,0) == LB_ERR)
        {
          // no selection possible, so disable the remove button
          GetDlgItem(IDC_GREMOVE)->EnableWindow(FALSE);
        }
      }
    }
  }
}


//-----------------------------------
// new OnOptionButton method
//-----------------------------------
void CGeneralDialog::OnOptionButton()
{
  CGeneralOptionsDialog GeneralOptionsDialog;

  // do the dialog
  GeneralOptionsDialog.Setup(pConfiguration);
  GeneralOptionsDialog.DoDialog();
}


//-----------------------------------
// do the dialog
//-----------------------------------
EDialogResultType CGeneralDialog::DoDialog()
{
  // do it
  DoModal();
  // return result
  return eDialogResult;
}


//-----------------------------------
// new OnGavSelChange method
//-----------------------------------
void CGeneralDialog::OnGavSelChange()
{
  LONG iItem;

  // selection in the general available ship listbox has changed

  // check if we have a selection
  iItem = SendDlgItemMessage(IDC_GAVSHIPS, LB_GETCURSEL,0,0);
  if (iItem != LB_ERR)
  {
    // unlock the select and add button
    GetDlgItem(IDC_GSELECT)->EnableWindow(TRUE);
    GetDlgItem(IDC_GADD)->EnableWindow(TRUE);
  } else
  {
    // lock the select and add button
    GetDlgItem(IDC_GSELECT)->EnableWindow(FALSE);
    GetDlgItem(IDC_GADD)->EnableWindow(FALSE);
  }

}


//-----------------------------------
// new OnEscortSelChange method
//-----------------------------------
void CGeneralDialog::OnEscortSelChange()
{
  LONG iItem;

  // selection in the escort listbox has changed

  // check if we have a selection
  iItem = SendDlgItemMessage(IDC_ESCORTS, LB_GETCURSEL,0,0);
  if (iItem != LB_ERR)
  {
    // unlock the remove button
    GetDlgItem(IDC_GREMOVE)->EnableWindow(TRUE);
  } else
  {
    // lock the remove button
    GetDlgItem(IDC_GREMOVE)->EnableWindow(FALSE);
  }
}


//-----------------------------------
// new OnEscortDoubleClick method
//-----------------------------------
void CGeneralDialog::OnEscortDoubleClick()
{
  CString         CStr;
  LONG            iItem;
  POSITION        pos;
  CShipData       *pShip;

  // get the selected escort ship
  iItem = SendDlgItemMessage(IDC_ESCORTS, LB_GETCURSEL,0,0);
  if (iItem != LB_ERR)
  {
    // find the ship
    pos = (pConfiguration->GetEscort())->FindIndex(iItem);
    if (pos != NULL)
    {
      // ask for the ship name
      pShip = (CShipData *)(pConfiguration->GetEscort())->GetAt(pos);
      if (AskForShipName(pShip, pConfiguration, ESCORTNAME_TEXT))
      {
        // update the list box
        FormatShipClassAndName(&CStr, pShip);
        SendDlgItemMessage(IDC_ESCORTS, LB_DELETESTRING, (WORD)iItem,(LONG)0);
        SendDlgItemMessage(IDC_ESCORTS, LB_INSERTSTRING, (WORD)iItem,(LONG)(const char *)CStr);
        // set the selection
        SendDlgItemMessage(IDC_ESCORTS, LB_SETCURSEL, (WORD)iItem,(LONG)0);
      }
    }
  }
}


//-----------------------------------
// new OnPlayerDoubleClick method
//-----------------------------------
void CGeneralDialog::OnPlayerDoubleClick()
{
  CString         CStr;
  CShipData       *pShip;
  LONG            iItem;

  // do we have a selection
  iItem = SendDlgItemMessage(IDC_PLAYERSHIP, LB_GETCURSEL,0,0);
  if (iItem != LB_ERR)
  {
    // ask for the new playership name
    pShip = (CShipData *)pConfiguration->GetPlayership();
    if (AskForShipName(pShip, pConfiguration, PLAYERNAME_TEXT))
    {
      // update the list box
      FormatShipClassAndName(&CStr, pShip);
      SendDlgItemMessage(IDC_PLAYERSHIP, LB_DELETESTRING, (WORD)0,(LONG)0);
      SendDlgItemMessage(IDC_PLAYERSHIP, LB_INSERTSTRING, (WORD)0,(LONG)(const char *)CStr);
      // set the selection
      SendDlgItemMessage(IDC_PLAYERSHIP, LB_SETCURSEL, (WORD)0,(LONG)0);
    }
  }
}


//-----------------------------------
// the message map
//-----------------------------------
BEGIN_MESSAGE_MAP(CGeneralDialog, CModalDialog)
  ON_BN_CLICKED(IDC_GFINISH,      OnFinishButton)
  ON_BN_CLICKED(IDC_GNEXT,        OnNextButton)
  ON_BN_CLICKED(IDC_GSELECT,      OnSelectButton)
  ON_BN_CLICKED(IDC_GADD,         OnAddButton)
  ON_BN_CLICKED(IDC_GREMOVE,      OnRemoveButton)
  ON_BN_CLICKED(IDC_GOPTIONS,     OnOptionButton)
  ON_LBN_SELCHANGE(IDC_GAVSHIPS,  OnGavSelChange)
  ON_LBN_SELCHANGE(IDC_ESCORTS,   OnEscortSelChange)
  ON_LBN_DBLCLK(IDC_PLAYERSHIP,   OnPlayerDoubleClick)
  ON_LBN_DBLCLK(IDC_ESCORTS,      OnEscortDoubleClick)
END_MESSAGE_MAP()




// ----------- the system dialog class ---------------------------------------

//-----------------------------------
// overriding the Constructor
//-----------------------------------
CSystemDialog::CSystemDialog(CWnd* pParentWnd) : CModalDialog("SYSTEMBOX", pParentWnd)
{
  // init
  pConfiguration = NULL;
  uSystemToEdit = 0;
  eDialogResult = CancelDialog;
  lastSpinPosition = -1;
}

//-----------------------------------
// overriding the OnInitDialog method
//-----------------------------------
BOOL CSystemDialog::OnInitDialog()
{

  CString      CStr;
  CSystemData  *pSystem;
  POSITION     pos;
  LONG         iItem, iItem1;

  // some basic checks
  if (pConfiguration == NULL) return TRUE;

  // set up the caption
  CStr = EDIT_CAPTION;
  CStr = CStr + *(pConfiguration->GetConfigName());
  CStr = CStr + SYSTEMS_CAPTION;
  SendMessage(WM_SETTEXT,0,(LONG)(const char *)CStr);

  // set the tab stops for the systems list box
  ((CListBox *)GetDlgItem(IDC_SYSTEMS))->SetTabStops(60);

  // copy the systems to the list box
  if ((pConfiguration->GetSystems())->GetCount() > 0)
  {
    // iterate through all systems
    for (pos = (pConfiguration->GetSystems())->GetHeadPosition(); pos != NULL; )
    {
      // get system
      pSystem = (CSystemData *)(pConfiguration->GetSystems())->GetNext(pos);
      // format and output name
      FormatSystemName(&CStr, pSystem);
      SendDlgItemMessage(IDC_SYSTEMS, LB_ADDSTRING,0,(LONG)(const char *)CStr);
    }
  }

  // create the available systems list
  AvailableSystemsAndPlanets.FillListBoxWithSystems(this, IDC_SAVSYS);

  // check if we have a selection in the general avaibale systems list box and less then max systems,
  // enable add button if so
  iItem = SendDlgItemMessage(IDC_SAVSYS, LB_GETCURSEL,0,0);
  if ((iItem != LB_ERR) && ((pConfiguration->GetSystems())->GetCount() < MAX_SYSTEMS))
  {
    GetDlgItem(IDC_SADD)->EnableWindow(TRUE);
  }

  // check if we have a selection in the systems list box,
  // enable remove, edit (= next), up and down button if so
  iItem1 = SendDlgItemMessage(IDC_SYSTEMS, LB_GETCURSEL,0,0);
  if (iItem1 != LB_ERR)
  {
    GetDlgItem(IDC_SREMOVE)->EnableWindow(TRUE);
    GetDlgItem(IDC_SEDIT)->EnableWindow(TRUE);
    GetDlgItem(IDC_SNEXT)->EnableWindow(TRUE);
    GetDlgItem(IDC_SUPDOWNSYSTEM)->EnableWindow(TRUE);
    GetDlgItem(IDC_SUPDOWNSPIN)->EnableWindow(TRUE);
  }

  // check if we have a selection in the systems and the general available systems list box
  // enable replace button if so
  if ((iItem != LB_ERR) && (iItem1 != LB_ERR))
  {
    GetDlgItem(IDC_SREPLACE)->EnableWindow(TRUE);
  }

  // check if we have at least one system,
  // enable the finish button if so
  if ((pConfiguration->GetSystems())->GetCount()>0) GetDlgItem(IDC_SFINISH)->EnableWindow(TRUE);

  return TRUE;
}

//-----------------------------------
// new UpSystem method
//-----------------------------------
void CSystemDialog::UpSystem()
{
  // move the selected system one position up
  CSystemData  *pSystem1, *pSystem2;
  POSITION     pos1, pos2;
  LONG         iItem;
  char         line[LINE_LENGTH];

  // get the selected battle
  iItem = SendDlgItemMessage(IDC_SYSTEMS, LB_GETCURSEL,0,0);
  if ((iItem != LB_ERR) && (iItem > 0))
  {
    // find the system
    pos1 = (pConfiguration->GetSystems())->FindIndex(iItem);
    if (pos1 != NULL)
    {
      // get the previous one
      pos2 = (pConfiguration->GetSystems())->FindIndex(iItem - 1);
      // exchange 
      pSystem1 = (CSystemData *)(pConfiguration->GetSystems())->GetAt(pos1);
      pSystem2 = (CSystemData *)(pConfiguration->GetSystems())->GetAt(pos2);
      (pConfiguration->GetSystems())->SetAt(pos2, pSystem1);
      (pConfiguration->GetSystems())->SetAt(pos1, pSystem2);

      // update the list box
      SendDlgItemMessage(IDC_SYSTEMS, LB_GETTEXT,(UINT)iItem,(LONG)(LPSTR)line);
      // delete the system on its old position
      SendDlgItemMessage(IDC_SYSTEMS, LB_DELETESTRING,(UINT)iItem,0);
      // put it to his new position
      SendDlgItemMessage(IDC_SYSTEMS, LB_INSERTSTRING,(UINT)iItem - 1,(LONG)(LPSTR)line);
      // and select it again
      SendDlgItemMessage(IDC_SYSTEMS, LB_SETCURSEL,(UINT)iItem - 1,0);
    }
  } 
}

//-----------------------------------
// new DownSystem method
//-----------------------------------
void CSystemDialog::DownSystem()
{
  // move the selected system one position down
  CSystemData  *pSystem1, *pSystem2;
  POSITION     pos1, pos2;
  LONG         iItem, iNumberOfItems;
  char         line[LINE_LENGTH];

  // get the selected battle
  iItem = SendDlgItemMessage(IDC_SYSTEMS, LB_GETCURSEL,0,0);
  // get the number of items in the box
  iNumberOfItems = SendDlgItemMessage(IDC_SYSTEMS, LB_GETCOUNT,0,0);
  if ((iItem != LB_ERR) && ((iItem + 1) < iNumberOfItems))
  {
    // find the system
    pos1 = (pConfiguration->GetSystems())->FindIndex(iItem);
    if (pos1 != NULL)
    {
      // get the next one
      pos2 = (pConfiguration->GetSystems())->FindIndex(iItem + 1);
      // exchange 
      pSystem1 = (CSystemData *)(pConfiguration->GetSystems())->GetAt(pos1);
      pSystem2 = (CSystemData *)(pConfiguration->GetSystems())->GetAt(pos2);
      (pConfiguration->GetSystems())->SetAt(pos2, pSystem1);
      (pConfiguration->GetSystems())->SetAt(pos1, pSystem2);

      // update the list box
      SendDlgItemMessage(IDC_SYSTEMS, LB_GETTEXT,(UINT)iItem,(LONG)(LPSTR)line);
      // delete the system on its old position
      SendDlgItemMessage(IDC_SYSTEMS, LB_DELETESTRING,(UINT)iItem,0);
      // put it to his new position
      SendDlgItemMessage(IDC_SYSTEMS, LB_INSERTSTRING,(UINT)iItem + 1,(LONG)(LPSTR)line);
      // and select it again
      SendDlgItemMessage(IDC_SYSTEMS, LB_SETCURSEL,(UINT)iItem + 1,0);
    }
  } 
}

//-----------------------------------
// setup data to edit
//-----------------------------------
void CSystemDialog::Setup(CConfiguration *Configuration)
{
  // assign the pointer to the configuration
  pConfiguration = Configuration;
}

//-----------------------------------
// overriding the OnCancel method
//-----------------------------------
void CSystemDialog::OnCancel()
{
  CCancelDialog CancelDia(this);

  // make sure the user wants to cancel
  CancelDia.DoModal();
  if (CancelDia.GetResult())
  {  
    // set the result
    eDialogResult = CancelDialog;
    // quit
    EndDialog(IDOK);
  }
}
 

//-----------------------------------
// new OnFinishButton method
//-----------------------------------
void CSystemDialog::OnFinishButton()
{
  // do some tests before finishing
  if (!FinishTest(pConfiguration)) return;
  // set the result
  eDialogResult = FinishDialog;
  // quit
  EndDialog(IDOK);
}

//-----------------------------------
// new OnPreviousButton method
//-----------------------------------
void CSystemDialog::OnPreviousButton()
{
  // set the result
  eDialogResult = PreviousDialog;
  // quit
  EndDialog(IDOK);
}

//-----------------------------------
// new OnAddButton method
//-----------------------------------
void CSystemDialog::OnAddButton()
{
  LONG         iItem;
  CString      SystemName, CStr;
  CSystemData  *pSystem;
  CErrorDialog ErrorDialog(this);

  // add a new system
  iItem = SendDlgItemMessage(IDC_SAVSYS, LB_GETCURSEL,0,0);
  if (iItem != LB_ERR)
  {
    // get the system
    if (AvailableSystemsAndPlanets.FindSystemName(iItem, &SystemName))
    {
      // system exists already ?
      if (pConfiguration->GetSystems()->ExistsSystem(SystemName))
      {
        ErrorDialog.Report(ERR_SYSTEMREDEF, ERR_HEADER);
        return;
      }

      // create the new system
      pSystem = new CSystemData;
      if (pSystem == NULL)
      {
        ErrorDialog.Report(ERR_OUTOFMEMORY, ERR_HEADER);
        return;
      }
      // set the data    
      pSystem->SetSystemName(SystemName);
      // put it to the systems list
      (pConfiguration->GetSystems())->AddTail(pSystem);
      // update the list box
      FormatSystemName(&CStr, pSystem);
      SendDlgItemMessage(IDC_SYSTEMS, LB_ADDSTRING,0,(LONG)(const char *)CStr);
      // enable Finish button
      GetDlgItem(IDC_SFINISH)->EnableWindow(TRUE);
      // disable Add button when max number of systems reached
      if ((pConfiguration->GetSystems())->GetCount() >= MAX_SYSTEMS)
        GetDlgItem(IDC_SADD)->EnableWindow(FALSE);
    }
  }
}


//-----------------------------------
// new OnReplaceButton method
//-----------------------------------
void CSystemDialog::OnReplaceButton()
{
  LONG         iItem, iItemToReplace;
  CString      SystemName, CStr;
  CSystemData  *pSystem;
  POSITION     pos;
  CErrorDialog ErrorDialog(this);

  // replace a system
  iItem = SendDlgItemMessage(IDC_SAVSYS, LB_GETCURSEL,0,0);
  iItemToReplace = SendDlgItemMessage(IDC_SYSTEMS, LB_GETCURSEL,0,0);
  if ((iItem != LB_ERR) && (iItemToReplace != LB_ERR))
  {
    // get the new system
    if (AvailableSystemsAndPlanets.FindSystemName(iItem, &SystemName))
    {
      // get the system to replace
      pos = (pConfiguration->GetSystems())->FindIndex(iItemToReplace);
      if (pos != NULL)
      {
        pSystem = (CSystemData *)(pConfiguration->GetSystems())->GetAt(pos);
        // check if it is not the same system
        if (SystemName.CompareNoCase(pSystem->GetSystemName()) != 0)
        {
          // new system exists already ?
          if (pConfiguration->GetSystems()->ExistsSystem(SystemName))
          {
            ErrorDialog.Report(ERR_SYSTEMREDEF, ERR_HEADER);
            return;
          }
          // confirm replacement
          CStr = pSystem->GetSystemName();
          CStr = CStr + SYS_REPLACEAPPTEXT1;
          CStr = CStr + SystemName;
          CStr = CStr + " ?";
          CReplaceDialog replaceDialog(NULL, CStr, SYS_REPLACEAPPTEXT2);
          replaceDialog.DoModal();
          if (replaceDialog.GetResult())
          {
            // replace the system
            pSystem->SetSystemName(SystemName);
            // delete all planetary position info, because this will be invalid in the new system
            pSystem->UnsetPlayerPosPlanet();
            pSystem->UnsetFriendPosPlanet();
            pSystem->UnsetEnemyPosPlanet();
            pSystem->UnsetNeutralPosPlanet();
            pSystem->UnsetHulkPosPlanet();
            pSystem->UnsetJumpGatePosPlanet();
            // update the list box
            SendDlgItemMessage(IDC_SYSTEMS, LB_DELETESTRING,(WORD)iItemToReplace,(LONG)0);
            FormatSystemName(&CStr, pSystem);
            SendDlgItemMessage(IDC_SYSTEMS, LB_INSERTSTRING,(WORD)iItemToReplace,(LONG)(const char *)CStr);
            // set the selection
            SendDlgItemMessage(IDC_SYSTEMS, LB_SETCURSEL, (WORD)iItemToReplace,(LONG)0);
          }
        }
      }
    }
  }
}


//-----------------------------------
// new OnRemoveButton method
//-----------------------------------
void CSystemDialog::OnRemoveButton()
{
  LONG         iItem;
  CSystemData  *pSystem;
  POSITION     pos;
  CString      CStr;

  // remove a system
  iItem = SendDlgItemMessage(IDC_SYSTEMS, LB_GETCURSEL,0,0);
  if (iItem != LB_ERR)
  {
    // find the system
    pos = (pConfiguration->GetSystems())->FindIndex(iItem);
    if (pos != NULL)
    {
      // confirm deletion
      pSystem = (CSystemData *)(pConfiguration->GetSystems())->GetAt(pos);
      CStr = pSystem->GetSystemName();
      CStr = CStr + SYS_DELAPPTEXT;
      CDeleteDialog deleteDialog(NULL, CStr);
      deleteDialog.DoModal();
      if (deleteDialog.GetResult())
      {
        // we delete it
        delete pSystem;
        (pConfiguration->GetSystems())->RemoveAt(pos);
        // update the list box
        SendDlgItemMessage(IDC_SYSTEMS, LB_DELETESTRING, (WORD)iItem,(LONG)0);
        // set the new selection
        if (SendDlgItemMessage(IDC_SYSTEMS, LB_SETCURSEL,(UINT)iItem,0) == LB_ERR)
        {
          // we cannot select this one, maybe end of the list. So lets try the previous one
          if (iItem > 0) iItem--;
          if (SendDlgItemMessage(IDC_SYSTEMS, LB_SETCURSEL,(UINT)iItem,0) == LB_ERR)
          {
            // no selection possible, so disable the remove, replace, edit, next, up, down button (spin & edit field)
            GetDlgItem(IDC_SREMOVE)->EnableWindow(FALSE);
            GetDlgItem(IDC_SREPLACE)->EnableWindow(FALSE);
            GetDlgItem(IDC_SEDIT)->EnableWindow(FALSE);
            GetDlgItem(IDC_SNEXT)->EnableWindow(FALSE);
            GetDlgItem(IDC_SUPDOWNSYSTEM)->EnableWindow(FALSE);
            GetDlgItem(IDC_SUPDOWNSPIN)->EnableWindow(FALSE);
          }
        }

        // check if we have a selection in the general avaibale systems list box and less then max systems,
        // enable add button if so
        iItem = SendDlgItemMessage(IDC_SAVSYS, LB_GETCURSEL,0,0);
        if ((iItem != LB_ERR) && ((pConfiguration->GetSystems())->GetCount() < MAX_SYSTEMS))
        {
          GetDlgItem(IDC_SADD)->EnableWindow(TRUE);
        }
        // check if we have at least one system,
        // disable the finish button if not
        if ((pConfiguration->GetSystems())->GetCount() == 0) GetDlgItem(IDC_SFINISH)->EnableWindow(FALSE);
      }
    }
  } 
}

//-----------------------------------
// new OnEditButton method
//-----------------------------------
void CSystemDialog::OnEditButton()
{
  LONG         iItem;

   // edit a system
  iItem = SendDlgItemMessage(IDC_SYSTEMS, LB_GETCURSEL,0,0);
  if (iItem != LB_ERR)
  {
    // set the system to edit
    uSystemToEdit = (unsigned)(iItem + 1);
    // set the result (edit is done in the next dialog)
    eDialogResult = NextDialog;
    // quit
    EndDialog(IDOK);
  }
}


//-----------------------------------
// new OnUpDownButton method
//-----------------------------------
void CSystemDialog::OnUpDownButton()
{
  char str[MAX_QB2_LINELENGTH+1];
  long actSpinPos;

  // Moves the selected system one position up or down in the list box
  // Uses UpSystem and DownSystem for this purpose
  //
  // Same tricky spin button solution as for battle up and down in the start dialog

  // init the last spin position ? (first event when setting the initial value in the hidden edit box)
  if (lastSpinPosition == -1) 
  {
    lastSpinPosition = 1;
  } else
  {
    // we have an event in the hidden edit box related to the spin up/down buttons
    // find out if the value has increased or decreased, that's all we need to know
    SendDlgItemMessage(IDC_SUPDOWNSYSTEM, WM_GETTEXT,(WORD)MAX_QB2_LINELENGTH,(LONG)(LPSTR)str);
    actSpinPos = atol(str);
    // increased means DOWN
    if (actSpinPos > lastSpinPosition) DownSystem(); else
    // decreased means UP
    if (actSpinPos < lastSpinPosition) UpSystem();
    // since we are only interested in relative position, set the the hidden edit box back to its initial value
    if (atol(str) != lastSpinPosition)
    {
      // note: we only do this if the value has changed. Otherwise we would cause an endless event series, because
      //       changing the value causes again an event a.s.o. So for the second event the value has not changed an
      //       we can stop this series. 
      lastSpinPosition = 1;
      sprintf(str,"%lu",lastSpinPosition);
      SendDlgItemMessage(IDC_SUPDOWNSYSTEM, WM_SETTEXT,(WORD)0,(LONG)(LPSTR)str);
    }
  }
}


//-----------------------------------
// new OnCheckButton method
//-----------------------------------
void CSystemDialog::OnCheckButton()
{
  CCheckConfigDialog CheckConfigDialog;

  // check the configuration for missing objects
  CheckConfigDialog.Setup(pConfiguration, &AvailableShips, &AvailableSystemsAndPlanets, NULL);
  CheckConfigDialog.DoDialog();
}


//-----------------------------------
// do the dialog
//-----------------------------------
EDialogResultType CSystemDialog::DoDialog(unsigned *uSystem)
{
  // do it
  DoModal();
  // return result
  *uSystem = uSystemToEdit;
  return eDialogResult;
}


//-----------------------------------
// new OnSavSelChange method
//-----------------------------------
void CSystemDialog::OnSavSelChange()
{
  LONG iItem, iItem1;

  // selection in the general available systems listbox has changed

  // check if we have a selection and less then max systems
  iItem = SendDlgItemMessage(IDC_SAVSYS, LB_GETCURSEL,0,0);
  if ((iItem != LB_ERR) && ((pConfiguration->GetSystems())->GetCount() < MAX_SYSTEMS))
  {
    // unlock the add button
    GetDlgItem(IDC_SADD)->EnableWindow(TRUE);
  } else
  {
    // lock the add button
    GetDlgItem(IDC_SADD)->EnableWindow(FALSE);
  }

  // check if we have a selection in the systems listbox as well
  iItem1 = SendDlgItemMessage(IDC_SYSTEMS, LB_GETCURSEL,0,0);
  // enable replace button if selection in both list boxes
  if ((iItem != LB_ERR) && (iItem1 != LB_ERR))
  {
    GetDlgItem(IDC_SREPLACE)->EnableWindow(TRUE);
  } else
  {
    GetDlgItem(IDC_SREPLACE)->EnableWindow(FALSE);
  }

}


//-----------------------------------
// new OnSystemSelChange method
//-----------------------------------
void CSystemDialog::OnSystemSelChange()
{
  LONG iItem, iItem1;

  // selection in the system listbox has changed

  // check if we have a selection
  iItem = SendDlgItemMessage(IDC_SYSTEMS, LB_GETCURSEL,0,0);
  if (iItem != LB_ERR)
  {
    // enable remove, edit, up and down button if so
    GetDlgItem(IDC_SREMOVE)->EnableWindow(TRUE);
    GetDlgItem(IDC_SEDIT)->EnableWindow(TRUE);
    GetDlgItem(IDC_SNEXT)->EnableWindow(TRUE);
    GetDlgItem(IDC_SUPDOWNSYSTEM)->EnableWindow(TRUE);
    GetDlgItem(IDC_SUPDOWNSPIN)->EnableWindow(TRUE);
  } else
  {
    // disable remove, edit, up and down button if so
    GetDlgItem(IDC_SREMOVE)->EnableWindow(FALSE);
    GetDlgItem(IDC_SEDIT)->EnableWindow(FALSE);
    GetDlgItem(IDC_SNEXT)->EnableWindow(FALSE);
    GetDlgItem(IDC_SUPDOWNSYSTEM)->EnableWindow(FALSE);
    GetDlgItem(IDC_SUPDOWNSPIN)->EnableWindow(FALSE);
  } 

  // check if we have a selection in the general availabe systems listbox as well
  iItem1 = SendDlgItemMessage(IDC_SAVSYS, LB_GETCURSEL,0,0);
  // enable replace button if selection in both list boxes
  if ((iItem != LB_ERR) && (iItem1 != LB_ERR))
  {
    GetDlgItem(IDC_SREPLACE)->EnableWindow(TRUE);
  } else
  {
    GetDlgItem(IDC_SREPLACE)->EnableWindow(FALSE);
  }

}




//-----------------------------------
// the message map
//-----------------------------------
BEGIN_MESSAGE_MAP(CSystemDialog, CModalDialog)
  ON_BN_CLICKED(IDC_SFINISH,      OnFinishButton)
  ON_BN_CLICKED(IDC_SPREV,        OnPreviousButton)
  ON_BN_CLICKED(IDC_SNEXT,        OnEditButton)
  ON_BN_CLICKED(IDC_SADD,         OnAddButton)
  ON_BN_CLICKED(IDC_SREPLACE,     OnReplaceButton)
  ON_BN_CLICKED(IDC_SREMOVE,      OnRemoveButton)
  ON_BN_CLICKED(IDC_SEDIT,        OnEditButton)
  ON_BN_CLICKED(IDC_SCHECK,       OnCheckButton)
  ON_EN_UPDATE(IDC_SUPDOWNSYSTEM, OnUpDownButton)
  ON_LBN_SELCHANGE(IDC_SAVSYS,    OnSavSelChange)
  ON_LBN_SELCHANGE(IDC_SYSTEMS,   OnSystemSelChange)
  ON_LBN_DBLCLK(IDC_SYSTEMS,      OnEditButton)
END_MESSAGE_MAP()




// ----------- the system friend enemy dialog class ---------------------------------------

//-----------------------------------
// overriding the Constructor
//-----------------------------------
CSystemFriendEnemyDialog::CSystemFriendEnemyDialog(CWnd* pParentWnd) : CModalDialog("SYSTEMBOX1", pParentWnd)
{
  // init
  pConfiguration = NULL;
  pSystem = NULL;
  uSystemNum = 0;
  eDialogResult = CancelDialog;
}

//-----------------------------------
// overriding the OnInitDialog method
//-----------------------------------
BOOL CSystemFriendEnemyDialog::OnInitDialog()
{

  CString      CStr;
  CShipData    *pShip;
  POSITION     pos;
  LONG         iItem;
  char         str[20];

  // some basic checks
  if (pConfiguration == NULL) return TRUE;
  if (uSystemNum == 0) return TRUE;
  if (pSystem == NULL) return TRUE;

  // set up the caption
  

  CStr = EDIT_CAPTION;
  CStr = CStr + *(pConfiguration->GetConfigName());
  CStr = CStr + SYSTEM_CAPTION;
  sprintf(str,"%u",uSystemNum);
  CStr = CStr + str;
  CStr = CStr + SYSTEM_CAPTION1;
  sprintf(str,"%u",(pConfiguration->GetSystems())->GetCount());
  CStr = CStr + str;
  CStr = CStr + SYSTEM_CAPTION2;
  CStr = CStr + pSystem->GetSystemName();
  CStr = CStr + SYSTEM1_CAPTION;
  SendMessage(WM_SETTEXT,0,(LONG)(const char *)CStr);

  // copy the friends to the list box
  if ((pSystem->GetFriends())->GetCount() > 0)
  {
    // iterate through all friends
    for (pos = (pSystem->GetFriends())->GetHeadPosition(); pos != NULL; )
    {
      // get ship class and name
      pShip = (CShipData *)(pSystem->GetFriends())->GetNext(pos);
      // format the class and optional name
      FormatShipClassAndName(&CStr, pShip);
      SendDlgItemMessage(IDC_FRIENDS, LB_ADDSTRING,0,(LONG)(const char *)CStr);
    }
  }

  // copy the enemies to the list box
  if ((pSystem->GetEnemies())->GetCount() > 0)
  {
    // iterate through all enemius
    for (pos = (pSystem->GetEnemies())->GetHeadPosition(); pos != NULL; )
    {
      // get ship class and name
      pShip = (CShipData *)(pSystem->GetEnemies())->GetNext(pos);
      // format the class and optional name
      FormatShipClassAndName(&CStr, pShip);
      SendDlgItemMessage(IDC_ENEMIES, LB_ADDSTRING,0,(LONG)(const char *)CStr);
    }
  }

  // create the available ship list
  AvailableShips.FillListBox(this, IDC_S1AVSHIPS);

  // check if we have a selection in the general avaibale ship list box,
  // enable add buttons if so
  iItem = SendDlgItemMessage(IDC_S1AVSHIPS, LB_GETCURSEL,0,0);
  if (iItem != LB_ERR)
  {
    GetDlgItem(IDC_S1FRIENDADD)->EnableWindow(TRUE);
    GetDlgItem(IDC_S1ENEMYADD)->EnableWindow(TRUE);
  }

  // check if we have a selection in the friend list box,
  // enable remove button if so
  iItem = SendDlgItemMessage(IDC_FRIENDS, LB_GETCURSEL,0,0);
  if (iItem != LB_ERR)
  {
    GetDlgItem(IDC_S1FRIENDREMOVE)->EnableWindow(TRUE);
  }

  // check if we have a selection in the enemy list box,
  // enable remove button if so
  iItem = SendDlgItemMessage(IDC_ENEMIES, LB_GETCURSEL,0,0);
  if (iItem != LB_ERR)
  {
    GetDlgItem(IDC_S1ENEMYREMOVE)->EnableWindow(TRUE);
  }

  return TRUE;
}

//-----------------------------------
// setup data to edit
//-----------------------------------
void CSystemFriendEnemyDialog::Setup(CConfiguration *Configuration, unsigned SystemNum)
{
  POSITION pos;

  // assign the pointer to the configuration
  pConfiguration = Configuration;
  // assign the system number
  uSystemNum = SystemNum;
  // get the system
  pos = (pConfiguration->GetSystems())->FindIndex(uSystemNum - 1);
  if (pos != NULL) pSystem = (CSystemData *)(pConfiguration->GetSystems())->GetAt(pos);
}

//-----------------------------------
// overriding the OnCancel method
//-----------------------------------
void CSystemFriendEnemyDialog::OnCancel()
{
  CCancelDialog CancelDia(this);

  // make sure the user wants to cancel
  CancelDia.DoModal();
  if (CancelDia.GetResult())
  {  
    // set the result
    eDialogResult = CancelDialog;
    // quit
    EndDialog(IDOK);
  }
}
 

//-----------------------------------
// new OnFinishButton method
//-----------------------------------
void CSystemFriendEnemyDialog::OnFinishButton()
{
  // do some tests before finishing
  if (!FinishTest(pConfiguration)) return;
  // set the result
  eDialogResult = FinishDialog;
  // quit
  EndDialog(IDOK);
}

//-----------------------------------
// new OnNextButton method
//-----------------------------------
void CSystemFriendEnemyDialog::OnNextButton()
{
  // set the result
  eDialogResult = NextDialog;
  // quit
  EndDialog(IDOK);
}

//-----------------------------------
// new OnPreviousButton method
//-----------------------------------
void CSystemFriendEnemyDialog::OnPreviousButton()
{
  // set the result
  eDialogResult = PreviousDialog;
  // quit
  EndDialog(IDOK);
}

//-----------------------------------
// new OnAddFriendButton method
//-----------------------------------
void CSystemFriendEnemyDialog::OnAddFriendButton()
{
  LONG         iItem;
  CString      ShipClass, CStr;
  CShipData    *pShip;
  CErrorDialog ErrorDialog(this);

  // add a new friend
  iItem = SendDlgItemMessage(IDC_S1AVSHIPS, LB_GETCURSEL,0,0);
  if (iItem != LB_ERR)
  {
    // get the ship
    if (AvailableShips.FindShipClass(iItem, &ShipClass))
    {
      // create the new ship
      pShip = new CShipData;
      if (pShip == NULL)
      {
        ErrorDialog.Report(ERR_OUTOFMEMORY, ERR_HEADER);
        return;
      }
      // set the data    
      pShip->SetShipClass(ShipClass);
      // put it to the escort list
      (pSystem->GetFriends())->AddTail(pShip);
      // update the list box
      FormatShipClassAndName(&CStr, pShip);
      SendDlgItemMessage(IDC_FRIENDS, LB_ADDSTRING,0,(LONG)(const char *)CStr);
    }
  }
  
}

//-----------------------------------
// new OnRemoveFriendButton method
//-----------------------------------
void CSystemFriendEnemyDialog::OnRemoveFriendButton()
{
  LONG         iItem;
  CShipData    *pShip;
  POSITION     pos;

  // remove an escort
  iItem = SendDlgItemMessage(IDC_FRIENDS, LB_GETCURSEL,0,0);
  if (iItem != LB_ERR)
  {
    // find the ship
    pos = (pSystem->GetFriends())->FindIndex(iItem);
    if (pos != NULL)
    {
      // delete the ship
      pShip = (CShipData *)(pSystem->GetFriends())->GetAt(pos);
      delete pShip;
      (pSystem->GetFriends())->RemoveAt(pos);
      // update the list box
      SendDlgItemMessage(IDC_FRIENDS, LB_DELETESTRING, (WORD)iItem,(LONG)0);
      // set the new selection
      if (SendDlgItemMessage(IDC_FRIENDS, LB_SETCURSEL,(UINT)iItem,0) == LB_ERR)
      {
        // we cannot select this one, maybe end of the list. So lets try the previous one
        if (iItem > 0) iItem--;
        if (SendDlgItemMessage(IDC_FRIENDS, LB_SETCURSEL,(UINT)iItem,0) == LB_ERR)
        {
          // no selection possible, so disable the remove button
          GetDlgItem(IDC_S1FRIENDREMOVE)->EnableWindow(FALSE);
        }
      }
    }
  }
}

//-----------------------------------
// new OnAddEnemyButton method
//-----------------------------------
void CSystemFriendEnemyDialog::OnAddEnemyButton()
{
  LONG         iItem;
  CString      ShipClass, CStr;
  CShipData    *pShip;
  CErrorDialog ErrorDialog(this);

  // add a new enemy
  iItem = SendDlgItemMessage(IDC_S1AVSHIPS, LB_GETCURSEL,0,0);
  if (iItem != LB_ERR)
  {
    // get the ship
    if (AvailableShips.FindShipClass(iItem, &ShipClass))
    {
      // create the new ship
      pShip = new CShipData;
      if (pShip == NULL)
      {
        ErrorDialog.Report(ERR_OUTOFMEMORY, ERR_HEADER);
        return;
      }
      // set the data    
      pShip->SetShipClass(ShipClass);
      // put it to the escort list
      (pSystem->GetEnemies())->AddTail(pShip);
      // update the list box
      FormatShipClassAndName(&CStr, pShip);
      SendDlgItemMessage(IDC_ENEMIES, LB_ADDSTRING,0,(LONG)(const char *)CStr);
    }
  }
  
}

//-----------------------------------
// new OnRemoveEnemyButton method
//-----------------------------------
void CSystemFriendEnemyDialog::OnRemoveEnemyButton()
{
  LONG         iItem;
  CShipData    *pShip;
  POSITION     pos;

  // remove an escort
  iItem = SendDlgItemMessage(IDC_ENEMIES, LB_GETCURSEL,0,0);
  if (iItem != LB_ERR)
  {
    // find the ship
    pos = (pSystem->GetEnemies())->FindIndex(iItem);
    if (pos != NULL)
    {
      // delete the ship
      pShip = (CShipData *)(pSystem->GetEnemies())->GetAt(pos);
      delete pShip;
      (pSystem->GetEnemies())->RemoveAt(pos);
      // update the list box
      SendDlgItemMessage(IDC_ENEMIES, LB_DELETESTRING, (WORD)iItem,(LONG)0);
      // set the new selection
      if (SendDlgItemMessage(IDC_ENEMIES, LB_SETCURSEL,(UINT)iItem,0) == LB_ERR)
      {
        // we cannot select this one, maybe end of the list. So lets try the previous one
        if (iItem > 0) iItem--;
        if (SendDlgItemMessage(IDC_ENEMIES, LB_SETCURSEL,(UINT)iItem,0) == LB_ERR)
        {
          // no selection possible, so disable the remove button
          GetDlgItem(IDC_S1ENEMYREMOVE)->EnableWindow(FALSE);
        }
      }
    }
  }
}

//-----------------------------------
// new OnPositionButton method
//-----------------------------------
void CSystemFriendEnemyDialog::OnPositionButton()
{
  CPositionDialog PositionDialog;

  // show the position dialog
  PositionDialog.Setup(pConfiguration, uSystemNum);
  PositionDialog.DoDialog();
}

//-----------------------------------
// new OnOptionButton method
//-----------------------------------
void CSystemFriendEnemyDialog::OnOptionButton()
{
  CSystemOptionsDialog SystemOptionsDialog;

  // show the option dialog
  SystemOptionsDialog.Setup(pConfiguration, uSystemNum);
  SystemOptionsDialog.DoDialog();
  
}

//-----------------------------------
// new OnActionButton method
//-----------------------------------
void CSystemFriendEnemyDialog::OnActionButton()
{
  CFriendEnemyActionDialog  FriendEnemyActionDialog;

  // show the action dialog
  FriendEnemyActionDialog.Setup(pConfiguration, uSystemNum);
  FriendEnemyActionDialog.DoDialog();
  
}

//-----------------------------------
// do the dialog
//-----------------------------------
EDialogResultType CSystemFriendEnemyDialog::DoDialog()
{
  // do it
  DoModal();
  // return result
  return eDialogResult;
}


//-----------------------------------
// new OnS1AvSelChange method
//-----------------------------------
void CSystemFriendEnemyDialog::OnS1AvSelChange()
{
  LONG iItem;

  // selection in the general available ship listbox has changed

  // check if we have a selection
  iItem = SendDlgItemMessage(IDC_S1AVSHIPS, LB_GETCURSEL,0,0);
  if (iItem != LB_ERR)
  {
    // unlock the add buttons
    GetDlgItem(IDC_S1FRIENDADD)->EnableWindow(TRUE);
    GetDlgItem(IDC_S1ENEMYADD)->EnableWindow(TRUE);
  } else
  {
    // lock the select and add button
    GetDlgItem(IDC_S1FRIENDADD)->EnableWindow(FALSE);
    GetDlgItem(IDC_S1ENEMYADD)->EnableWindow(FALSE);
  }

}


//-----------------------------------
// new OnFriendsSelChange method
//-----------------------------------
void CSystemFriendEnemyDialog::OnFriendsSelChange()
{
  LONG iItem;

  // selection in the friend listbox has changed

  // check if we have a selection
  iItem = SendDlgItemMessage(IDC_FRIENDS, LB_GETCURSEL,0,0);
  if (iItem != LB_ERR)
  {
    // unlock the remove button
    GetDlgItem(IDC_S1FRIENDREMOVE)->EnableWindow(TRUE);
  } else
  {
    // lock the remove button
    GetDlgItem(IDC_S1FRIENDREMOVE)->EnableWindow(FALSE);
  }
}


//-----------------------------------
// new OnEnemiesSelChange method
//-----------------------------------
void CSystemFriendEnemyDialog::OnEnemiesSelChange()
{
  LONG iItem;

  // selection in the enemy listbox has changed

  // check if we have a selection
  iItem = SendDlgItemMessage(IDC_ENEMIES, LB_GETCURSEL,0,0);
  if (iItem != LB_ERR)
  {
    // unlock the remove button
    GetDlgItem(IDC_S1ENEMYREMOVE)->EnableWindow(TRUE);
  } else
  {
    // lock the remove button
    GetDlgItem(IDC_S1ENEMYREMOVE)->EnableWindow(FALSE);
  }
}


//-----------------------------------
// new OnFriendsDoubleClick method
//-----------------------------------
void CSystemFriendEnemyDialog::OnFriendsDoubleClick()
{
  CString         CStr;
  LONG            iItem;
  POSITION        pos;
  CShipData       *pShip;

  // get the selected friendly ship
  iItem = SendDlgItemMessage(IDC_FRIENDS, LB_GETCURSEL,0,0);
  if (iItem != LB_ERR)
  {
    // find the ship
    pos = (pSystem->GetFriends())->FindIndex(iItem);
    if (pos != NULL)
    {
      // ask for the ship name
      pShip = (CShipData *)(pSystem->GetFriends())->GetAt(pos);
      if (AskForShipName(pShip, pConfiguration, FRIENDNAME_TEXT))
      {
        // update the list box
        FormatShipClassAndName(&CStr, pShip);
        SendDlgItemMessage(IDC_FRIENDS, LB_DELETESTRING, (WORD)iItem,(LONG)0);
        SendDlgItemMessage(IDC_FRIENDS, LB_INSERTSTRING, (WORD)iItem,(LONG)(const char *)CStr);
        // set the selection
        SendDlgItemMessage(IDC_FRIENDS, LB_SETCURSEL, (WORD)iItem,(LONG)0);
      }
    }
  }
}


//-----------------------------------
// new OnEnemiesDoubleClick method
//-----------------------------------
void CSystemFriendEnemyDialog::OnEnemiesDoubleClick()
{
  CString         CStr;
  LONG            iItem;
  POSITION        pos;
  CShipData       *pShip;

  // get the selected enemy ship
  iItem = SendDlgItemMessage(IDC_ENEMIES, LB_GETCURSEL,0,0);
  if (iItem != LB_ERR)
  {
    // find the ship
    pos = (pSystem->GetEnemies())->FindIndex(iItem);
    if (pos != NULL)
    {
      // ask for the ship name
      pShip = (CShipData *)(pSystem->GetEnemies())->GetAt(pos);
      if (AskForShipName(pShip, pConfiguration, ENEMYNAME_TEXT))
      {
        // update the list box
        FormatShipClassAndName(&CStr, pShip);
        SendDlgItemMessage(IDC_ENEMIES, LB_DELETESTRING, (WORD)iItem,(LONG)0);
        SendDlgItemMessage(IDC_ENEMIES, LB_INSERTSTRING, (WORD)iItem,(LONG)(const char *)CStr);
        // set the selection
        SendDlgItemMessage(IDC_ENEMIES, LB_SETCURSEL, (WORD)iItem,(LONG)0);
      }
    }
  }
}


//-----------------------------------
// the message map
//-----------------------------------
BEGIN_MESSAGE_MAP(CSystemFriendEnemyDialog, CModalDialog)
  ON_BN_CLICKED(IDC_S1FINISH,     OnFinishButton)
  ON_BN_CLICKED(IDC_S1NEXT,       OnNextButton)
  ON_BN_CLICKED(IDC_S1PREVIOUS,   OnPreviousButton)
  ON_BN_CLICKED(IDC_S1FRIENDADD,    OnAddFriendButton)
  ON_BN_CLICKED(IDC_S1FRIENDREMOVE, OnRemoveFriendButton)
  ON_BN_CLICKED(IDC_S1ENEMYADD,     OnAddEnemyButton)
  ON_BN_CLICKED(IDC_S1ENEMYREMOVE,  OnRemoveEnemyButton)
  ON_BN_CLICKED(IDC_S1POSITIONS,  OnPositionButton)
  ON_BN_CLICKED(IDC_S1OPTIONS,    OnOptionButton)
  ON_BN_CLICKED(IDC_S1ACTIONS,    OnActionButton)
  ON_LBN_SELCHANGE(IDC_S1AVSHIPS, OnS1AvSelChange)
  ON_LBN_SELCHANGE(IDC_FRIENDS,   OnFriendsSelChange)
  ON_LBN_SELCHANGE(IDC_ENEMIES,   OnEnemiesSelChange)
  ON_LBN_DBLCLK(IDC_FRIENDS,      OnFriendsDoubleClick)
  ON_LBN_DBLCLK(IDC_ENEMIES,      OnEnemiesDoubleClick)
END_MESSAGE_MAP()




// ----------- the system neutral hulk dialog class ---------------------------------------

//-----------------------------------
// overriding the Constructor
//-----------------------------------
CSystemNeutralHulkDialog::CSystemNeutralHulkDialog(CWnd* pParentWnd) : CModalDialog("SYSTEMBOX2", pParentWnd)
{
  // init
  pConfiguration = NULL;
  uSystemNum = 0;
  eDialogResult = CancelDialog;
}

//-----------------------------------
// overriding the OnInitDialog method
//-----------------------------------
BOOL CSystemNeutralHulkDialog::OnInitDialog()
{

  CString      CStr;
  CShipData    *pShip;
  POSITION     pos;
  LONG         iItem;
  char         str[20];

  // some basic checks
  if (pConfiguration == NULL) return TRUE;
  if (uSystemNum == 0) return TRUE;
  if (pSystem == NULL) return TRUE;

  // set up the caption
  

  CStr = EDIT_CAPTION;
  CStr = CStr + *(pConfiguration->GetConfigName());
  CStr = CStr + SYSTEM_CAPTION;
  sprintf(str,"%u",uSystemNum);
  CStr = CStr + str;
  CStr = CStr + SYSTEM_CAPTION1;
  sprintf(str,"%u",(pConfiguration->GetSystems())->GetCount());
  CStr = CStr + str;
  CStr = CStr + SYSTEM_CAPTION2;
  CStr = CStr + pSystem->GetSystemName();
  CStr = CStr + SYSTEM2_CAPTION;
  SendMessage(WM_SETTEXT,0,(LONG)(const char *)CStr);

  // copy the neutrals to the list box
  if ((pSystem->GetNeutrals())->GetCount() > 0)
  {
    // iterate through all neutrals
    for (pos = (pSystem->GetNeutrals())->GetHeadPosition(); pos != NULL; )
    {
      // get ship class and name
      pShip = (CShipData *)(pSystem->GetNeutrals())->GetNext(pos);
      // format the class and optional name
      FormatShipClassAndName(&CStr, pShip);
      SendDlgItemMessage(IDC_NEUTRALS, LB_ADDSTRING,0,(LONG)(const char *)CStr);
    }
  }

  // copy the hulks to the list box
  if ((pSystem->GetHulks())->GetCount() > 0)
  {
    // iterate through all hulks
    for (pos = (pSystem->GetHulks())->GetHeadPosition(); pos != NULL; )
    {
      // get ship class and name
      pShip = (CShipData *)(pSystem->GetHulks())->GetNext(pos);
      // format the class and optional name
      FormatShipClassAndName(&CStr, pShip);
      SendDlgItemMessage(IDC_HULKS, LB_ADDSTRING,0,(LONG)(const char *)CStr);
    }
  }

  // create the available ship list
  AvailableShips.FillListBox(this, IDC_S2AVSHIPS);

  // check if we have a selection in the general avaibale ship list box,
  // enable add buttons if so
  iItem = SendDlgItemMessage(IDC_S2AVSHIPS, LB_GETCURSEL,0,0);
  if (iItem != LB_ERR)
  {
    GetDlgItem(IDC_S2NEUTRALADD)->EnableWindow(TRUE);
    GetDlgItem(IDC_S2HULKADD)->EnableWindow(TRUE);
  }

  // check if we have a selection in the friend list box,
  // enable remove button if so
  iItem = SendDlgItemMessage(IDC_NEUTRALS, LB_GETCURSEL,0,0);
  if (iItem != LB_ERR)
  {
    GetDlgItem(IDC_S2NEUTRALREMOVE)->EnableWindow(TRUE);
  }

  // check if we have a selection in the enemy list box,
  // enable remove button if so
  iItem = SendDlgItemMessage(IDC_HULKS, LB_GETCURSEL,0,0);
  if (iItem != LB_ERR)
  {
    GetDlgItem(IDC_S2HULKREMOVE)->EnableWindow(TRUE);
  }

  return TRUE;
}

//-----------------------------------
// setup data to edit
//-----------------------------------
void CSystemNeutralHulkDialog::Setup(CConfiguration *Configuration, unsigned SystemNum)
{
  POSITION pos;

  // assign the pointer to the configuration
  pConfiguration = Configuration;
  // assign the system number
  uSystemNum = SystemNum;
  // get the system
  pos = (pConfiguration->GetSystems())->FindIndex(uSystemNum - 1);
  if (pos != NULL) pSystem = (CSystemData *)(pConfiguration->GetSystems())->GetAt(pos);
}

//-----------------------------------
// overriding the OnCancel method
//-----------------------------------
void CSystemNeutralHulkDialog::OnCancel()
{
  CCancelDialog CancelDia(this);

  // make sure the user wants to cancel
  CancelDia.DoModal();
  if (CancelDia.GetResult())
  {  
    // set the result
    eDialogResult = CancelDialog;
    // quit
    EndDialog(IDOK);
  }
}
 

//-----------------------------------
// new OnFinishButton method
//-----------------------------------
void CSystemNeutralHulkDialog::OnFinishButton()
{
  // do some tests before finishing
  if (!FinishTest(pConfiguration)) return;
  // set the result
  eDialogResult = FinishDialog;
  // quit
  EndDialog(IDOK);
}

//-----------------------------------
// new OnNextButton method
//-----------------------------------
void CSystemNeutralHulkDialog::OnNextButton()
{
  // set the result
  eDialogResult = NextDialog;
  // quit
  EndDialog(IDOK);
}

//-----------------------------------
// new OnPreviousButton method
//-----------------------------------
void CSystemNeutralHulkDialog::OnPreviousButton()
{
  // set the result
  eDialogResult = PreviousDialog;
  // quit
  EndDialog(IDOK);
}

//-----------------------------------
// new OnAddNeutralButton method
//-----------------------------------
void CSystemNeutralHulkDialog::OnAddNeutralButton()
{
  LONG         iItem;
  CString      ShipClass, CStr;
  CShipData    *pShip;
  CErrorDialog ErrorDialog(this);

  // add a new neutral
  iItem = SendDlgItemMessage(IDC_S2AVSHIPS, LB_GETCURSEL,0,0);
  if (iItem != LB_ERR)
  {
    // get the ship
    if (AvailableShips.FindShipClass(iItem, &ShipClass))
    {
      // create the new ship
      pShip = new CShipData;
      if (pShip == NULL)
      {
        ErrorDialog.Report(ERR_OUTOFMEMORY, ERR_HEADER);
        return;
      }
      // set the data    
      pShip->SetShipClass(ShipClass);
      // put it to the neutral list
      (pSystem->GetNeutrals())->AddTail(pShip);
      // update the list box
      FormatShipClassAndName(&CStr, pShip);
      SendDlgItemMessage(IDC_NEUTRALS, LB_ADDSTRING,0,(LONG)(const char *)CStr);
    }
  }
  
}

//-----------------------------------
// new OnRemoveNeutralButton method
//-----------------------------------
void CSystemNeutralHulkDialog::OnRemoveNeutralButton()
{
  LONG         iItem;
  CShipData    *pShip;
  POSITION     pos;

  // remove a neutral
  iItem = SendDlgItemMessage(IDC_NEUTRALS, LB_GETCURSEL,0,0);
  if (iItem != LB_ERR)
  {
    // find the ship
    pos = (pSystem->GetNeutrals())->FindIndex(iItem);
    if (pos != NULL)
    {
      // delete the ship
      pShip = (CShipData *)(pSystem->GetNeutrals())->GetAt(pos);
      delete pShip;
      (pSystem->GetNeutrals())->RemoveAt(pos);
      // update the list box
      SendDlgItemMessage(IDC_NEUTRALS, LB_DELETESTRING, (WORD)iItem,(LONG)0);
      // set the new selection
      if (SendDlgItemMessage(IDC_NEUTRALS, LB_SETCURSEL,(UINT)iItem,0) == LB_ERR)
      {
        // we cannot select this one, maybe end of the list. So lets try the previous one
        if (iItem > 0) iItem--;
        if (SendDlgItemMessage(IDC_NEUTRALS, LB_SETCURSEL,(UINT)iItem,0) == LB_ERR)
        {
          // no selection possible, so disable the remove button
          GetDlgItem(IDC_S2NEUTRALREMOVE)->EnableWindow(FALSE);
        }
      }
    }
  }
}

//-----------------------------------
// new OnAddHulkButton method
//-----------------------------------
void CSystemNeutralHulkDialog::OnAddHulkButton()
{
  LONG         iItem;
  CString      ShipClass, CStr;
  CShipData    *pShip;
  CErrorDialog ErrorDialog(this);

  // add a new enemy
  iItem = SendDlgItemMessage(IDC_S2AVSHIPS, LB_GETCURSEL,0,0);
  if (iItem != LB_ERR)
  {
    // get the ship
    if (AvailableShips.FindShipClass(iItem, &ShipClass))
    {
      // create the new ship
      pShip = new CShipData;
      if (pShip == NULL)
      {
        ErrorDialog.Report(ERR_OUTOFMEMORY, ERR_HEADER);
        return;
      }
      // set the data    
      pShip->SetShipClass(ShipClass);
      // put it to the hulk list
      (pSystem->GetHulks())->AddTail(pShip);
      // update the list box
      FormatShipClassAndName(&CStr, pShip);
      SendDlgItemMessage(IDC_HULKS, LB_ADDSTRING,0,(LONG)(const char *)CStr);
    }
  }
  
}

//-----------------------------------
// new OnRemoveHulksButton method
//-----------------------------------
void CSystemNeutralHulkDialog::OnRemoveHulkButton()
{
  LONG         iItem;
  CShipData    *pShip;
  POSITION     pos;

  // remove a hulk
  iItem = SendDlgItemMessage(IDC_HULKS, LB_GETCURSEL,0,0);
  if (iItem != LB_ERR)
  {
    // find the ship
    pos = (pSystem->GetHulks())->FindIndex(iItem);
    if (pos != NULL)
    {
      // delete the ship
      pShip = (CShipData *)(pSystem->GetHulks())->GetAt(pos);
      delete pShip;
      (pSystem->GetHulks())->RemoveAt(pos);
      // update the list box
      SendDlgItemMessage(IDC_HULKS, LB_DELETESTRING, (WORD)iItem,(LONG)0);
      // set the new selection
      if (SendDlgItemMessage(IDC_HULKS, LB_SETCURSEL,(UINT)iItem,0) == LB_ERR)
      {
        // we cannot select this one, maybe end of the list. So lets try the previous one
        if (iItem > 0) iItem--;
        if (SendDlgItemMessage(IDC_HULKS, LB_SETCURSEL,(UINT)iItem,0) == LB_ERR)
        {
          // no selection possible, so disable the remove button
          GetDlgItem(IDC_S2HULKREMOVE)->EnableWindow(FALSE);
        }
      }
    }
  }
}


//-----------------------------------
// new OnPositionButton method
//-----------------------------------
void CSystemNeutralHulkDialog::OnPositionButton()
{
  CPositionDialog PositionDialog;

  // show the position dialog
  PositionDialog.Setup(pConfiguration, uSystemNum);
  PositionDialog.DoDialog();
}

//-----------------------------------
// new OnBehaviorButton method
//-----------------------------------
void CSystemNeutralHulkDialog::OnBehaviorButton()
{
  CNeutralBehaviorDialog  NeutralBehaviorDialog;

  // show the behavior dialog
  NeutralBehaviorDialog.Setup(pConfiguration, uSystemNum);
  NeutralBehaviorDialog.DoDialog();
  
}

//-----------------------------------
// do the dialog
//-----------------------------------
EDialogResultType CSystemNeutralHulkDialog::DoDialog()
{
  // do it
  DoModal();
  // return result
  return eDialogResult;
}


//-----------------------------------
// new OnS2AvSelChange method
//-----------------------------------
void CSystemNeutralHulkDialog::OnS2AvSelChange()
{
  LONG iItem;

  // selection in the general available ship listbox has changed

  // check if we have a selection
  iItem = SendDlgItemMessage(IDC_S2AVSHIPS, LB_GETCURSEL,0,0);
  if (iItem != LB_ERR)
  {
    // unlock the add buttons
    GetDlgItem(IDC_S2NEUTRALADD)->EnableWindow(TRUE);
    GetDlgItem(IDC_S2HULKADD)->EnableWindow(TRUE);
  } else
  {
    // lock the select and add button
    GetDlgItem(IDC_S2NEUTRALADD)->EnableWindow(FALSE);
    GetDlgItem(IDC_S2HULKADD)->EnableWindow(FALSE);
  }

}


//-----------------------------------
// new OnNeutralsSelChange method
//-----------------------------------
void CSystemNeutralHulkDialog::OnNeutralsSelChange()
{
  LONG iItem;

  // selection in the neutral listbox has changed

  // check if we have a selection
  iItem = SendDlgItemMessage(IDC_NEUTRALS, LB_GETCURSEL,0,0);
  if (iItem != LB_ERR)
  {
    // unlock the remove button
    GetDlgItem(IDC_S2NEUTRALREMOVE)->EnableWindow(TRUE);
  } else
  {
    // lock the remove button
    GetDlgItem(IDC_S2NEUTRALREMOVE)->EnableWindow(FALSE);
  }
}


//-----------------------------------
// new OnHulksSelChange method
//-----------------------------------
void CSystemNeutralHulkDialog::OnHulksSelChange()
{
  LONG iItem;

  // selection in the hulk listbox has changed

  // check if we have a selection
  iItem = SendDlgItemMessage(IDC_HULKS, LB_GETCURSEL,0,0);
  if (iItem != LB_ERR)
  {
    // unlock the remove button
    GetDlgItem(IDC_S2HULKREMOVE)->EnableWindow(TRUE);
  } else
  {
    // lock the remove button
    GetDlgItem(IDC_S2HULKREMOVE)->EnableWindow(FALSE);
  }
}


//-----------------------------------
// new OnNeutralsDoubleClick method
//-----------------------------------
void CSystemNeutralHulkDialog::OnNeutralsDoubleClick()
{
  CString         CStr;
  LONG            iItem;
  POSITION        pos;
  CShipData       *pShip;

  // get the selected neutral ship
  iItem = SendDlgItemMessage(IDC_NEUTRALS, LB_GETCURSEL,0,0);
  if (iItem != LB_ERR)
  {
    // find the ship
    pos = (pSystem->GetNeutrals())->FindIndex(iItem);
    if (pos != NULL)
    {
      // ask for the ship name
      pShip = (CShipData *)(pSystem->GetNeutrals())->GetAt(pos);
      if (AskForShipName(pShip, pConfiguration, NEUTRALNAME_TEXT))
      {
        // update the list box
        FormatShipClassAndName(&CStr, pShip);
        SendDlgItemMessage(IDC_NEUTRALS, LB_DELETESTRING, (WORD)iItem,(LONG)0);
        SendDlgItemMessage(IDC_NEUTRALS, LB_INSERTSTRING, (WORD)iItem,(LONG)(const char *)CStr);
        // set the selection
        SendDlgItemMessage(IDC_NEUTRALS, LB_SETCURSEL, (WORD)iItem,(LONG)0);
      }
    }
  }
}


//-----------------------------------
// new OnHulksDoubleClick method
//-----------------------------------
void CSystemNeutralHulkDialog::OnHulksDoubleClick()
{
  CString         CStr;
  LONG            iItem;
  POSITION        pos;
  CShipData       *pShip;

  // get the selected hulk ship
  iItem = SendDlgItemMessage(IDC_HULKS, LB_GETCURSEL,0,0);
  if (iItem != LB_ERR)
  {
    // find the ship
    pos = (pSystem->GetHulks())->FindIndex(iItem);
    if (pos != NULL)
    {
      // ask for the ship name
      pShip = (CShipData *)(pSystem->GetHulks())->GetAt(pos);
      if (AskForShipName(pShip, pConfiguration, HULKNAME_TEXT))
      {
        // update the list box
        FormatShipClassAndName(&CStr, pShip);
        SendDlgItemMessage(IDC_HULKS, LB_DELETESTRING, (WORD)iItem,(LONG)0);
        SendDlgItemMessage(IDC_HULKS, LB_INSERTSTRING, (WORD)iItem,(LONG)(const char *)CStr);
        // set the selection
        SendDlgItemMessage(IDC_HULKS, LB_SETCURSEL, (WORD)iItem,(LONG)0);
      }
    }
  }
}


//-----------------------------------
// the message map
//-----------------------------------
BEGIN_MESSAGE_MAP(CSystemNeutralHulkDialog, CModalDialog)
  ON_BN_CLICKED(IDC_S2FINISH,        OnFinishButton)
  ON_BN_CLICKED(IDC_S2NEXT,          OnNextButton)
  ON_BN_CLICKED(IDC_S2PREVIOUS,      OnPreviousButton)
  ON_BN_CLICKED(IDC_S2NEUTRALADD,    OnAddNeutralButton)
  ON_BN_CLICKED(IDC_S2NEUTRALREMOVE, OnRemoveNeutralButton)
  ON_BN_CLICKED(IDC_S2HULKADD,       OnAddHulkButton)
  ON_BN_CLICKED(IDC_S2HULKREMOVE,    OnRemoveHulkButton)
  ON_BN_CLICKED(IDC_S2POSITIONS,     OnPositionButton)
  ON_BN_CLICKED(IDC_S2NBEHAV,        OnBehaviorButton)
  ON_LBN_SELCHANGE(IDC_S2AVSHIPS,    OnS2AvSelChange)
  ON_LBN_SELCHANGE(IDC_NEUTRALS,     OnNeutralsSelChange)
  ON_LBN_SELCHANGE(IDC_HULKS,        OnHulksSelChange)
  ON_LBN_DBLCLK(IDC_NEUTRALS,        OnNeutralsDoubleClick)
  ON_LBN_DBLCLK(IDC_HULKS,           OnHulksDoubleClick)
END_MESSAGE_MAP()



// ----------- the system jump gate dialog class ---------------------------------------

//-----------------------------------
// overriding the Constructor
//-----------------------------------
CSystemJumpGateDialog::CSystemJumpGateDialog(CWnd* pParentWnd) : CModalDialog("SYSTEMBOX3", pParentWnd)
{
  // init
  pConfiguration = NULL;
  uSystemNum = 0;
  eDialogResult = CancelDialog;
}

//-----------------------------------
// overriding the OnInitDialog method
//-----------------------------------
BOOL CSystemJumpGateDialog::OnInitDialog()
{

  CString      CStr;
  LONG         iItem;
  char         str[20];

  // some basic checks
  if (pConfiguration == NULL) return TRUE;
  if (uSystemNum == 0) return TRUE;
  if (pSystem == NULL) return TRUE;

  // set up the caption
  

  CStr = EDIT_CAPTION;
  CStr = CStr + *(pConfiguration->GetConfigName());
  CStr = CStr + SYSTEM_CAPTION;
  sprintf(str,"%u",uSystemNum);
  CStr = CStr + str;
  CStr = CStr + SYSTEM_CAPTION1;
  sprintf(str,"%u",(pConfiguration->GetSystems())->GetCount());
  CStr = CStr + str;
  CStr = CStr + SYSTEM_CAPTION2;
  CStr = CStr + pSystem->GetSystemName();
  CStr = CStr + SYSTEM3_CAPTION;
  SendMessage(WM_SETTEXT,0,(LONG)(const char *)CStr);

  // place the jump gate in the list box
  if ((pSystem->GetJumpGate())->IsSetShipClass())
  {
    // format the class and optional name
    FormatShipClassAndName(&CStr, pSystem->GetJumpGate());
    SendDlgItemMessage(IDC_JUMPGATE, LB_ADDSTRING,0,(LONG)(const char *)CStr);
  }

  // create the available ship list
  AvailableShips.FillListBox(this, IDC_S3AVSHIPS);

  // check if we have a selection in the general avaibale ship list box,
  // enable select buttons if so
  iItem = SendDlgItemMessage(IDC_S3AVSHIPS, LB_GETCURSEL,0,0);
  if (iItem != LB_ERR)
  {
    GetDlgItem(IDC_S3SELECT)->EnableWindow(TRUE);
  }

  // check if we have a selection in the jump gate list box,
  // enable remove button if so
  iItem = SendDlgItemMessage(IDC_JUMPGATE, LB_GETCURSEL,0,0);
  if (iItem != LB_ERR)
  {
    GetDlgItem(IDC_S3REMOVE)->EnableWindow(TRUE);
  }

  // show the state of the gate
  ShowState();

  return TRUE;
}

//-----------------------------------
// setup data to edit
//-----------------------------------
void CSystemJumpGateDialog::Setup(CConfiguration *Configuration, unsigned SystemNum)
{
  POSITION pos;

  // assign the pointer to the configuration
  pConfiguration = Configuration;
  // assign the system number
  uSystemNum = SystemNum;
  // get the system
  pos = (pConfiguration->GetSystems())->FindIndex(uSystemNum - 1);
  if (pos != NULL) pSystem = (CSystemData *)(pConfiguration->GetSystems())->GetAt(pos);
}

//-----------------------------------
// display jump gate state
//-----------------------------------
void CSystemJumpGateDialog::ShowState()
{
  BOOL bGateIncomplete, bValidDest, bPosSet, bTimeSet;

  // is there no gate ?
  if (!pSystem->GetJumpGate()->IsSetShipClass())
  {
    // yes, tell it
    SendDlgItemMessage(IDC_STATUS1, WM_SETTEXT,0,(LONG)JUMPSTATE_INACTIVE);
    // clear all the remaining lines
    SendDlgItemMessage(IDC_STATUS2, WM_SETTEXT,0,(LONG)" ");
    SendDlgItemMessage(IDC_STATUS3, WM_SETTEXT,0,(LONG)" ");
    SendDlgItemMessage(IDC_STATUS4, WM_SETTEXT,0,(LONG)" ");
    SendDlgItemMessage(IDC_STATUS5, WM_SETTEXT,0,(LONG)" ");
  } else
  {
    // there is a gate. First check if we have a valid destination
    bValidDest = FALSE;
    if (pSystem->IsSetJumpDestination())
      bValidDest = pConfiguration->CheckJumpDestination(pSystem->GetJumpDestination());

    // is there a complete definition ?
    if ((pSystem->IsSetJumpGatePos() || pSystem->IsSetJumpGatePosPlanet() || pSystem->IsSetJumpTime()) &&
        (bValidDest))
    {
      // yes, tell it
      SendDlgItemMessage(IDC_STATUS1, WM_SETTEXT,0,(LONG)JUMPSTATE_ACTIVE);
      bGateIncomplete = FALSE;
    } else
    {
      // no, tell it
      SendDlgItemMessage(IDC_STATUS1, WM_SETTEXT,0,(LONG)JUMPSTATE_INCOMP);
      bGateIncomplete = TRUE;
    }

    // show details
    if (pSystem->IsSetJumpGatePos() || pSystem->IsSetJumpGatePosPlanet())
    {
      // we have a position
      bPosSet = TRUE;
    } else
    {
      // we have no position
      bPosSet = FALSE;
    }
    if (pSystem->IsSetJumpTime())
    {
      // we have a time
      bTimeSet = TRUE;
    } else
    {
      // we have no time
      bTimeSet = FALSE;
    }
    if (bPosSet && bTimeSet)
      SendDlgItemMessage(IDC_STATUS2, WM_SETTEXT,0,(LONG)JUMPSTATE_POSTIMESET);
    else if (bPosSet && !bTimeSet)
      SendDlgItemMessage(IDC_STATUS2, WM_SETTEXT,0,(LONG)JUMPSTATE_POSSET);
    else if (!bPosSet && bTimeSet)
      SendDlgItemMessage(IDC_STATUS2, WM_SETTEXT,0,(LONG)JUMPSTATE_TIMESET);
    else 
      SendDlgItemMessage(IDC_STATUS2, WM_SETTEXT,0,(LONG)JUMPSTATE_NOPOSTIMESET);

    // what about the destination
    if (bValidDest)
    {
      // we have a fine destination
      SendDlgItemMessage(IDC_STATUS3, WM_SETTEXT,0,(LONG)JUMPSTATE_DEST);
    } else
    {
      // do we at least HAVE a destination (even so it is not valid) ?
      if (pSystem->IsSetJumpDestination())
      {
        // yes, tell about it
        SendDlgItemMessage(IDC_STATUS3, WM_SETTEXT,0,(LONG)JUMPSTATE_INVDEST);
      } else
      {
        // we have nothing!
        SendDlgItemMessage(IDC_STATUS3, WM_SETTEXT,0,(LONG)JUMPSTATE_NODEST);
      }
    }

    // if the gate is complete, show more state info
    if (!bGateIncomplete)
    {
      // is the gate visible
      if (pSystem->GetJumpOptions()->IsSetGateHidden())
      {
        // yes, show it
        SendDlgItemMessage(IDC_STATUS4, WM_SETTEXT,0,(LONG)JUMPSTATE_INVIS);
      } else
      {
        // no, show it
        SendDlgItemMessage(IDC_STATUS4, WM_SETTEXT,0,(LONG)JUMPSTATE_VIS);
      }

      // where and when will the gate appear
      if (bPosSet && bTimeSet)
      {
        // position and time given
        SendDlgItemMessage(IDC_STATUS5, WM_SETTEXT,0,(LONG)JUMPSTATE_TIMEPOS);
      } else
      if (bPosSet)
      {
        // position only given
        SendDlgItemMessage(IDC_STATUS5, WM_SETTEXT,0,(LONG)JUMPSTATE_POSONLY);
      } else
      {
        // time only given
        SendDlgItemMessage(IDC_STATUS5, WM_SETTEXT,0,(LONG)JUMPSTATE_TIMEONLY);
      }
    } else
    {
      // clear additional state info
      SendDlgItemMessage(IDC_STATUS4, WM_SETTEXT,0,(LONG)" ");
      SendDlgItemMessage(IDC_STATUS5, WM_SETTEXT,0,(LONG)" ");
    }
  }
}

//-----------------------------------
// overriding the OnCancel method
//-----------------------------------
void CSystemJumpGateDialog::OnCancel()
{
  CCancelDialog CancelDia(this);

  // make sure the user wants to cancel
  CancelDia.DoModal();
  if (CancelDia.GetResult())
  {  
    // set the result
    eDialogResult = CancelDialog;
    // quit
    EndDialog(IDOK);
  }
}
 

//-----------------------------------
// new OnFinishButton method
//-----------------------------------
void CSystemJumpGateDialog::OnFinishButton()
{
  // do some tests before finishing
  if (!FinishTest(pConfiguration)) return;
  // set the result
  eDialogResult = FinishDialog;
  // quit
  EndDialog(IDOK);
}

//-----------------------------------
// new OnPreviousButton method
//-----------------------------------
void CSystemJumpGateDialog::OnPreviousButton()
{
  // set the result
  eDialogResult = PreviousDialog;
  // quit
  EndDialog(IDOK);
}

//-----------------------------------
// new OnSelectButton method
//-----------------------------------
void CSystemJumpGateDialog::OnSelectButton()
{
  LONG         iItem;
  CString      ShipClass, CStr;

  // select a new jump gate
  iItem = SendDlgItemMessage(IDC_S3AVSHIPS, LB_GETCURSEL,0,0);
  if (iItem != LB_ERR)
  {
    // get the ship
    if (AvailableShips.FindShipClass(iItem, &ShipClass))
    {
      // set the data    
      (pSystem->GetJumpGate())->SetShipClass(ShipClass);
      // update the list box
      SendDlgItemMessage(IDC_JUMPGATE, LB_DELETESTRING,0,(LONG)0);
      FormatShipClassAndName(&CStr, pSystem->GetJumpGate());
      SendDlgItemMessage(IDC_JUMPGATE, LB_ADDSTRING,0,(LONG)(const char *)CStr);
    }
  }
  // show the state of the gate (may have changed)
  ShowState();
}

//-----------------------------------
// new OnRemoveButton method
//-----------------------------------
void CSystemJumpGateDialog::OnRemoveButton()
{
  LONG         iItem;

  // remove the jump gate
  iItem = SendDlgItemMessage(IDC_JUMPGATE, LB_GETCURSEL,0,0);
  if (iItem != LB_ERR)
  {
    // remove the gate
    pSystem->GetJumpGate()->UnsetShipClass();
    pSystem->GetJumpGate()->UnsetShipName();
    // update the list box
    SendDlgItemMessage(IDC_JUMPGATE, LB_DELETESTRING,0,(LONG)0);
    // we loose the selection, so disable the remove button
    GetDlgItem(IDC_S3REMOVE)->EnableWindow(FALSE);
  }
  // show the state of the gate (may have changed)
  ShowState();
}

//-----------------------------------
// new OnPositionButton method
//-----------------------------------
void CSystemJumpGateDialog::OnPositionButton()
{
  CPositionDialog PositionDialog;

  // show the position dialog
  PositionDialog.Setup(pConfiguration, uSystemNum);
  PositionDialog.DoDialog();
  // show the state of the gate (may have changed)
  ShowState();
}

//-----------------------------------
// new OnOptionButton method
//-----------------------------------
void CSystemJumpGateDialog::OnOptionButton()
{
  CSystemJumpOptionsDialog SystemJumpOptionsDialog;

  // show the option dialog
  SystemJumpOptionsDialog.Setup(pConfiguration, uSystemNum);
  SystemJumpOptionsDialog.DoDialog();
  // show the state of the gate (may have changed)
  ShowState();
}

//-----------------------------------
// new OnDestinationButton method
//-----------------------------------
void CSystemJumpGateDialog::OnDestinationButton()
{
  CSystemJumpDestinationDialog SystemJumpDestinationDialog;

  // show the destination dialog
  SystemJumpDestinationDialog.Setup(pConfiguration, uSystemNum);
  SystemJumpDestinationDialog.DoDialog();
  // show the state of the gate (may have changed)
  ShowState();
}

//-----------------------------------
// do the dialog
//-----------------------------------
EDialogResultType CSystemJumpGateDialog::DoDialog()
{
  // do it
  DoModal();
  // return result
  return eDialogResult;
}


//-----------------------------------
// new OnS3AvSelChange method
//-----------------------------------
void CSystemJumpGateDialog::OnS3AvSelChange()
{
  LONG iItem;

  // selection in the general available ship listbox has changed

  // check if we have a selection
  iItem = SendDlgItemMessage(IDC_S3AVSHIPS, LB_GETCURSEL,0,0);
  if (iItem != LB_ERR)
  {
    // unlock the select button
    GetDlgItem(IDC_S3SELECT)->EnableWindow(TRUE);
  } else
  {
    // lock the select button
    GetDlgItem(IDC_S3SELECT)->EnableWindow(FALSE);
  }
}


//-----------------------------------
// new OnJumpGateSelChange method
//-----------------------------------
void CSystemJumpGateDialog::OnJumpGateSelChange()
{
  LONG iItem;

  // selection in the jump gate listbox has changed

  // check if we have a selection
  iItem = SendDlgItemMessage(IDC_JUMPGATE, LB_GETCURSEL,0,0);
  if (iItem != LB_ERR)
  {
    // unlock the remove button
    GetDlgItem(IDC_S3REMOVE)->EnableWindow(TRUE);
  } else
  {
    // lock the remove button
    GetDlgItem(IDC_S3REMOVE)->EnableWindow(FALSE);
  }
}


//-----------------------------------
// new OnJumpGateDoubleClick method
//-----------------------------------
void CSystemJumpGateDialog::OnJumpGateDoubleClick()
{
  CString         CStr;
  LONG            iItem;
  CShipData       *pShip;

  // do we have a selection
  iItem = SendDlgItemMessage(IDC_JUMPGATE, LB_GETCURSEL,0,0);
  if (iItem != LB_ERR)
  {
    // ask for the new jump gate name
    pShip = (CShipData *)pSystem->GetJumpGate();
    if (AskForShipName(pShip, pConfiguration, JUMPGATENAME_TEXT))
    {
      // update the list box
      FormatShipClassAndName(&CStr, pShip);
      SendDlgItemMessage(IDC_JUMPGATE, LB_DELETESTRING, (WORD)0,(LONG)0);
      SendDlgItemMessage(IDC_JUMPGATE, LB_INSERTSTRING, (WORD)0,(LONG)(const char *)CStr);
      // set the selection
      SendDlgItemMessage(IDC_JUMPGATE, LB_SETCURSEL, (WORD)0,(LONG)0);
    }
  }
}

//-----------------------------------
// the message map
//-----------------------------------
BEGIN_MESSAGE_MAP(CSystemJumpGateDialog, CModalDialog)
  ON_BN_CLICKED(IDC_S3FINISH,        OnFinishButton)
  ON_BN_CLICKED(IDC_S3PREVIOUS,      OnPreviousButton)
  ON_BN_CLICKED(IDC_S3SELECT,        OnSelectButton)
  ON_BN_CLICKED(IDC_S3REMOVE,        OnRemoveButton)
  ON_BN_CLICKED(IDC_S3POSITIONS,     OnPositionButton)
  ON_BN_CLICKED(IDC_S3OPTIONS,       OnOptionButton)
  ON_BN_CLICKED(IDC_S3DESTINATION,   OnDestinationButton)
  ON_LBN_SELCHANGE(IDC_S3AVSHIPS,    OnS3AvSelChange)
  ON_LBN_SELCHANGE(IDC_JUMPGATE,     OnJumpGateSelChange)
  ON_LBN_DBLCLK(IDC_JUMPGATE,        OnJumpGateDoubleClick)
END_MESSAGE_MAP()



// ----------- the position class ---------------------------------------

//-----------------------------------
// overriding the Constructor
//-----------------------------------
CPositionDialog::CPositionDialog(CWnd* pParentWnd) : CModalDialog("POSITIONBOX", pParentWnd)
{
  // init
  pConfiguration = NULL;
  uSystemNum = 0;
  pSystem = NULL;
  ePlayerPosDef  = NoPositionDefined;
  eFriendPosDef  = NoPositionDefined;
  eEnemyPosDef   = NoPositionDefined;
  eNeutralPosDef = NoPositionDefined;
  eHulkPosDef    = NoPositionDefined;
  eJumpPosDef    = NoPositionDefined;
}

//-----------------------------------
// overriding the OnInitDialog method
//-----------------------------------
BOOL CPositionDialog::OnInitDialog()
{
  CString      CStr;

  // some basic checks
  if (pConfiguration == NULL) return TRUE;
  if (pSystem == NULL) return TRUE;

  CStr = pSystem->GetSystemName();
  CStr = CStr + POSITION_CAPTION;
  SendMessage(WM_SETTEXT,0,(LONG)(const char *)CStr);

  // show the player position
  FormatPosition(&CStr, &ePlayerPosDef, pSystem, 0);
  SendDlgItemMessage(IDC_PLAYERPOS, LB_ADDSTRING,0,(LONG)(const char *)CStr);

  // show the friend position
  FormatPosition(&CStr,  &eFriendPosDef, pSystem, 1);
  SendDlgItemMessage(IDC_FRIENDSPOS, LB_ADDSTRING,0,(LONG)(const char *)CStr);

  // show the enemy position
  FormatPosition(&CStr,  &eEnemyPosDef, pSystem, 2);
  SendDlgItemMessage(IDC_ENEMIESPOS, LB_ADDSTRING,0,(LONG)(const char *)CStr);

  // show the neutral position
  FormatPosition(&CStr,  &eNeutralPosDef, pSystem, 3);
  SendDlgItemMessage(IDC_NEUTRALSPOS, LB_ADDSTRING,0,(LONG)(const char *)CStr);

  // show the hulk position
  FormatPosition(&CStr,  &eHulkPosDef, pSystem, 4);
  SendDlgItemMessage(IDC_HULKSPOS, LB_ADDSTRING,0,(LONG)(const char *)CStr);

  // show the jump gate position
  FormatPosition(&CStr,  &eJumpPosDef, pSystem, 5);
  SendDlgItemMessage(IDC_JUMPPOS, LB_ADDSTRING,0,(LONG)(const char *)CStr);

  // create the available planets list
  AvailableSystemsAndPlanets.FillListBoxWithSystemPlanets(pSystem, this, IDC_PLANETS);

  return TRUE;
}

//-----------------------------------
// setup data to edit
//-----------------------------------
void CPositionDialog::Setup(CConfiguration *Configuration, unsigned SystemNum)
{
  POSITION pos;

  // assign the pointer to the configuration
  pConfiguration = Configuration;
  // assign the system number
  uSystemNum = SystemNum;
  // get the system
  pos = (pConfiguration->GetSystems())->FindIndex(uSystemNum - 1);
  if (pos != NULL) pSystem = (CSystemData *)(pConfiguration->GetSystems())->GetAt(pos);
}

//-----------------------------------
// overriding the OnOK method
//-----------------------------------
void CPositionDialog::OnOK()
{
  char str[MAX_QB2_LINELENGTH + 1], str1[MAX_QB2_LINELENGTH + 1];

  // get the new player position
  if (ePlayerPosDef == NoPositionDefined)
  {
    // no position
    pSystem->UnsetPlayerPos();
    pSystem->UnsetPlayerPosPlanet();
  } else
  if (ePlayerPosDef == PlanetPositionDefined)
  {
    // a planet, get it
    pSystem->UnsetPlayerPos();
    SendDlgItemMessage(IDC_PLAYERPOS, LB_GETTEXT,(UINT)0,(LONG)(LPSTR)str);
    pSystem->SetPlayerPosPlanet(str);
  } else
  {
    // coordinates, get it
    pSystem->UnsetPlayerPosPlanet();
    SendDlgItemMessage(IDC_PLAYERPOS, LB_GETTEXT,(UINT)0,(LONG)(LPSTR)str);
    GetNthWordInString(str1, str, 1);
    pSystem->SetPlayerPosX(atol(str1));
    GetNthWordInString(str1, str, 2);
    pSystem->SetPlayerPosY(atol(str1));
    GetNthWordInString(str1, str, 3);
    pSystem->SetPlayerPosZ(atol(str1));
  }

  // get the new friends position
  if (eFriendPosDef == NoPositionDefined)
  {
    // no position
    pSystem->UnsetFriendPos();
    pSystem->UnsetFriendPosPlanet();
  } else
  if (eFriendPosDef == PlanetPositionDefined)
  {
    // a planet, get it
    pSystem->UnsetFriendPos();
    SendDlgItemMessage(IDC_FRIENDSPOS, LB_GETTEXT,(UINT)0,(LONG)(LPSTR)str);
    pSystem->SetFriendPosPlanet(str);
  } else
  {
    // coordinates, get it
    pSystem->UnsetFriendPosPlanet();
    SendDlgItemMessage(IDC_FRIENDSPOS, LB_GETTEXT,(UINT)0,(LONG)(LPSTR)str);
    GetNthWordInString(str1, str, 1);
    pSystem->SetFriendPosX(atol(str1));
    GetNthWordInString(str1, str, 2);
    pSystem->SetFriendPosY(atol(str1));
    GetNthWordInString(str1, str, 3);
    pSystem->SetFriendPosZ(atol(str1));
  }

  // get the new enemies position
  if (eEnemyPosDef == NoPositionDefined)
  {
    // no position
    pSystem->UnsetEnemyPos();
    pSystem->UnsetEnemyPosPlanet();
  } else
  if (eEnemyPosDef == PlanetPositionDefined)
  {
    // a planet, get it
    pSystem->UnsetEnemyPos();
    SendDlgItemMessage(IDC_ENEMIESPOS, LB_GETTEXT,(UINT)0,(LONG)(LPSTR)str);
    pSystem->SetEnemyPosPlanet(str);
  } else
  {
    // coordinates, get it
    pSystem->UnsetEnemyPosPlanet();
    SendDlgItemMessage(IDC_ENEMIESPOS, LB_GETTEXT,(UINT)0,(LONG)(LPSTR)str);
    GetNthWordInString(str1, str, 1);
    pSystem->SetEnemyPosX(atol(str1));
    GetNthWordInString(str1, str, 2);
    pSystem->SetEnemyPosY(atol(str1));
    GetNthWordInString(str1, str, 3);
    pSystem->SetEnemyPosZ(atol(str1));
  }

  // get the new neutral position
  if (eNeutralPosDef == NoPositionDefined)
  {
    // no position
    pSystem->UnsetNeutralPos();
    pSystem->UnsetNeutralPosPlanet();
  } else
  if (eNeutralPosDef == PlanetPositionDefined)
  {
    // a planet, get it
    pSystem->UnsetNeutralPos();
    SendDlgItemMessage(IDC_NEUTRALSPOS, LB_GETTEXT,(UINT)0,(LONG)(LPSTR)str);
    pSystem->SetNeutralPosPlanet(str);
  } else
  {
    // coordinates, get it
    pSystem->UnsetNeutralPosPlanet();
    SendDlgItemMessage(IDC_NEUTRALSPOS, LB_GETTEXT,(UINT)0,(LONG)(LPSTR)str);
    GetNthWordInString(str1, str, 1);
    pSystem->SetNeutralPosX(atol(str1));
    GetNthWordInString(str1, str, 2);
    pSystem->SetNeutralPosY(atol(str1));
    GetNthWordInString(str1, str, 3);
    pSystem->SetNeutralPosZ(atol(str1));
  }

  // get the new hulk position
  if (eHulkPosDef == NoPositionDefined)
  {
    // no position
    pSystem->UnsetHulkPos();
    pSystem->UnsetHulkPosPlanet();
  } else
  if (eHulkPosDef == PlanetPositionDefined)
  {
    // a planet, get it
    pSystem->UnsetHulkPos();
    SendDlgItemMessage(IDC_HULKSPOS, LB_GETTEXT,(UINT)0,(LONG)(LPSTR)str);
    pSystem->SetHulkPosPlanet(str);
  } else
  {
    // coordinates, get it
    pSystem->UnsetHulkPosPlanet();
    SendDlgItemMessage(IDC_HULKSPOS, LB_GETTEXT,(UINT)0,(LONG)(LPSTR)str);
    GetNthWordInString(str1, str, 1);
    pSystem->SetHulkPosX(atol(str1));
    GetNthWordInString(str1, str, 2);
    pSystem->SetHulkPosY(atol(str1));
    GetNthWordInString(str1, str, 3);
    pSystem->SetHulkPosZ(atol(str1));
  }

  // get the new jump gate position
  if (eJumpPosDef == NoPositionDefined)
  {
    // no position
    pSystem->UnsetJumpGatePos();
    pSystem->UnsetJumpGatePosPlanet();
  } else
  if (eJumpPosDef == PlanetPositionDefined)
  {
    // a planet, get it
    pSystem->UnsetJumpGatePos();
    SendDlgItemMessage(IDC_JUMPPOS, LB_GETTEXT,(UINT)0,(LONG)(LPSTR)str);
    pSystem->SetJumpGatePosPlanet(str);
  } else
  {
    // coordinates, get it
    pSystem->UnsetJumpGatePosPlanet();
    SendDlgItemMessage(IDC_JUMPPOS, LB_GETTEXT,(UINT)0,(LONG)(LPSTR)str);
    GetNthWordInString(str1, str, 1);
    pSystem->SetJumpGatePosX(atol(str1));
    GetNthWordInString(str1, str, 2);
    pSystem->SetJumpGatePosY(atol(str1));
    GetNthWordInString(str1, str, 3);
    pSystem->SetJumpGatePosZ(atol(str1));
  }

  EndDialog(IDOK);
}
 

//-----------------------------------
// new OnPlayerPlanetButton method
//-----------------------------------
void CPositionDialog::OnPlayerPlanetButton()
{
  LONG         iItem;
  char         line[LINE_LENGTH];

  // select a new planet
  iItem = SendDlgItemMessage(IDC_PLANETS, LB_GETCURSEL,0,0);
  if (iItem != LB_ERR)
  {
    // get the planet
    SendDlgItemMessage(IDC_PLANETS, LB_GETTEXT,(UINT)iItem,(LONG)(LPSTR)line);
    ePlayerPosDef = PlanetPositionDefined;
    // update the planet display
    SendDlgItemMessage(IDC_PLAYERPOS, LB_DELETESTRING,0,(LONG)0);
    SendDlgItemMessage(IDC_PLAYERPOS, LB_ADDSTRING,0,(LONG)line);
  }
}


//-----------------------------------
// new OnFriendsPlanetButton method
//-----------------------------------
void CPositionDialog::OnFriendsPlanetButton()
{
  LONG         iItem;
  char         line[LINE_LENGTH];

  // select a new planet
  iItem = SendDlgItemMessage(IDC_PLANETS, LB_GETCURSEL,0,0);
  if (iItem != LB_ERR)
  {
    // get the planet
    SendDlgItemMessage(IDC_PLANETS, LB_GETTEXT,(UINT)iItem,(LONG)(LPSTR)line);
    eFriendPosDef = PlanetPositionDefined;
    // update the planet display
    SendDlgItemMessage(IDC_FRIENDSPOS, LB_DELETESTRING,0,(LONG)0);
    SendDlgItemMessage(IDC_FRIENDSPOS, LB_ADDSTRING,0,(LONG)line);
  }
}


//-----------------------------------
// new OnEnemiesPlanetButton method
//-----------------------------------
void CPositionDialog::OnEnemiesPlanetButton()
{
  LONG         iItem;
  char         line[LINE_LENGTH];

  // select a new planet
  iItem = SendDlgItemMessage(IDC_PLANETS, LB_GETCURSEL,0,0);
  if (iItem != LB_ERR)
  {
    // get the planet
    SendDlgItemMessage(IDC_PLANETS, LB_GETTEXT,(UINT)iItem,(LONG)(LPSTR)line);
    eEnemyPosDef = PlanetPositionDefined;
    // update the planet display
    SendDlgItemMessage(IDC_ENEMIESPOS, LB_DELETESTRING,0,(LONG)0);
    SendDlgItemMessage(IDC_ENEMIESPOS, LB_ADDSTRING,0,(LONG)line);
  }
}


//-----------------------------------
// new OnNeutralsPlanetButton method
//-----------------------------------
void CPositionDialog::OnNeutralsPlanetButton()
{
  LONG         iItem;
  char         line[LINE_LENGTH];

  // select a new planet
  iItem = SendDlgItemMessage(IDC_PLANETS, LB_GETCURSEL,0,0);
  if (iItem != LB_ERR)
  {
    // get the planet
    SendDlgItemMessage(IDC_PLANETS, LB_GETTEXT,(UINT)iItem,(LONG)(LPSTR)line);
    eNeutralPosDef = PlanetPositionDefined;
    // update the planet display
    SendDlgItemMessage(IDC_NEUTRALSPOS, LB_DELETESTRING,0,(LONG)0);
    SendDlgItemMessage(IDC_NEUTRALSPOS, LB_ADDSTRING,0,(LONG)line);
  }
}


//-----------------------------------
// new OnHulksPlanetButton method
//-----------------------------------
void CPositionDialog::OnHulksPlanetButton()
{
  LONG         iItem;
  char         line[LINE_LENGTH];

  // select a new planet
  iItem = SendDlgItemMessage(IDC_PLANETS, LB_GETCURSEL,0,0);
  if (iItem != LB_ERR)
  {
    // get the planet
    SendDlgItemMessage(IDC_PLANETS, LB_GETTEXT,(UINT)iItem,(LONG)(LPSTR)line);
    eHulkPosDef = PlanetPositionDefined;
    // update the planet display
    SendDlgItemMessage(IDC_HULKSPOS, LB_DELETESTRING,0,(LONG)0);
    SendDlgItemMessage(IDC_HULKSPOS, LB_ADDSTRING,0,(LONG)line);
  }
}


//-----------------------------------
// new OnJumpGatePlanetButton method
//-----------------------------------
void CPositionDialog::OnJumpGatePlanetButton()
{
  LONG         iItem;
  char         line[LINE_LENGTH];

  // select a new planet
  iItem = SendDlgItemMessage(IDC_PLANETS, LB_GETCURSEL,0,0);
  if (iItem != LB_ERR)
  {
    // get the planet
    SendDlgItemMessage(IDC_PLANETS, LB_GETTEXT,(UINT)iItem,(LONG)(LPSTR)line);
    eJumpPosDef = PlanetPositionDefined;
    // update the planet display
    SendDlgItemMessage(IDC_JUMPPOS, LB_DELETESTRING,0,(LONG)0);
    SendDlgItemMessage(IDC_JUMPPOS, LB_ADDSTRING,0,(LONG)line);
  }
}


//-----------------------------------
// new OnPlayerCoordButton method
//-----------------------------------
void CPositionDialog::OnPlayerCoordButton()
{
  CString         strXPos, strYPos, strZPos;
  CPosCoordDialog PosCoordDialog;
  char            line[MAX_QB2_LINELENGTH], str[MAX_QB2_LINELENGTH];

  // init the strings
  strXPos = ""; strYPos = ""; strZPos = "";

  // check if coordinates are given already
  if (ePlayerPosDef == CoordPositionDefined)
  {
    // get it
    SendDlgItemMessage(IDC_PLAYERPOS, LB_GETTEXT,(UINT)0,(LONG)(LPSTR)line);
    GetNthWordInString(str, line, 1);
    strXPos = str;
    GetNthWordInString(str, line, 2);
    strYPos = str;
    GetNthWordInString(str, line, 3);
    strZPos = str;
  }

  // ask for new coordinates
  PosCoordDialog.Setup(strXPos, strYPos, strZPos);
  if (PosCoordDialog.DoDialog(&strXPos, &strYPos, &strZPos))
  {
    // set the new coordinates in the list box
    strXPos = strXPos + "   ";
    strXPos = strXPos + strYPos;
    strXPos = strXPos + "   ";
    strXPos = strXPos + strZPos;
    SendDlgItemMessage(IDC_PLAYERPOS, LB_DELETESTRING,0,(LONG)0);
    SendDlgItemMessage(IDC_PLAYERPOS, LB_ADDSTRING,0,(LONG)(const char *)strXPos);
    ePlayerPosDef = CoordPositionDefined;
  }
}


//-----------------------------------
// new OnFriendsCoordButton method
//-----------------------------------
void CPositionDialog::OnFriendsCoordButton()
{
  CString         strXPos, strYPos, strZPos;
  CPosCoordDialog PosCoordDialog;
  char            line[MAX_QB2_LINELENGTH], str[MAX_QB2_LINELENGTH];

  // init the strings
  strXPos = ""; strYPos = ""; strZPos = "";

  // check if coordinates are given already
  if (eFriendPosDef == CoordPositionDefined)
  {
    // get it
    SendDlgItemMessage(IDC_FRIENDSPOS, LB_GETTEXT,(UINT)0,(LONG)(LPSTR)line);
    GetNthWordInString(str, line, 1);
    strXPos = str;
    GetNthWordInString(str, line, 2);
    strYPos = str;
    GetNthWordInString(str, line, 3);
    strZPos = str;
  }

  // ask for new coordinates
  PosCoordDialog.Setup(strXPos, strYPos, strZPos);
  if (PosCoordDialog.DoDialog(&strXPos, &strYPos, &strZPos))
  {
    // set the new coordinates in the list box
    strXPos = strXPos + "   ";
    strXPos = strXPos + strYPos;
    strXPos = strXPos + "   ";
    strXPos = strXPos + strZPos;
    SendDlgItemMessage(IDC_FRIENDSPOS, LB_DELETESTRING,0,(LONG)0);
    SendDlgItemMessage(IDC_FRIENDSPOS, LB_ADDSTRING,0,(LONG)(const char *)strXPos);
    eFriendPosDef = CoordPositionDefined;
  }
}


//-----------------------------------
// new OnEnemiesCoordButton method
//-----------------------------------
void CPositionDialog::OnEnemiesCoordButton()
{
  CString         strXPos, strYPos, strZPos;
  CPosCoordDialog PosCoordDialog;
  char            line[MAX_QB2_LINELENGTH], str[MAX_QB2_LINELENGTH];

  // init the strings
  strXPos = ""; strYPos = ""; strZPos = "";

  // check if coordinates are given already
  if (eEnemyPosDef == CoordPositionDefined)
  {
    // get it
    SendDlgItemMessage(IDC_ENEMIESPOS, LB_GETTEXT,(UINT)0,(LONG)(LPSTR)line);
    GetNthWordInString(str, line, 1);
    strXPos = str;
    GetNthWordInString(str, line, 2);
    strYPos = str;
    GetNthWordInString(str, line, 3);
    strZPos = str;
  }

  // ask for new coordinates
  PosCoordDialog.Setup(strXPos, strYPos, strZPos);
  if (PosCoordDialog.DoDialog(&strXPos, &strYPos, &strZPos))
  {
    // set the new coordinates in the list box
    strXPos = strXPos + "   ";
    strXPos = strXPos + strYPos;
    strXPos = strXPos + "   ";
    strXPos = strXPos + strZPos;
    SendDlgItemMessage(IDC_ENEMIESPOS, LB_DELETESTRING,0,(LONG)0);
    SendDlgItemMessage(IDC_ENEMIESPOS, LB_ADDSTRING,0,(LONG)(const char *)strXPos);
    eEnemyPosDef = CoordPositionDefined;
  }
}


//-----------------------------------
// new OnNeutralsCoordButton method
//-----------------------------------
void CPositionDialog::OnNeutralsCoordButton()
{
  CString         strXPos, strYPos, strZPos;
  CPosCoordDialog PosCoordDialog;
  char            line[MAX_QB2_LINELENGTH], str[MAX_QB2_LINELENGTH];

  // init the strings
  strXPos = ""; strYPos = ""; strZPos = "";

  // check if coordinates are given already
  if (eNeutralPosDef == CoordPositionDefined)
  {
    // get it
    SendDlgItemMessage(IDC_NEUTRALSPOS, LB_GETTEXT,(UINT)0,(LONG)(LPSTR)line);
    GetNthWordInString(str, line, 1);
    strXPos = str;
    GetNthWordInString(str, line, 2);
    strYPos = str;
    GetNthWordInString(str, line, 3);
    strZPos = str;
  }

  // ask for new coordinates
  PosCoordDialog.Setup(strXPos, strYPos, strZPos);
  if (PosCoordDialog.DoDialog(&strXPos, &strYPos, &strZPos))
  {
    // set the new coordinates in the list box
    strXPos = strXPos + "   ";
    strXPos = strXPos + strYPos;
    strXPos = strXPos + "   ";
    strXPos = strXPos + strZPos;
    SendDlgItemMessage(IDC_NEUTRALSPOS, LB_DELETESTRING,0,(LONG)0);
    SendDlgItemMessage(IDC_NEUTRALSPOS, LB_ADDSTRING,0,(LONG)(const char *)strXPos);
    eNeutralPosDef = CoordPositionDefined;
  }
}




//-----------------------------------
// new OnHulksCoordButton method
//-----------------------------------
void CPositionDialog::OnHulksCoordButton()
{
  CString         strXPos, strYPos, strZPos;
  CPosCoordDialog PosCoordDialog;
  char            line[MAX_QB2_LINELENGTH], str[MAX_QB2_LINELENGTH];

  // init the strings
  strXPos = ""; strYPos = ""; strZPos = "";

  // check if coordinates are given already
  if (eHulkPosDef == CoordPositionDefined)
  {
    // get it
    SendDlgItemMessage(IDC_HULKSPOS, LB_GETTEXT,(UINT)0,(LONG)(LPSTR)line);
    GetNthWordInString(str, line, 1);
    strXPos = str;
    GetNthWordInString(str, line, 2);
    strYPos = str;
    GetNthWordInString(str, line, 3);
    strZPos = str;
  }

  // ask for new coordinates
  PosCoordDialog.Setup(strXPos, strYPos, strZPos);
  if (PosCoordDialog.DoDialog(&strXPos, &strYPos, &strZPos))
  {
    // set the new coordinates in the list box
    strXPos = strXPos + "   ";
    strXPos = strXPos + strYPos;
    strXPos = strXPos + "   ";
    strXPos = strXPos + strZPos;
    SendDlgItemMessage(IDC_HULKSPOS, LB_DELETESTRING,0,(LONG)0);
    SendDlgItemMessage(IDC_HULKSPOS, LB_ADDSTRING,0,(LONG)(const char *)strXPos);
    eHulkPosDef = CoordPositionDefined;
  }
}


//-----------------------------------
// new OnJumpGateCoordButton method
//-----------------------------------
void CPositionDialog::OnJumpGateCoordButton()
{
  CString         strXPos, strYPos, strZPos;
  CPosCoordDialog PosCoordDialog;
  char            line[MAX_QB2_LINELENGTH], str[MAX_QB2_LINELENGTH];

  // init the strings
  strXPos = ""; strYPos = ""; strZPos = "";

  // check if coordinates are given already
  if (eJumpPosDef == CoordPositionDefined)
  {
    // get it
    SendDlgItemMessage(IDC_JUMPPOS, LB_GETTEXT,(UINT)0,(LONG)(LPSTR)line);
    GetNthWordInString(str, line, 1);
    strXPos = str;
    GetNthWordInString(str, line, 2);
    strYPos = str;
    GetNthWordInString(str, line, 3);
    strZPos = str;
  }

  // ask for new coordinates
  PosCoordDialog.Setup(strXPos, strYPos, strZPos);
  if (PosCoordDialog.DoDialog(&strXPos, &strYPos, &strZPos))
  {
    // set the new coordinates in the list box
    strXPos = strXPos + "   ";
    strXPos = strXPos + strYPos;
    strXPos = strXPos + "   ";
    strXPos = strXPos + strZPos;
    SendDlgItemMessage(IDC_JUMPPOS, LB_DELETESTRING,0,(LONG)0);
    SendDlgItemMessage(IDC_JUMPPOS, LB_ADDSTRING,0,(LONG)(const char *)strXPos);
    eJumpPosDef = CoordPositionDefined;
  }
}


//-----------------------------------
// new OnPlayerClearButton method
//-----------------------------------
void CPositionDialog::OnPlayerClearButton()
{
  ePlayerPosDef = NoPositionDefined;
  SendDlgItemMessage(IDC_PLAYERPOS, LB_DELETESTRING,0,(LONG)0);
}


//-----------------------------------
// new OnFriendsClearButton method
//-----------------------------------
void CPositionDialog::OnFriendsClearButton()
{
  eFriendPosDef = NoPositionDefined;
  SendDlgItemMessage(IDC_FRIENDSPOS, LB_DELETESTRING,0,(LONG)0);
}


//-----------------------------------
// new OnEnemiesClearButton method
//-----------------------------------
void CPositionDialog::OnEnemiesClearButton()
{
  eEnemyPosDef = NoPositionDefined;
  SendDlgItemMessage(IDC_ENEMIESPOS, LB_DELETESTRING,0,(LONG)0);
}


//-----------------------------------
// new OnNeutralsClearButton method
//-----------------------------------
void CPositionDialog::OnNeutralsClearButton()
{
  eNeutralPosDef = NoPositionDefined;
  SendDlgItemMessage(IDC_NEUTRALSPOS, LB_DELETESTRING,0,(LONG)0);
}


//-----------------------------------
// new OnHulksClearButton method
//-----------------------------------
void CPositionDialog::OnHulksClearButton()
{
  eHulkPosDef = NoPositionDefined;
  SendDlgItemMessage(IDC_HULKSPOS, LB_DELETESTRING,0,(LONG)0);
}


//-----------------------------------
// new OnJumpGateClearButton method
//-----------------------------------
void CPositionDialog::OnJumpGateClearButton()
{
  eJumpPosDef = NoPositionDefined;
  SendDlgItemMessage(IDC_JUMPPOS, LB_DELETESTRING,0,(LONG)0);
}


//-----------------------------------
// do the dialog
//-----------------------------------
void CPositionDialog::DoDialog()
{
  // do it
  DoModal();
}



//-----------------------------------
// the message map
//-----------------------------------
BEGIN_MESSAGE_MAP(CPositionDialog, CModalDialog)
  ON_BN_CLICKED(IDC_PLAYERPLANET,   OnPlayerPlanetButton)
  ON_BN_CLICKED(IDC_ENEMIESPLANET,  OnEnemiesPlanetButton)
  ON_BN_CLICKED(IDC_FRIENDSPLANET,  OnFriendsPlanetButton)
  ON_BN_CLICKED(IDC_NEUTRALSPLANET, OnNeutralsPlanetButton)
  ON_BN_CLICKED(IDC_HULKSPLANET,    OnHulksPlanetButton)
  ON_BN_CLICKED(IDC_JUMPPLANET,     OnJumpGatePlanetButton)
  ON_BN_CLICKED(IDC_PLAYERCOORD,    OnPlayerCoordButton)
  ON_BN_CLICKED(IDC_ENEMIESCOORD,   OnEnemiesCoordButton)
  ON_BN_CLICKED(IDC_FRIENDSCOORD,   OnFriendsCoordButton)
  ON_BN_CLICKED(IDC_NEUTRALSCOORD,  OnNeutralsCoordButton)
  ON_BN_CLICKED(IDC_HULKSCOORD,     OnHulksCoordButton)
  ON_BN_CLICKED(IDC_JUMPCOORD,      OnJumpGateCoordButton)
  ON_BN_CLICKED(IDC_PLAYERCLEAR,    OnPlayerClearButton)
  ON_BN_CLICKED(IDC_ENEMIESCLEAR,   OnEnemiesClearButton)
  ON_BN_CLICKED(IDC_FRIENDSCLEAR,   OnFriendsClearButton)
  ON_BN_CLICKED(IDC_NEUTRALSCLEAR,  OnNeutralsClearButton)
  ON_BN_CLICKED(IDC_HULKSCLEAR,     OnHulksClearButton)
  ON_BN_CLICKED(IDC_JUMPCLEAR,      OnJumpGateClearButton)
END_MESSAGE_MAP()



// ----------- the position coordinates class ---------------------------------------

//-----------------------------------
// overriding the Constructor
//-----------------------------------
CPosCoordDialog::CPosCoordDialog(CWnd* pParentWnd) : CModalDialog("POSCOORDBOX", pParentWnd)
{
  // init
  strXPos = "0";  strYPos = "0";  strZPos = "0";
  bPosSet = FALSE;
}

//-----------------------------------
// overriding the OnInitDialog method
//-----------------------------------
BOOL CPosCoordDialog::OnInitDialog()
{
  // show the values to edit

  if (strXPos.GetLength() > 0)
    SendDlgItemMessage(IDC_XCOORD, WM_SETTEXT ,0,(LONG)(const char *)strXPos);

  if (strYPos.GetLength() > 0)
    SendDlgItemMessage(IDC_YCOORD, WM_SETTEXT ,0,(LONG)(const char *)strYPos);

  if (strZPos.GetLength() > 0)
    SendDlgItemMessage(IDC_ZCOORD, WM_SETTEXT ,0,(LONG)(const char *)strZPos);

  return TRUE;
}

//-----------------------------------
// setup data to edit
//-----------------------------------
void CPosCoordDialog::Setup(const char *strX, const char *strY, const char *strZ)
{
  strXPos = strX;  strYPos = strY; strZPos = strZ;
}


//-----------------------------------
// overriding the OnOK method
//-----------------------------------
void CPosCoordDialog::OnOK()
{
  CErrorDialog ErrorDialog(this);
  char str[MAX_QB2_LINELENGTH + 1];

  // get the x value
  SendDlgItemMessage(IDC_XCOORD, WM_GETTEXT,(WORD)MAX_QB2_LINELENGTH,(LONG)(LPSTR)str);
  StripString(str,str);
  // test it
  if (!IsNumber(str) || (str[0] == '\0'))
  {
    ErrorDialog.Report(ERR_COORDINATES, ERR_HEADER);
    // set the focus on the wrong value
    GetDlgItem(IDC_XCOORD)->SetFocus();
    SendDlgItemMessage(IDC_XCOORD, EM_SETSEL, 0, MAKELONG(0,32767));
    return;
  }
  // it is ok
  strXPos = str;

  // get the y value
  SendDlgItemMessage(IDC_YCOORD, WM_GETTEXT,(WORD)MAX_QB2_LINELENGTH,(LONG)(LPSTR)str);
  StripString(str,str);
  // test it
  if (!IsNumber(str) || (str[0] == '\0'))
  {
    ErrorDialog.Report(ERR_COORDINATES, ERR_HEADER);
    // set the focus on the wrong value
    GetDlgItem(IDC_YCOORD)->SetFocus();
    SendDlgItemMessage(IDC_YCOORD, EM_SETSEL, 0, MAKELONG(0,32767));
    return;
  }
  // it is ok
  strYPos = str;

  // get the z value
  SendDlgItemMessage(IDC_ZCOORD, WM_GETTEXT,(WORD)MAX_QB2_LINELENGTH,(LONG)(LPSTR)str);
  StripString(str,str);
  // test it
  if (!IsNumber(str) || (str[0] == '\0'))
  {
    ErrorDialog.Report(ERR_COORDINATES, ERR_HEADER);
    // set the focus on the wrong value
    GetDlgItem(IDC_ZCOORD)->SetFocus();
    SendDlgItemMessage(IDC_ZCOORD, EM_SETSEL, 0, MAKELONG(0,32767));
    return;
  }
  // it is ok
  strZPos = str;

  // all ok
  bPosSet = TRUE;
  EndDialog(IDOK);
}


//-----------------------------------
// do the dialog
//-----------------------------------
BOOL CPosCoordDialog::DoDialog(CString *strX, CString *strY, CString *strZ)
{
  // do it
  DoModal();
  if (bPosSet)
  {
    *strX = strXPos;  *strY = strYPos;  *strZ = strZPos;
    return TRUE;
  }
  return FALSE;
}


// ----------- the general options dialog class ---------------------------------------

//-----------------------------------
// overriding the Constructor
//-----------------------------------
CGeneralOptionsDialog::CGeneralOptionsDialog(CWnd* pParentWnd) : CModalDialog("GENERALOPTIONBOX", pParentWnd)
{
  // init
  pConfiguration = NULL;
}

//-----------------------------------
// overriding the OnInitDialog method
//-----------------------------------
BOOL CGeneralOptionsDialog::OnInitDialog()
{
  CString CStr, *pCStr, *pCStr1;
  POSITION pos, pos1;
  BOOL     bFirstRemark;

  // set the radio button boxes
  // ship change mode
  if (pConfiguration->GetGeneralMode()->IsSetShipChangeMode())
  {
    if (pConfiguration->GetGeneralMode()->GetShipChangeMode() == NoShipChange)
      CheckRadioButton(IDC_AUTOCHANGE, IDC_NOCHANGE, IDC_NOCHANGE); else
    if (pConfiguration->GetGeneralMode()->GetShipChangeMode() == ManualShipChange)
      CheckRadioButton(IDC_AUTOCHANGE, IDC_NOCHANGE, IDC_MANCHANGE); else
      CheckRadioButton(IDC_AUTOCHANGE, IDC_NOCHANGE, IDC_AUTOCHANGE);
  } else
    CheckRadioButton(IDC_AUTOCHANGE, IDC_NOCHANGE, IDC_NOCHANGE);

  // collision mode
  if (pConfiguration->GetGeneralMode()->IsSetCollisionMode())
  {
    if (pConfiguration->GetGeneralMode()->GetCollisionMode() == CollisionsOn)
      CheckRadioButton(IDC_COLLISIONSON, IDC_COLLISIONSOFFFORALL, IDC_COLLISIONSON); else
    if (pConfiguration->GetGeneralMode()->GetCollisionMode() == CollisionsOff)
      CheckRadioButton(IDC_COLLISIONSON, IDC_COLLISIONSOFFFORALL, IDC_COLLISIONSOFF); else
      CheckRadioButton(IDC_COLLISIONSON, IDC_COLLISIONSOFFFORALL, IDC_COLLISIONSOFFFORALL);
  } else
    CheckRadioButton(IDC_COLLISIONSON, IDC_COLLISIONSOFFFORALL, IDC_COLLISIONSON);

  // warp out mode
  if (pConfiguration->GetGeneralMode()->IsSetWarpOutMode())
  {
    if (pConfiguration->GetGeneralMode()->GetWarpOutMode() == Anytime)
      CheckRadioButton(IDC_WARPANYTIME, IDC_WARPSEQUENTIAL, IDC_WARPANYTIME); else
    if (pConfiguration->GetGeneralMode()->GetWarpOutMode() == WhenCleared)
      CheckRadioButton(IDC_WARPANYTIME, IDC_WARPSEQUENTIAL, IDC_WARPWHENCLEARED); else
      CheckRadioButton(IDC_WARPANYTIME, IDC_WARPSEQUENTIAL, IDC_WARPSEQUENTIAL);
  } else
    CheckRadioButton(IDC_WARPANYTIME, IDC_WARPSEQUENTIAL, IDC_WARPANYTIME);

  // perfect cloak mode
  if (pConfiguration->GetGeneralMode()->IsSetPerfectCloakMode())
  {
    if (pConfiguration->GetGeneralMode()->GetPerfectCloakMode() == PerfectCloakOff)
      CheckRadioButton(IDC_PERFECTCLOAKOFF, IDC_PERFECTCLOAKON, IDC_PERFECTCLOAKOFF); else
      CheckRadioButton(IDC_PERFECTCLOAKOFF, IDC_PERFECTCLOAKON, IDC_PERFECTCLOAKON);
  } else
    CheckRadioButton(IDC_PERFECTCLOAKOFF, IDC_PERFECTCLOAKON, IDC_PERFECTCLOAKOFF);

  // set the title and title lines
  if (pConfiguration->GetTitle()->IsSetTitle())
  {
    CStr = pConfiguration->GetTitle()->GetTitle();
    SendDlgItemMessage(IDC_TITLE, WM_SETTEXT ,0,(LONG)(const char *)CStr);
  }
  // limit the title edit box length
  SendDlgItemMessage(IDC_TITLE, EM_LIMITTEXT ,(WORD)MAX_TITLE,(LONG)0);


  // iterate through all title lines
  CStr = "";
  pos1 = (pConfiguration->GetTitle()->GetTitleVoices())->GetHeadPosition();
  for (pos = pConfiguration->GetTitle()->GetTitleLines()->GetHeadPosition(); pos != NULL; )
  {
    // get the lines, put it together and seperate it by CR LF
    pCStr = (CString *)pConfiguration->GetTitle()->GetTitleLines()->GetNext(pos);
    // get the optional voice file
    if (pos1 != NULL)
    {
      pCStr1 = (CString *)pConfiguration->GetTitle()->GetTitleVoices()->GetNext(pos1);
      if (!pCStr1->IsEmpty())
      {
        // add the voice file if present
        CStr = CStr + "[" + *pCStr1 + "] ";
      }
      CStr = CStr + *pCStr + "\015\012";
    }
  }
  // put them in the box
  SendDlgItemMessage(IDC_TITLELINES, WM_SETTEXT ,0,(LONG)(const char *)CStr);

  // set the epilog lines
  // iterate through all epilog lines
  CStr = "";
  pos1 = (pConfiguration->GetEpilog()->GetEpilogVoices())->GetHeadPosition();
  for (pos = pConfiguration->GetEpilog()->GetEpilogLines()->GetHeadPosition(); pos != NULL; )
  {
    // get the lines, put it together and seperate it by CR LF
    pCStr = (CString *)pConfiguration->GetEpilog()->GetEpilogLines()->GetNext(pos);
    // get the optional voice file
    if (pos1 != NULL)
    {
      pCStr1 = (CString *)pConfiguration->GetEpilog()->GetEpilogVoices()->GetNext(pos1);
      if (!pCStr1->IsEmpty())
      {
        // add the voice file if present
        CStr = CStr + "[" + *pCStr1 + "] ";
      }
      CStr = CStr + *pCStr + "\015\012";
    }
  }
  // put them in the box
  SendDlgItemMessage(IDC_EPILOGLINES, WM_SETTEXT ,0,(LONG)(const char *)CStr);

  // set the remarks
  // iterate through all remarks
  CStr = "";
  bFirstRemark = TRUE;
  for (pos = pConfiguration->GetRemarks()->GetHeadPosition(); pos != NULL; )
  {
    // get the lines, put it together and seperate it by CR LF
    pCStr = (CString *)pConfiguration->GetRemarks()->GetNext(pos);
    // skip the first remark line, this is the qb2edit brand
    if (!bFirstRemark)
      CStr = CStr + *pCStr + "\015\012";
    else
      bFirstRemark = FALSE;
  }
  // set title lines
  SendDlgItemMessage(IDC_REMARKS, WM_SETTEXT ,0,(LONG)(const char *)CStr);

  return TRUE;
}

//-----------------------------------
// setup data to edit
//-----------------------------------
void CGeneralOptionsDialog::Setup(CConfiguration *Configuration)
{
  // assign the pointer to the configuration
  pConfiguration = Configuration;
}

//-----------------------------------
// overriding the OnOK method
//-----------------------------------
void CGeneralOptionsDialog::OnOK()
{
  CString      *pCStr, *pCStr1, *pFirstCStr;
  CErrorDialog ErrorDialog;
  CMessageDialog Message;
  POSITION     pos;
  int          i, len, id;
  unsigned     j;
  BOOL         bTruncate, bEmptyLine, bFisRemoved, bFirstRemark, bTail;
  char         str[LINE_LENGTH + 1], strVoice[LINE_LENGTH + 1];


  bTruncate = FALSE;
  bFisRemoved = FALSE;
  bEmptyLine = FALSE;

  // get the new title from the edit box
  SendDlgItemMessage(IDC_TITLE, WM_GETTEXT,(WORD)MAX_QB2_LINELENGTH,(LONG)(LPSTR)str);
  // remove # characters from the title
  for (j = 0; j < strlen(str); j++) if (str[j] == '#')
  {
    str[j] = '*';
    bFisRemoved = TRUE;
  }
  // strip the title
  StripString(str, str);
  // if title is non empty, set the new title
  if (str[0] !='\0') pConfiguration->GetTitle()->SetTitle(str);
  // else clear title
                else pConfiguration->GetTitle()->UnsetTitle(); 

  // a warning if # removed
  if (bFisRemoved) Message.Report(WARN_TITFISREMOVED, WARN_HEADER);

  // clear the warning flag
  bFisRemoved = FALSE; 

  // clear the old title lines
  for (pos = pConfiguration->GetTitle()->GetTitleLines()->GetHeadPosition(); pos != NULL; )
  {
    // clear the objects in the list
    pCStr = (CString *)pConfiguration->GetTitle()->GetTitleLines()->GetNext(pos);
    delete pCStr;
  }
  //clear the list itself
  pConfiguration->GetTitle()->GetTitleLines()->RemoveAll();
  // clear the old title voices
  for (pos = pConfiguration->GetTitle()->GetTitleVoices()->GetHeadPosition(); pos != NULL; )
  {
    // clear the objects in the list
    pCStr = (CString *)pConfiguration->GetTitle()->GetTitleVoices()->GetNext(pos);
    delete pCStr;
  }
  //clear the list itself
  pConfiguration->GetTitle()->GetTitleVoices()->RemoveAll();
 
  // read the new title lines back from the edit box
  for (i = 0; i < ((CEdit *)GetDlgItem(IDC_TITLELINES))->GetLineCount(); i++)
  {
    // get the line
    len = ((CEdit *)GetDlgItem(IDC_TITLELINES))->GetLine(i, str, LINE_LENGTH - 1);
    // set string terminator
    str[len] = '\0';
    // format it
    if (FormatTitleEpilogLine(str, strVoice, &bTruncate, &bEmptyLine, &bFisRemoved))
    {
      // create a new object
      pCStr = new CString;
      if (pCStr == NULL)
      {
        ErrorDialog.Report(ERR_OUTOFMEMORY, ERR_HEADER);
        return;
      }
      pCStr1 = new CString;
      if (pCStr1 == NULL)
      {
        ErrorDialog.Report(ERR_OUTOFMEMORY, ERR_HEADER);
        return;
      }
      *pCStr = str;
      *pCStr1 = strVoice;
      // add it to the configuration
      pConfiguration->GetTitle()->GetTitleLines()->AddTail((CObject *)pCStr);
      pConfiguration->GetTitle()->GetTitleVoices()->AddTail((CObject *)pCStr1);
    }
  }

  // a warning if empty line
  if (bEmptyLine) Message.Report(WARN_TITLEEMPTY, WARN_HEADER);
  // a warning if truncated
  if (bTruncate) Message.Report(WARN_TITLETRUNC, WARN_HEADER);
  // a warning if # removed
  if (bFisRemoved) Message.Report(WARN_TITLEFISREMOVED, WARN_HEADER);

  // clear the warning flags
  bTruncate = FALSE;
  bFisRemoved = FALSE;
  bEmptyLine = FALSE;

  // do the same with the epilog: first clear the old epilog lines
  for (pos = pConfiguration->GetEpilog()->GetEpilogLines()->GetHeadPosition(); pos != NULL; )
  {
    // clear the objects in the list
    pCStr = (CString *)pConfiguration->GetEpilog()->GetEpilogLines()->GetNext(pos);
    delete pCStr;
  }
  //clear the list itself
  pConfiguration->GetEpilog()->GetEpilogLines()->RemoveAll();
  // clear the old epilog voices
  for (pos = pConfiguration->GetEpilog()->GetEpilogVoices()->GetHeadPosition(); pos != NULL; )
  {
    // clear the objects in the list
    pCStr = (CString *)pConfiguration->GetEpilog()->GetEpilogVoices()->GetNext(pos);
    delete pCStr;
  }
  //clear the list itself
  pConfiguration->GetEpilog()->GetEpilogVoices()->RemoveAll();
 

  // read the new epilog lines back from the edit box
  for (i = 0; i < ((CEdit *)GetDlgItem(IDC_EPILOGLINES))->GetLineCount(); i++)
  {
    // get the line
    len = ((CEdit *)GetDlgItem(IDC_EPILOGLINES))->GetLine(i, str, LINE_LENGTH - 1);
    // set string terminator
    str[len] = '\0';
    // format it
    if (FormatTitleEpilogLine(str, strVoice, &bTruncate, &bEmptyLine, &bFisRemoved))
    {
      // create a new object
      pCStr = new CString;
      if (pCStr == NULL)
      {
        ErrorDialog.Report(ERR_OUTOFMEMORY, ERR_HEADER);
        return;
      }
      pCStr1 = new CString;
      if (pCStr1 == NULL)
      {
        ErrorDialog.Report(ERR_OUTOFMEMORY, ERR_HEADER);
        return;
      }
      *pCStr = str;
      *pCStr1 = strVoice;
      // add it to the configuration
      pConfiguration->GetEpilog()->GetEpilogLines()->AddTail((CObject *)pCStr);
      pConfiguration->GetEpilog()->GetEpilogVoices()->AddTail((CObject *)pCStr1);
    }
  }

  // a warning if empty line
  if (bEmptyLine) Message.Report(WARN_EPILOGEMPTY, WARN_HEADER);
  // a warning if truncated
  if (bTruncate) Message.Report(WARN_EPILOGTRUNC, WARN_HEADER);
  // a warning if # removed
  if (bFisRemoved) Message.Report(WARN_EPILOGFISREMOVED, WARN_HEADER);

  // clear the old remarks except the first one, which is the qb2edit brand
  bFirstRemark = TRUE;
  for (pos = pConfiguration->GetRemarks()->GetHeadPosition(); pos != NULL; )
  {
    // clear the objects in the list
    pCStr = (CString *)pConfiguration->GetRemarks()->GetNext(pos);
    // delete if not the first
    if (!bFirstRemark) delete pCStr;
                  else
    {
      pFirstCStr = pCStr;
      bFirstRemark = FALSE;
    }
  }
  //clear the list itself
  pConfiguration->GetRemarks()->RemoveAll();

  // read the new remark lines back from the edit box
  bTail = TRUE;
  for (i = ((CEdit *)GetDlgItem(IDC_REMARKS))->GetLineCount(); i > 0; i--)
  {
    // get the line from the back to remove a blank line tail
    len = ((CEdit *)GetDlgItem(IDC_REMARKS))->GetLine(i-1, str, MAX_TITLELINE);
    // set string terminator
    str[len] = '\0';
    // see if the line is empty (we use the strVoice buffer here, because we
    // don't need it any longer)
    StripString(strVoice,str);
    // don't add a blank tail
    if (!(bTail && (strVoice[0] == '\0')))
    {
      // create a new object
      bTail = FALSE;
      pCStr = new CString;
      if (pCStr == NULL)
      {
        ErrorDialog.Report(ERR_OUTOFMEMORY, ERR_HEADER);
        return;
      }
      *pCStr = str;
      // add it to the configuration
      pConfiguration->GetRemarks()->AddHead((CObject *)pCStr);
    }
  }

  // add the first remark again, which is the brand
  pConfiguration->GetRemarks()->AddHead((CObject *)pFirstCStr);


  // get the collision mode
  id = GetCheckedRadioButton(IDC_COLLISIONSON, IDC_COLLISIONSOFFFORALL);
  if (id == IDC_COLLISIONSOFFFORALL)
  {
    pConfiguration->GetGeneralMode()->SetCollisionMode(CollisionsOffForAll);
  } else
  if (id == IDC_COLLISIONSOFF)
  {
    pConfiguration->GetGeneralMode()->SetCollisionMode(CollisionsOff);
  } else
  {
    pConfiguration->GetGeneralMode()->SetCollisionMode(CollisionsOn);
  }


  // get the ship change mode
  id = GetCheckedRadioButton(IDC_AUTOCHANGE, IDC_NOCHANGE);
  if (id == IDC_NOCHANGE)
  {
    pConfiguration->GetGeneralMode()->SetShipChangeMode(NoShipChange);
  } else
  if (id == IDC_MANCHANGE)
  {
    pConfiguration->GetGeneralMode()->SetShipChangeMode(ManualShipChange);
  } else
  {
    pConfiguration->GetGeneralMode()->SetShipChangeMode(AutoShipChange);
  }


  // get the warp out mode
  id = GetCheckedRadioButton(IDC_WARPANYTIME, IDC_WARPSEQUENTIAL);
  if (id == IDC_WARPSEQUENTIAL)
  {
    pConfiguration->GetGeneralMode()->SetWarpOutMode(Sequential);
  } else
  if (id == IDC_WARPWHENCLEARED)
  {
    pConfiguration->GetGeneralMode()->SetWarpOutMode(WhenCleared);
  } else
  {
    pConfiguration->GetGeneralMode()->SetWarpOutMode(Anytime);
  }

  // get the perfect cloak mode
  id = GetCheckedRadioButton(IDC_PERFECTCLOAKOFF, IDC_PERFECTCLOAKON);
  if (id == IDC_PERFECTCLOAKOFF)
  {
    pConfiguration->GetGeneralMode()->SetPerfectCloakMode(PerfectCloakOff);
  } else
  {
    pConfiguration->GetGeneralMode()->SetPerfectCloakMode(PerfectCloakOn);
  }

  EndDialog(IDOK);
}
 

//-----------------------------------
// new OnSetupSelfDestructButton method
//-----------------------------------
void CGeneralOptionsDialog::OnSetupSelfDestructButton()
{
  CSelfDestructDialog SelfDestructDialog;

  // do the self destruct dialog
  SelfDestructDialog.Setup(pConfiguration);
  SelfDestructDialog.DoDialog();
}

//-----------------------------------
// do the dialog
//-----------------------------------
void CGeneralOptionsDialog::DoDialog()
{
  // do it
  DoModal();
}

//-----------------------------------
// the message map
//-----------------------------------
BEGIN_MESSAGE_MAP(CGeneralOptionsDialog, CModalDialog)
  ON_BN_CLICKED(IDC_DEFSELFDESTRUCT,  OnSetupSelfDestructButton)
END_MESSAGE_MAP()


// ----------- the self destruct dialog class ----------------------------------

//-----------------------------------
// overriding the Constructor
//-----------------------------------
CSelfDestructDialog::CSelfDestructDialog(CWnd* pParentWnd) : CModalDialog("SELFDESTRUCTBOX", pParentWnd)
{
  // init
  pConfiguration = NULL;
}

//-----------------------------------
// overriding the OnInitDialog method
//-----------------------------------
BOOL CSelfDestructDialog::OnInitDialog()
{
  char        str[MAX_QB2_LINELENGTH + 1];
 
  // some basic checks
  if (pConfiguration == NULL) return TRUE;

  // limit the code edit box length
  SendDlgItemMessage(IDC_CODE, EM_LIMITTEXT ,(WORD)MAX_SELFDESTRUCTCODE,(LONG)0);

  // get the current self destruct mode and data
  // do we have an extendend self destruct set ?
  if (!pConfiguration->GetSelfDestruct()->IsSetMode())
  {
    // not set
    CheckRadioButton(IDC_SELFDESTRUCT1, IDC_SELFDESTRUCT4, IDC_SELFDESTRUCT1);
    // disable distance window
    GetDlgItem(IDC_OVERKILLDISTANCE)->EnableWindow(FALSE);
    // disable code window
    GetDlgItem(IDC_CODE)->EnableWindow(FALSE);
  } else
  {
    // mode is set
    // standard mode ?
    if (pConfiguration->GetSelfDestruct()->GetMode() == StandardDestruct)
    {
      // set the button
      CheckRadioButton(IDC_SELFDESTRUCT1, IDC_SELFDESTRUCT4, IDC_SELFDESTRUCT2);
      // disable distance window
      GetDlgItem(IDC_OVERKILLDISTANCE)->EnableWindow(FALSE);
    } else
    // overkill mode ?
    if (pConfiguration->GetSelfDestruct()->GetMode() == OverkillDestruct)
    {
      // set the button
      CheckRadioButton(IDC_SELFDESTRUCT1, IDC_SELFDESTRUCT4, IDC_SELFDESTRUCT3);
      // enable distance window
      GetDlgItem(IDC_OVERKILLDISTANCE)->EnableWindow(TRUE);
      // set distance value
      sprintf(str,"%lu",pConfiguration->GetSelfDestruct()->GetOverkillDistance());
      SendDlgItemMessage(IDC_OVERKILLDISTANCE, WM_SETTEXT,0,(LONG)(LPSTR)str);
    } else
    // selective overkill mode
    {
      // set the button
      CheckRadioButton(IDC_SELFDESTRUCT1, IDC_SELFDESTRUCT4, IDC_SELFDESTRUCT4);
      // enable distance window
      GetDlgItem(IDC_OVERKILLDISTANCE)->EnableWindow(TRUE);
      // set distance value
      sprintf(str,"%lu",pConfiguration->GetSelfDestruct()->GetOverkillDistance());
      SendDlgItemMessage(IDC_OVERKILLDISTANCE, WM_SETTEXT,0,(LONG)(LPSTR)str);
    }
    
    // set the code
    GetDlgItem(IDC_CODE)->EnableWindow(TRUE);
    sprintf(str,"%s",pConfiguration->GetSelfDestruct()->GetCode());
    SendDlgItemMessage(IDC_CODE, WM_SETTEXT,0,(LONG)(LPSTR)str);

  }

  return TRUE;
}

//-----------------------------------
// setup data to edit
//-----------------------------------
void CSelfDestructDialog::Setup(CConfiguration *Configuration)
{
  // assign the pointer to the configuration
  pConfiguration = Configuration;
}

//-----------------------------------
// overriding the OnOK method
//-----------------------------------
void CSelfDestructDialog::OnOK()
{
  CErrorDialog ErrorDialog;
  int          id;
  long         dist;
  char         str[MAX_QB2_LINELENGTH + 1];

  // check the mode selection
  id = GetCheckedRadioButton(IDC_SELFDESTRUCT1, IDC_SELFDESTRUCT4);
  if (id == IDC_SELFDESTRUCT1)
  {
    // no self destruct
    pConfiguration->GetSelfDestruct()->UnsetMode();
  } else
  if (id == IDC_SELFDESTRUCT2)
  {
    // standard self destruct
    pConfiguration->GetSelfDestruct()->SetMode(StandardDestruct);
    // get the code
    SendDlgItemMessage(IDC_CODE, WM_GETTEXT,(WORD)MAX_QB2_LINELENGTH,(LONG)(LPSTR)str);
    // check the code
    if (!TestSelfDestruct(str)) return;
    // set the code
    pConfiguration->GetSelfDestruct()->SetCode(str);
  } else
  if (id == IDC_SELFDESTRUCT3)
  {
    // overkill self destruct
    pConfiguration->GetSelfDestruct()->SetMode(OverkillDestruct);
    // get the distance
    SendDlgItemMessage(IDC_OVERKILLDISTANCE, WM_GETTEXT,(WORD)MAX_QB2_LINELENGTH,(LONG)(LPSTR)str);
    // distance specified ?
    if (str[0] == '\0')
    {
      // no, error
      ErrorDialog.Report(ERR_DESTRUCTNODISTSPEC, ERR_HEADER);
      return;
    }
    dist = atol(str);
    if (dist<0) dist = 0;
    pConfiguration->GetSelfDestruct()->SetOverkillDistance(dist);
    // get the code
    SendDlgItemMessage(IDC_CODE, WM_GETTEXT,(WORD)MAX_QB2_LINELENGTH,(LONG)(LPSTR)str);
    // check the code
    if (!TestSelfDestruct(str)) return;
    // set the code
    pConfiguration->GetSelfDestruct()->SetCode(str);
  } else
  {
    // selective overkill self destruct
    pConfiguration->GetSelfDestruct()->SetMode(SelectiveOverkillDestruct);
    // get the distance
    SendDlgItemMessage(IDC_OVERKILLDISTANCE, WM_GETTEXT,(WORD)MAX_QB2_LINELENGTH,(LONG)(LPSTR)str);
    // distance specified ?
    if (str[0] == '\0')
    {
      // no, error
      ErrorDialog.Report(ERR_DESTRUCTNODISTSPEC, ERR_HEADER);
      return;
    }
    dist = atol(str);
    if (dist<0) dist = 0;
    pConfiguration->GetSelfDestruct()->SetOverkillDistance(dist);
    // get the code
    SendDlgItemMessage(IDC_CODE, WM_GETTEXT,(WORD)MAX_QB2_LINELENGTH,(LONG)(LPSTR)str);
    // check the code
    if (!TestSelfDestruct(str)) return;
    // set the code
    pConfiguration->GetSelfDestruct()->SetCode(str);
  }

  EndDialog(IDOK);
}
 
//-----------------------------------
// new OnNoneButton method
//-----------------------------------
void CSelfDestructDialog::OnNoneButton()
{
  // disable code and distance box
  GetDlgItem(IDC_OVERKILLDISTANCE)->EnableWindow(FALSE);
  GetDlgItem(IDC_CODE)->EnableWindow(FALSE);
}

//-----------------------------------
// new OnStandardButton method
//-----------------------------------
void CSelfDestructDialog::OnStandardButton()
{
  // enable code box, disable distance box
  GetDlgItem(IDC_OVERKILLDISTANCE)->EnableWindow(FALSE);
  GetDlgItem(IDC_CODE)->EnableWindow(TRUE);
}

//-----------------------------------
// new OnOverkillButton method
//-----------------------------------
void CSelfDestructDialog::OnOverkillButton()
{
  // enable code and distance box
  GetDlgItem(IDC_OVERKILLDISTANCE)->EnableWindow(TRUE);
  GetDlgItem(IDC_CODE)->EnableWindow(TRUE);
}

//-----------------------------------
// new OnSelectiveOverkillButton method
//-----------------------------------
void CSelfDestructDialog::OnSelectiveOverkillButton()
{
  // enable code and distance box
  GetDlgItem(IDC_OVERKILLDISTANCE)->EnableWindow(TRUE);
  GetDlgItem(IDC_CODE)->EnableWindow(TRUE);
}

//-----------------------------------
// do the dialog
//-----------------------------------
void CSelfDestructDialog::DoDialog()
{
  // do it
  DoModal();
}


//-----------------------------------
// the message map
//-----------------------------------
BEGIN_MESSAGE_MAP(CSelfDestructDialog, CModalDialog)
  ON_BN_CLICKED(IDC_SELFDESTRUCT1,  OnNoneButton)
  ON_BN_CLICKED(IDC_SELFDESTRUCT2,  OnStandardButton)
  ON_BN_CLICKED(IDC_SELFDESTRUCT3,  OnOverkillButton)
  ON_BN_CLICKED(IDC_SELFDESTRUCT4,  OnSelectiveOverkillButton)
END_MESSAGE_MAP()



// ----------- the system options dialog class ---------------------------------------

//-----------------------------------
// overriding the Constructor
//-----------------------------------
CSystemOptionsDialog::CSystemOptionsDialog(CWnd* pParentWnd) : CModalDialog("SYSTEMOPTIONBOX", pParentWnd)
{
  // init
  pConfiguration = NULL;
  uSystemNum = 0;
  pSystem = NULL;
}

//-----------------------------------
// overriding the OnInitDialog method
//-----------------------------------
BOOL CSystemOptionsDialog::OnInitDialog()
{
  CString     CStr;
  CDialogData *pDialog;
  POSITION    pos;
  char        str[MAX_QB2_LINELENGTH + 1];

  // set the window caption
  CStr = pSystem->GetSystemName();
  CStr = CStr + SYSOPT_CAPTION;
  SendMessage(WM_SETTEXT,0,(LONG)(const char *)CStr);

  // set the radio button boxes
  // battle mode
  if (pSystem->IsSetBattleMode())
  {
    if (pSystem->GetBattleMode() == AIBattle)
      CheckRadioButton(IDC_AIBATTLE, IDC_STATBATTLE, IDC_AIBATTLE); else
      CheckRadioButton(IDC_AIBATTLE, IDC_STATBATTLE, IDC_STATBATTLE);
  } else
    CheckRadioButton(IDC_AIBATTLE, IDC_STATBATTLE, IDC_AIBATTLE); 

  // set the check boxes
  if (pSystem->IsSetPlayerWarpDisable())  CheckDlgButton(IDC_PLAYERWARPDISABLE, 1); 
  if (pSystem->IsSetFriendWarpDisable())  CheckDlgButton(IDC_FRIENDWARPDISABLE, 1); 
  if (pSystem->IsSetEnemyWarpDisable())   CheckDlgButton(IDC_ENEMYWARPDISABLE, 1); 
  if (pSystem->IsSetNeutralWarpDisable()) CheckDlgButton(IDC_NEUTRALWARPDISABLE, 1); 

  // set the stardate
  if (pSystem->IsSetStardate())
  {
    sprintf(str,"%li", pSystem->GetStardate());
    CStr = pConfiguration->GetTitle()->GetTitle();
    SendDlgItemMessage(IDC_STARDATE, WM_SETTEXT ,0,(LONG)str);
  }

  // set the starting dialogs
  CStr = "";
  for (pos = pSystem->GetStartingDialogs()->GetHeadPosition(); pos != NULL; )
  {
    // get the lines, put it together and seperate it by CR LF
    pDialog = (CDialogData *)pSystem->GetStartingDialogs()->GetNext(pos);
    if (!pDialog->GetVoiceFile().IsEmpty())
    {
      CStr = CStr + "[" + pDialog->GetVoiceFile() + "] ";
    }
    CStr = CStr + pDialog->GetSpeaker();
    CStr = CStr + ": ";
    CStr = CStr + pDialog->GetDialogLine();
    CStr = CStr + "\015\012";
  }
  // set dialog lines
  SendDlgItemMessage(IDC_STARTDIALOG, WM_SETTEXT ,0,(LONG)(const char *)CStr);

  // set the ending dialogs
  CStr = "";
  for (pos = pSystem->GetEndingDialogs()->GetHeadPosition(); pos != NULL; )
  {
    // get the lines, put it together and seperate it by CR LF
    pDialog = (CDialogData *)pSystem->GetEndingDialogs()->GetNext(pos);
    if (!pDialog->GetVoiceFile().IsEmpty())
    {
      CStr = CStr + "[" + pDialog->GetVoiceFile() + "] ";
    }
    CStr = CStr + pDialog->GetSpeaker();
    CStr = CStr + ": ";
    CStr = CStr + pDialog->GetDialogLine();
    CStr = CStr + "\015\012";
  }
  // set dialog lines
  SendDlgItemMessage(IDC_ENDDIALOG, WM_SETTEXT ,0,(LONG)(const char *)CStr);

  return TRUE;
}

//-----------------------------------
// setup data to edit
//-----------------------------------
void CSystemOptionsDialog::Setup(CConfiguration *Configuration, unsigned SystemNum)
{
  POSITION pos;

  // assign the pointer to the configuration
  pConfiguration = Configuration;
  // assign the system number
  uSystemNum = SystemNum;
  // get the system
  pos = (pConfiguration->GetSystems())->FindIndex(uSystemNum - 1);
  if (pos != NULL) pSystem = (CSystemData *)(pConfiguration->GetSystems())->GetAt(pos);
}

//-----------------------------------
// overriding the OnOK method
//-----------------------------------
void CSystemOptionsDialog::OnOK()
{
  CDialogData  *pDialog;
  CErrorDialog ErrorDialog;
  CMessageDialog Message;
  POSITION     pos;
  int          i, len, id;
  BOOL         bTruncate, bDefaultSpeaker, bEmptyLine, bFisRemoved;
  char         str[LINE_LENGTH], strSpeaker[LINE_LENGTH], strVoice[LINE_LENGTH];

  // get the battle mode
  id = GetCheckedRadioButton(IDC_AIBATTLE, IDC_STATBATTLE);
  if (id == IDC_AIBATTLE)
  {
    pSystem->SetBattleMode(AIBattle);
  } else
  {
    pSystem->SetBattleMode(StatisticBattle);
  }

  // get the warp disables
  if (IsDlgButtonChecked(IDC_PLAYERWARPDISABLE)  != 0) pSystem->SetPlayerWarpDisable(TRUE);
                                                  else pSystem->UnsetPlayerWarpDisable();
  if (IsDlgButtonChecked(IDC_FRIENDWARPDISABLE)  != 0) pSystem->SetFriendWarpDisable(TRUE);
                                                  else pSystem->UnsetFriendWarpDisable();
  if (IsDlgButtonChecked(IDC_ENEMYWARPDISABLE)   != 0) pSystem->SetEnemyWarpDisable(TRUE);
                                                  else pSystem->UnsetEnemyWarpDisable();
  if (IsDlgButtonChecked(IDC_NEUTRALWARPDISABLE) != 0) pSystem->SetNeutralWarpDisable(TRUE);
                                                  else pSystem->UnsetNeutralWarpDisable();

 
  // get the stardate
  SendDlgItemMessage(IDC_STARDATE, WM_GETTEXT,(WORD)MAX_QB2_LINELENGTH,(LONG)(LPSTR)str);
  // stardate specified ?
  if (str[0] == '\0') pSystem->UnsetStardate();
                 else pSystem->SetStardate(atol(str));

  // init the warning flags
  bTruncate = FALSE;
  bDefaultSpeaker = FALSE;
  bEmptyLine = FALSE;
  bFisRemoved = FALSE;

  // clear the old starting dialog  lines
  for (pos = pSystem->GetStartingDialogs()->GetHeadPosition(); pos != NULL; )
  {
    // clear the objects in the list
    pDialog = (CDialogData *)pSystem->GetStartingDialogs()->GetNext(pos);
    delete pDialog;
  }
  //clear the list itself
  pSystem->GetStartingDialogs()->RemoveAll();
 
  // read the new dialog lines back from the edit box
  for (i = 0; i < ((CEdit *)GetDlgItem(IDC_STARTDIALOG))->GetLineCount(); i++)
  {
    // get the line
    len = ((CEdit *)GetDlgItem(IDC_STARTDIALOG))->GetLine(i, str, LINE_LENGTH - 1);
    // set string terminator
    str[len] = '\0';
    // format it
    if (FormatDialog(str, strSpeaker, strVoice, &bTruncate, &bDefaultSpeaker, &bEmptyLine, &bFisRemoved))
    {
      // all is fine, create the object
      pDialog = new CDialogData;
      if (pDialog == NULL)
      {
        ErrorDialog.Report(ERR_OUTOFMEMORY, ERR_HEADER);
        return;
      }
      pDialog->SetSpeaker(strSpeaker);
      pDialog->SetDialogLine(str);
      pDialog->SetVoiceFile(strVoice);
      // add it to the list
      pSystem->GetStartingDialogs()->AddTail(pDialog);
    }
  }


  // clear the old ending dialog  lines
  for (pos = pSystem->GetEndingDialogs()->GetHeadPosition(); pos != NULL; )
  {
    // clear the objects in the list
    pDialog = (CDialogData *)pSystem->GetEndingDialogs()->GetNext(pos);
    delete pDialog;
  }
  //clear the list itself
  pSystem->GetEndingDialogs()->RemoveAll();
 
  // read the new dialog lines back from the edit box
  for (i = 0; i < ((CEdit *)GetDlgItem(IDC_ENDDIALOG))->GetLineCount(); i++)
  {
    // get the line
    len = ((CEdit *)GetDlgItem(IDC_ENDDIALOG))->GetLine(i, str, LINE_LENGTH - 1);
    // set string terminator
    str[len] = '\0';
    // format it
    if (FormatDialog(str, strSpeaker, strVoice, &bTruncate, &bDefaultSpeaker, &bEmptyLine, &bFisRemoved))
    {
      // all is fine, create the object
      pDialog = new CDialogData;
      if (pDialog == NULL)
      {
        ErrorDialog.Report(ERR_OUTOFMEMORY, ERR_HEADER);
        return;
      }
      pDialog->SetSpeaker(strSpeaker);
      pDialog->SetDialogLine(str);
      pDialog->SetVoiceFile(strVoice);
      // add it to the list
      pSystem->GetEndingDialogs()->AddTail(pDialog);
    }
  }

  // a warning if truncated
  if (bTruncate) Message.Report(WARN_DIALOGTRUNC, WARN_HEADER);
  // a warning if no speaker
  if (bDefaultSpeaker) Message.Report(WARN_DIALOGNOSPEAKER, WARN_HEADER);
  // a warning if # removed
  if (bFisRemoved) Message.Report(WARN_DIALOGFISREMOVED, WARN_HEADER);
  // a warning if empty dialog line
  if (bEmptyLine) Message.Report(WARN_DIALOGEMPTY, WARN_HEADER);

  EndDialog(IDOK);
}
 

//-----------------------------------
// do the dialog
//-----------------------------------
void CSystemOptionsDialog::DoDialog()
{
  // do it
  DoModal();
}


// ----------- the system jump options dialog class ---------------------------------------

//-----------------------------------
// overriding the Constructor
//-----------------------------------
CSystemJumpOptionsDialog::CSystemJumpOptionsDialog(CWnd* pParentWnd) : CModalDialog("JUMPOPTIONBOX", pParentWnd)
{
  // init
  pConfiguration = NULL;
  uSystemNum = 0;
  pSystem = NULL;
}

//-----------------------------------
// overriding the OnInitDialog method
//-----------------------------------
BOOL CSystemJumpOptionsDialog::OnInitDialog()
{
  CString     CStr;
  CDialogData *pDialog;
  POSITION    pos;
  char        str[MAX_QB2_LINELENGTH + 1];

  // set the window caption
  CStr = pSystem->GetSystemName();
  CStr = CStr + SYSJUMPOPT_CAPTION;
  SendMessage(WM_SETTEXT,0,(LONG)(const char *)CStr);

  // set the option check boxes
  if (pSystem->GetJumpOptions()->IsSetGateHidden())         CheckDlgButton(IDC_GATEHIDDEN, 1); 
  if (pSystem->GetJumpOptions()->IsSetJumpOnly())           CheckDlgButton(IDC_JUMPONLY, 1); 
  if (pSystem->GetJumpOptions()->IsSetClearJump())          CheckDlgButton(IDC_CLEARJUMP, 1); 
  if (pSystem->GetJumpOptions()->IsSetNoSpin())             CheckDlgButton(IDC_NOSPIN, 1); 
  if (pSystem->GetJumpOptions()->IsSetActOnHail())          CheckDlgButton(IDC_ACTONHAIL, 1); 
  if (pSystem->GetJumpOptions()->IsSetShowActivation())     CheckDlgButton(IDC_SHOWACT, 1); 
  if (pSystem->GetJumpOptions()->IsSetActivationDistance()) 
  {
    // set the box
    CheckDlgButton(IDC_ACTDIST, 1); 
    // enable distance window
    GetDlgItem(IDC_ACTIVATIONDISTANCE)->EnableWindow(TRUE);
    // set distance value
    sprintf(str,"%lu",pSystem->GetJumpOptions()->GetActivationDistance());
    SendDlgItemMessage(IDC_ACTIVATIONDISTANCE, WM_SETTEXT,0,(LONG)(LPSTR)str);
  } else
  {
    // disable distance window
    GetDlgItem(IDC_ACTIVATIONDISTANCE)->EnableWindow(FALSE);
  }

  // set the jump time
  if (pSystem->IsSetJumpTime()) 
  {
    // set the box
    CheckDlgButton(IDC_APPEAR, 1); 
    // enable time window
    GetDlgItem(IDC_JUMPTIME)->EnableWindow(TRUE);
    // set time value
    sprintf(str,"%lu",pSystem->GetJumpTime());
    SendDlgItemMessage(IDC_JUMPTIME, WM_SETTEXT,0,(LONG)(LPSTR)str);
  } else
  {
    // disable jump time window
    GetDlgItem(IDC_JUMPTIME)->EnableWindow(FALSE);
  }

  // set the gate appears dialogs
  CStr = "";
  for (pos = pSystem->GetJumpGateAppearsDialogs()->GetHeadPosition(); pos != NULL; )
  {
    // get the lines, put it together and seperate it by CR LF
    pDialog = (CDialogData *)pSystem->GetJumpGateAppearsDialogs()->GetNext(pos);
    if (!pDialog->GetVoiceFile().IsEmpty())
    {
      CStr = CStr + "[" + pDialog->GetVoiceFile() + "] ";
    }
    CStr = CStr + pDialog->GetSpeaker();
    CStr = CStr + ": ";
    CStr = CStr + pDialog->GetDialogLine();
    CStr = CStr + "\015\012";
  }
  // set dialog lines
  SendDlgItemMessage(IDC_APPEARSDIALOG, WM_SETTEXT ,0,(LONG)(const char *)CStr);

  // set the jump dialogs
  CStr = "";
  for (pos = pSystem->GetJumpDialogs()->GetHeadPosition(); pos != NULL; )
  {
    // get the lines, put it together and seperate it by CR LF
    pDialog = (CDialogData *)pSystem->GetJumpDialogs()->GetNext(pos);
    if (!pDialog->GetVoiceFile().IsEmpty())
    {
      CStr = CStr + "[" + pDialog->GetVoiceFile() + "] ";
    }
    CStr = CStr + pDialog->GetSpeaker();
    CStr = CStr + ": ";
    CStr = CStr + pDialog->GetDialogLine();
    CStr = CStr + "\015\012";
  }
  // set dialog lines
  SendDlgItemMessage(IDC_JUMPDIALOG, WM_SETTEXT ,0,(LONG)(const char *)CStr);


  return TRUE;
}

//-----------------------------------
// setup data to edit
//-----------------------------------
void CSystemJumpOptionsDialog::Setup(CConfiguration *Configuration, unsigned SystemNum)
{
  POSITION pos;

  // assign the pointer to the configuration
  pConfiguration = Configuration;
  // assign the system number
  uSystemNum = SystemNum;
  // get the system
  pos = (pConfiguration->GetSystems())->FindIndex(uSystemNum - 1);
  if (pos != NULL) pSystem = (CSystemData *)(pConfiguration->GetSystems())->GetAt(pos);
}

//-----------------------------------
// overriding the OnOK method
//-----------------------------------
void CSystemJumpOptionsDialog::OnOK()
{
  CDialogData  *pDialog;
  CErrorDialog ErrorDialog;
  CMessageDialog Message;
  POSITION     pos;
  int          i, len;
  BOOL         bTruncate, bDefaultSpeaker, bEmptyLine, bFisRemoved;
  char         str[LINE_LENGTH], strSpeaker[LINE_LENGTH], strVoice[LINE_LENGTH];

  // get the options
  if (IsDlgButtonChecked(IDC_GATEHIDDEN) != 0) pSystem->GetJumpOptions()->SetGateHidden();
                                          else pSystem->GetJumpOptions()->UnsetGateHidden();
  if (IsDlgButtonChecked(IDC_JUMPONLY)   != 0) pSystem->GetJumpOptions()->SetJumpOnly();
                                          else pSystem->GetJumpOptions()->UnsetJumpOnly();
  if (IsDlgButtonChecked(IDC_CLEARJUMP)  != 0) pSystem->GetJumpOptions()->SetClearJump();
                                          else pSystem->GetJumpOptions()->UnsetClearJump();
  if (IsDlgButtonChecked(IDC_NOSPIN)     != 0) pSystem->GetJumpOptions()->SetNoSpin();
                                          else pSystem->GetJumpOptions()->UnsetNoSpin();
  if (IsDlgButtonChecked(IDC_ACTONHAIL)  != 0) pSystem->GetJumpOptions()->SetActOnHail();
                                          else pSystem->GetJumpOptions()->UnsetActOnHail();
  if (IsDlgButtonChecked(IDC_SHOWACT)    != 0) pSystem->GetJumpOptions()->SetShowActivation();
                                          else pSystem->GetJumpOptions()->UnsetShowActivation();
  if (IsDlgButtonChecked(IDC_ACTDIST)    != 0)
  {
    // get the distance
    SendDlgItemMessage(IDC_ACTIVATIONDISTANCE, WM_GETTEXT,(WORD)LINE_LENGTH-1,(LONG)(LPSTR)str);
    // distance specified ?
    if (str[0] == '\0')
    {
      // no, error
      ErrorDialog.Report(ERR_JUMPNODISTSPEC, ERR_HEADER);
      return;
    }
    pSystem->GetJumpOptions()->SetActivationDistance(atol(str));
  } else
  {
    // not set
    pSystem->GetJumpOptions()->UnsetActivationDistance();
  }

  // get the jump time
  if (IsDlgButtonChecked(IDC_APPEAR)!= 0)
  {
    // get the time
    SendDlgItemMessage(IDC_JUMPTIME, WM_GETTEXT,(WORD)LINE_LENGTH-1,(LONG)(LPSTR)str);
    // time specified ?
    if (str[0] == '\0')
    {
      // no, error
      ErrorDialog.Report(ERR_JUMPNOTIMESPEC, ERR_HEADER);
      return;
    }
    pSystem->SetJumpTime(atol(str));
  } else
  {
    // not set
    pSystem->UnsetJumpTime();
  }

  // init the warning flags
  bTruncate = FALSE;
  bDefaultSpeaker = FALSE;
  bEmptyLine = FALSE;
  bFisRemoved = FALSE;

  // clear the old gate appears dialog lines
  for (pos = pSystem->GetJumpGateAppearsDialogs()->GetHeadPosition(); pos != NULL; )
  {
    // clear the objects in the list
    pDialog = (CDialogData *)pSystem->GetJumpGateAppearsDialogs()->GetNext(pos);
    delete pDialog;
  }
  //clear the list itself
  pSystem->GetJumpGateAppearsDialogs()->RemoveAll();
 
  // read the new dialog lines back from the edit box
  for (i = 0; i < ((CEdit *)GetDlgItem(IDC_APPEARSDIALOG))->GetLineCount(); i++)
  {
    // get the line
    len = ((CEdit *)GetDlgItem(IDC_APPEARSDIALOG))->GetLine(i, str, LINE_LENGTH - 1);
    // set string terminator
    str[len] = '\0';
    // format it
    if (FormatDialog(str, strSpeaker, strVoice, &bTruncate, &bDefaultSpeaker, &bEmptyLine, &bFisRemoved))
    {
      // all is fine, create the object
      pDialog = new CDialogData;
      if (pDialog == NULL)
      {
        ErrorDialog.Report(ERR_OUTOFMEMORY, ERR_HEADER);
        return;
      }
      pDialog->SetSpeaker(strSpeaker);
      pDialog->SetDialogLine(str);
      pDialog->SetVoiceFile(strVoice);
      // add it to the list
      pSystem->GetJumpGateAppearsDialogs()->AddTail(pDialog);
    }
  }


  // clear the old jump dialog lines
  for (pos = pSystem->GetJumpDialogs()->GetHeadPosition(); pos != NULL; )
  {
    // clear the objects in the list
    pDialog = (CDialogData *)pSystem->GetJumpDialogs()->GetNext(pos);
    delete pDialog;
  }
  //clear the list itself
  pSystem->GetJumpDialogs()->RemoveAll();
 
  // read the new dialog lines back from the edit box
  for (i = 0; i < ((CEdit *)GetDlgItem(IDC_JUMPDIALOG))->GetLineCount(); i++)
  {
    // get the line
    len = ((CEdit *)GetDlgItem(IDC_JUMPDIALOG))->GetLine(i, str, LINE_LENGTH - 1);
    // set string terminator
    str[len] = '\0';
    // format it
    if (FormatDialog(str, strSpeaker, strVoice, &bTruncate, &bDefaultSpeaker, &bEmptyLine, &bFisRemoved))
    {
      // all is fine, create the object
      pDialog = new CDialogData;
      if (pDialog == NULL)
      {
        ErrorDialog.Report(ERR_OUTOFMEMORY, ERR_HEADER);
        return;
      }
      pDialog->SetSpeaker(strSpeaker);
      pDialog->SetDialogLine(str);
      pDialog->SetVoiceFile(strVoice);
      // add it to the list
      pSystem->GetJumpDialogs()->AddTail(pDialog);
    }
  }

  // a warning if truncated
  if (bTruncate) Message.Report(WARN_DIALOGTRUNC, WARN_HEADER);
  // a warning if no speaker
  if (bDefaultSpeaker) Message.Report(WARN_DIALOGNOSPEAKER, WARN_HEADER);
  // a warning if # removed
  if (bFisRemoved) Message.Report(WARN_DIALOGFISREMOVED, WARN_HEADER);
  // a warning if empty dialog line
  if (bEmptyLine) Message.Report(WARN_DIALOGEMPTY, WARN_HEADER);

  EndDialog(IDOK);
}
 
//-----------------------------------
// new OnAppearsButton method
//-----------------------------------
void CSystemJumpOptionsDialog::OnAppearsButton()
{
  // enable time box if apear box is checked
  if (IsDlgButtonChecked(IDC_APPEAR)!= 0)
  {
    GetDlgItem(IDC_JUMPTIME)->EnableWindow(TRUE);
  } else
  {
    GetDlgItem(IDC_JUMPTIME)->EnableWindow(FALSE);
  }
}

//-----------------------------------
// new OnActDistButton method
//-----------------------------------
void CSystemJumpOptionsDialog::OnActDistButton()
{
  // enable distance box if act dist box is checked
  if (IsDlgButtonChecked(IDC_ACTDIST)!= 0)
  {
    GetDlgItem(IDC_ACTIVATIONDISTANCE)->EnableWindow(TRUE);
  } else
  {
    GetDlgItem(IDC_ACTIVATIONDISTANCE)->EnableWindow(FALSE);
  }
}

//-----------------------------------
// do the dialog
//-----------------------------------
void CSystemJumpOptionsDialog::DoDialog()
{
  // do it
  DoModal();
}


//-----------------------------------
// the message map
//-----------------------------------
BEGIN_MESSAGE_MAP(CSystemJumpOptionsDialog, CModalDialog)
  ON_BN_CLICKED(IDC_APPEAR,  OnAppearsButton)
  ON_BN_CLICKED(IDC_ACTDIST, OnActDistButton)
END_MESSAGE_MAP()


// ----------- the system jump destination dialog class ---------------------------------------

//-----------------------------------
// overriding the Constructor
//-----------------------------------
CSystemJumpDestinationDialog::CSystemJumpDestinationDialog(CWnd* pParentWnd) : CModalDialog("JUMPDESTINATIONBOX", pParentWnd)
{
  // init
  pConfiguration = NULL;
  uSystemNum = 0;
  DestSystemName = "";
  bShowAllSystems = FALSE;
}

//-----------------------------------
// overriding the OnInitDialog method
//-----------------------------------
BOOL CSystemJumpDestinationDialog::OnInitDialog()
{

  CString      CStr;
  LONG         iItem;

  // some basic checks
  if (pConfiguration == NULL) return TRUE;
  if (uSystemNum == 0) return TRUE;
  if (pSystem == NULL) return TRUE;

  // set the window caption
  CStr = pSystem->GetSystemName();
  CStr = CStr + SYSJUMPDEST_CAPTION;
  SendMessage(WM_SETTEXT,0,(LONG)(const char *)CStr);

  // clear the note
  SendDlgItemMessage(IDC_NOTE, WM_SETTEXT,0,(LONG)"");

  // place the destination in the list box
  if (pSystem->IsSetJumpDestination())
  {
    DestSystemName = pSystem->GetJumpDestination();
    SendDlgItemMessage(IDC_DESTINATION, LB_ADDSTRING,0,(LONG)(const char *)DestSystemName);
    // show a note if the system is not contained in the battle yet
    if (!(pConfiguration->CheckJumpDestination(DestSystemName)))
      SendDlgItemMessage(IDC_NOTE, WM_SETTEXT,0,(LONG)NOTE_JUMPDEST);
  }

  // create the available systems list
  ShowSystems();

  // set the list selection radio button
  if (bShowAllSystems) CheckDlgButton(IDC_SHOWALLSYS, 1);
                  else CheckDlgButton(IDC_SHOWDEFINEDSYS, 1);

  // check if we have a selection in the general avaibale systems list box,
  // enable select button if so
  iItem = SendDlgItemMessage(IDC_AVSYS, LB_GETCURSEL,0,0);
  if (iItem != LB_ERR)
  {
    GetDlgItem(IDC_SELECT)->EnableWindow(TRUE);
  }

  // check if we have a selection in the destination system list box,
  // enable remove button if so
  iItem = SendDlgItemMessage(IDC_DESTINATION, LB_GETCURSEL,0,0);
  if (iItem != LB_ERR)
  {
    GetDlgItem(IDC_REMOVE)->EnableWindow(TRUE);
  }

  return TRUE;
}

//-----------------------------------
// setup data to edit
//-----------------------------------
void CSystemJumpDestinationDialog::Setup(CConfiguration *Configuration, unsigned SystemNum)
{
  POSITION pos;

  // assign the pointer to the configuration
  pConfiguration = Configuration;
  // assign the system number
  uSystemNum = SystemNum;
  // get the system
  pos = (pConfiguration->GetSystems())->FindIndex(uSystemNum - 1);
  if (pos != NULL) pSystem = (CSystemData *)(pConfiguration->GetSystems())->GetAt(pos);
}


//-----------------------------------
// show systems
//-----------------------------------
void CSystemJumpDestinationDialog::ShowSystems()
{
  POSITION pos;
  CString  CStr;

  // clear the list box first
  SendDlgItemMessage(IDC_AVSYS, LB_RESETCONTENT,0,(LONG)(const char *)CStr);

  if (bShowAllSystems) 
  {
    // show all available systems
    AvailableSystemsAndPlanets.FillListBoxWithSystems(this, IDC_AVSYS);
    // set the corresponding caption
    SendDlgItemMessage(IDC_AVSYSCAPTION, WM_SETTEXT,0,(LONG)DESTAVSYS_CAPTION1);
  }
                  else
  {
    // show only the defined systems
    if ((pConfiguration->GetSystems())->GetCount() > 0)
    {
      // iterate through all systems
      for (pos = (pConfiguration->GetSystems())->GetHeadPosition(); pos != NULL; )
      {
        // get system
        CStr = ((CSystemData *)(pConfiguration->GetSystems())->GetNext(pos))->GetSystemName();
        // put it in the box
        SendDlgItemMessage(IDC_AVSYS, LB_ADDSTRING,0,(LONG)(const char *)CStr);
      }
    }
    // set the corresponding caption
    SendDlgItemMessage(IDC_AVSYSCAPTION, WM_SETTEXT,0,(LONG)DESTAVSYS_CAPTION2);
  }
}


//-----------------------------------
// overriding the OnOK method
//-----------------------------------
void CSystemJumpDestinationDialog::OnOK()
{
  // set the selected destination system
  if (DestSystemName.GetLength() > 0) pSystem->SetJumpDestination(DestSystemName);
                                 else pSystem->UnsetJumpDestination();

  EndDialog(IDOK);
}
 
//-----------------------------------
// new OnSelectButton method
//-----------------------------------
void CSystemJumpDestinationDialog::OnSelectButton()
{
  LONG         iItem;

  // select a new destination system
  iItem = SendDlgItemMessage(IDC_AVSYS, LB_GETCURSEL,0,0);
  if (iItem != LB_ERR)
  {
    // get the system, store the data temporarilly 
    // (the data is finally set when ok is pressed, this leaves the possibility to cancel the action)
    ((CListBox *)GetDlgItem(IDC_AVSYS))->GetText(iItem, DestSystemName);
 
    // update the list box
    SendDlgItemMessage(IDC_DESTINATION, LB_DELETESTRING,0,(LONG)0);
    SendDlgItemMessage(IDC_DESTINATION, LB_ADDSTRING,0,(LONG)(const char *)DestSystemName);

    // and show a note if the system is not contained in the battle yet
    if (!(pConfiguration->CheckJumpDestination(DestSystemName)))
      SendDlgItemMessage(IDC_NOTE, WM_SETTEXT,0,(LONG)NOTE_JUMPDEST);
    else 
      SendDlgItemMessage(IDC_NOTE, WM_SETTEXT,0,(LONG)"");
  }
}

//-----------------------------------
// new OnRemoveButton method
//-----------------------------------
void CSystemJumpDestinationDialog::OnRemoveButton()
{
  LONG         iItem;

  // remove the destination syste,
  iItem = SendDlgItemMessage(IDC_DESTINATION, LB_GETCURSEL,0,0);
  if (iItem != LB_ERR)
  {
    // remove it in the temporary storage
    DestSystemName = "";
    // clear the note
    SendDlgItemMessage(IDC_NOTE, WM_SETTEXT,0,(LONG)"");
    // update the list box
    SendDlgItemMessage(IDC_DESTINATION, LB_DELETESTRING,0,(LONG)0);
    // we loose the selection, so disable the remove button
    GetDlgItem(IDC_REMOVE)->EnableWindow(FALSE);
  }
}


//-----------------------------------
// new OnSelectSysButton method
//-----------------------------------
void CSystemJumpDestinationDialog::OnSelectSysButton()
{
  // a click on one of the show system selection radio button
  // find out which one
  if (IsDlgButtonChecked(IDC_SHOWALLSYS) != 0) bShowAllSystems = TRUE;
                                          else bShowAllSystems = FALSE;
  // update system list
  ShowSystems();
 }


//-----------------------------------
// do the dialog
//-----------------------------------
void CSystemJumpDestinationDialog::DoDialog()
{
  // do it
  DoModal();
}


//-----------------------------------
// new OnAvSelChange method
//-----------------------------------
void CSystemJumpDestinationDialog::OnAvSelChange()
{
  LONG iItem;

  // selection in the general available systems listbox has changed

  // check if we have a selection
  iItem = SendDlgItemMessage(IDC_AVSYS, LB_GETCURSEL,0,0);
  if (iItem != LB_ERR)
  {
    // unlock the select button
    GetDlgItem(IDC_SELECT)->EnableWindow(TRUE);
  } else
  {
    // lock the select button
    GetDlgItem(IDC_SELECT)->EnableWindow(FALSE);
  }
}


//-----------------------------------
// new OnDestinationSelChange method
//-----------------------------------
void CSystemJumpDestinationDialog::OnDestinationSelChange()
{
  LONG iItem;

  // selection in the destination listbox has changed

  // check if we have a selection
  iItem = SendDlgItemMessage(IDC_DESTINATION, LB_GETCURSEL,0,0);
  if (iItem != LB_ERR)
  {
    // unlock the remove button
    GetDlgItem(IDC_REMOVE)->EnableWindow(TRUE);
  } else
  {
    // lock the remove button
    GetDlgItem(IDC_REMOVE)->EnableWindow(FALSE);
  }
}


//-----------------------------------
// the message map
//-----------------------------------
BEGIN_MESSAGE_MAP(CSystemJumpDestinationDialog, CModalDialog)
  ON_BN_CLICKED(IDC_SELECT,         OnSelectButton)
  ON_BN_CLICKED(IDC_REMOVE,         OnRemoveButton)
  ON_BN_CLICKED(IDC_SHOWALLSYS,     OnSelectSysButton)
  ON_BN_CLICKED(IDC_SHOWDEFINEDSYS, OnSelectSysButton)
  ON_LBN_SELCHANGE(IDC_AVSYS,       OnAvSelChange)
  ON_LBN_SELCHANGE(IDC_DESTINATION, OnDestinationSelChange)
END_MESSAGE_MAP()





// ----------- the neutral behavior and action dialog class ----------------------------------

//-----------------------------------
// overriding the Constructor
//-----------------------------------
CNeutralBehaviorDialog::CNeutralBehaviorDialog(CWnd* pParentWnd) : CModalDialog("NBEHAVIORBOX", pParentWnd)
{
  // init
  pConfiguration = NULL;
  uSystemNum = 0;
  pSystem = NULL;
}

//-----------------------------------
// overriding the OnInitDialog method
//-----------------------------------
BOOL CNeutralBehaviorDialog::OnInitDialog()
{
  CString CStr;
  char    str[MAX_QB2_LINELENGTH + 1];
  
  // some basic checks
  if (pConfiguration == NULL) return TRUE;
  if (pSystem == NULL) return TRUE;

  // the the window caption
  CStr = BEHAVIOR_CAPTION;
  CStr = CStr + pSystem->GetSystemName();
  SendMessage(WM_SETTEXT,0,(LONG)(const char *)CStr);

  // get the current behavior
  if (pSystem->IsSetNeutralBehavior())
  {
    if (pSystem->GetNeutralBehavior() == EnemyWhenAttacked)
    {
       CheckRadioButton(IDC_BEHAVIOR1, IDC_BEHAVIOR4, IDC_BEHAVIOR1);
       // disable time window
       GetDlgItem(IDC_BEHAVIORTIME)->EnableWindow(FALSE);
    } else
    if (pSystem->GetNeutralBehavior() == EnemyAfter)
    {
       CheckRadioButton(IDC_BEHAVIOR1, IDC_BEHAVIOR4, IDC_BEHAVIOR2);
       // enable time window
       GetDlgItem(IDC_BEHAVIORTIME)->EnableWindow(TRUE);
       // set time value
       sprintf(str,"%lu",pSystem->GetNeutralBehaviorTime());
       SendDlgItemMessage(IDC_BEHAVIORTIME, WM_SETTEXT,0,(LONG)(LPSTR)str);
    } else
    if (pSystem->GetNeutralBehavior() == FriendAfter)
    {
       CheckRadioButton(IDC_BEHAVIOR1, IDC_BEHAVIOR4, IDC_BEHAVIOR3);
       // enable time window
       GetDlgItem(IDC_BEHAVIORTIME)->EnableWindow(TRUE);
       // set time value
       sprintf(str,"%lu",pSystem->GetNeutralBehaviorTime());
       SendDlgItemMessage(IDC_BEHAVIORTIME, WM_SETTEXT,0,(LONG)(LPSTR)str);
    } else
    {
       CheckRadioButton(IDC_BEHAVIOR1, IDC_BEHAVIOR4, IDC_BEHAVIOR4);
       // disable time window
       GetDlgItem(IDC_BEHAVIORTIME)->EnableWindow(FALSE);
    }
  } else
  {
    // default
    CheckRadioButton(IDC_BEHAVIOR1, IDC_BEHAVIOR4, IDC_BEHAVIOR1);
    // disable time window
    GetDlgItem(IDC_BEHAVIORTIME)->EnableWindow(FALSE);
  }

  // get the current actions
  // warp in
  if (pSystem->IsSetNeutralWarpInTime())
  {
    // warp in time set, set the appropriate radio button
    CheckRadioButton(IDC_NWARPIN1, IDC_NWARPIN2, IDC_NWARPIN2);
    // enable time window
    GetDlgItem(IDC_NWARPINTIME)->EnableWindow(TRUE);
    // set time value
    sprintf(str,"%lu",pSystem->GetNeutralWarpInTime());
    SendDlgItemMessage(IDC_NWARPINTIME, WM_SETTEXT,0,(LONG)(LPSTR)str);
  } else
  {
    // no warp in set, set the button and disable time window
    CheckRadioButton(IDC_NWARPIN1, IDC_NWARPIN2, IDC_NWARPIN1);
    // disable time window
    GetDlgItem(IDC_NWARPINTIME)->EnableWindow(FALSE);
  }

  // warp out
  if (pSystem->IsSetNeutralWarpOutTime())
  {
    // time given or warp out when cleared ?
    if (pSystem->GetNeutralWarpOutTime() < 0)
    {
      // when cleared, set the appropriate radio button and disable time window
      CheckRadioButton(IDC_NWARPOUT1, IDC_NWARPOUT3, IDC_NWARPOUT3);
      GetDlgItem(IDC_NWARPOUTTIME)->EnableWindow(FALSE);
    } else
    {
      // warp out time set, set the appropriate radio button
      CheckRadioButton(IDC_NWARPOUT1, IDC_NWARPOUT3, IDC_NWARPOUT2);
      // enable time window
      GetDlgItem(IDC_NWARPOUTTIME)->EnableWindow(TRUE);
      // set time value
      sprintf(str,"%lu",pSystem->GetNeutralWarpOutTime());
      SendDlgItemMessage(IDC_NWARPOUTTIME, WM_SETTEXT,0,(LONG)(LPSTR)str);
    }
  } else
  {
    // no warp out set, set the button and disable time window
    CheckRadioButton(IDC_NWARPOUT1, IDC_NWARPOUT3, IDC_NWARPOUT1);
    // disable time window
    GetDlgItem(IDC_NWARPOUTTIME)->EnableWindow(FALSE);
  }

  return TRUE;
}

//-----------------------------------
// setup data to edit
//-----------------------------------
void CNeutralBehaviorDialog::Setup(CConfiguration *Configuration, unsigned SystemNum)
{
  POSITION pos;

  // assign the pointer to the configuration
  pConfiguration = Configuration;
  // assign the system number
  uSystemNum = SystemNum;
  // get the system
  pos = (pConfiguration->GetSystems())->FindIndex(uSystemNum - 1);
  if (pos != NULL) pSystem = (CSystemData *)(pConfiguration->GetSystems())->GetAt(pos);
}

//-----------------------------------
// overriding the OnOK method
//-----------------------------------
void CNeutralBehaviorDialog::OnOK()
{
  CErrorDialog ErrorDialog;
  CMessageDialog Message;
  int          id;
  long         t;
  char         str[MAX_QB2_LINELENGTH + 1];

  // check the warp in selection
  id = GetCheckedRadioButton(IDC_NWARPIN1, IDC_NWARPIN2);
  if (id == IDC_NWARPIN1)
  {
    // no warp in
    pSystem->UnsetNeutralWarpInTime();
  } else
  {
    // get the time
    SendDlgItemMessage(IDC_NWARPINTIME, WM_GETTEXT,(WORD)MAX_QB2_LINELENGTH,(LONG)(LPSTR)str);
    // time specified ?
    if (str[0] == '\0')
    {
      // no, error
      ErrorDialog.Report(ERR_NWARPINNOTIMESPEC, ERR_HEADER);
      return;
    }
    t = atol(str);
    if (t<0) t = 0;
    pSystem->SetNeutralWarpInTime(t);
  }

  // check the warp out selection
  id = GetCheckedRadioButton(IDC_NWARPOUT1, IDC_NWARPOUT3);
  if (id == IDC_NWARPOUT1)
  {
    // no warp out
    pSystem->UnsetNeutralWarpOutTime();
  } else
  if (id == IDC_NWARPOUT2)
  {
    // get the time
    SendDlgItemMessage(IDC_NWARPOUTTIME, WM_GETTEXT,(WORD)MAX_QB2_LINELENGTH,(LONG)(LPSTR)str);
    // time specified ?
    if (str[0] == '\0')
    {
      // no, error
      ErrorDialog.Report(ERR_NWARPOUTNOTIMESPEC, ERR_HEADER);
      return;
    }
    t = atol(str);
    if (t<0) t = 0;
    pSystem->SetNeutralWarpOutTime(t);
    // check and give a warning when warp out time is less then warp in time
    if (pSystem->IsSetNeutralWarpInTime())
    {
      if (t < pSystem->GetNeutralWarpInTime()) Message.Report(WARN_NWARPOUTTIME, WARN_HEADER);
    }
  } else
  {
    // warp out when cleared
    pSystem->SetNeutralWarpOutTime(-1);
  }

  // check the behavior selection
  id = GetCheckedRadioButton(IDC_BEHAVIOR1, IDC_BEHAVIOR4);
  if (id == IDC_BEHAVIOR4)
  {
    pSystem->SetNeutralBehavior(RandomBehavior); 
  } else
  if (id == IDC_BEHAVIOR3)
  {
    pSystem->SetNeutralBehavior(FriendAfter); 
    // get the time
    SendDlgItemMessage(IDC_BEHAVIORTIME, WM_GETTEXT,(WORD)MAX_QB2_LINELENGTH,(LONG)(LPSTR)str);
    // time specified ?
    if (str[0] == '\0')
    {
      // no, error
      ErrorDialog.Report(ERR_NBEHAVNOTIMESPEC, ERR_HEADER);
      return;
    }
    t = atol(str);
    if (t<0) t = 0;
    pSystem->SetNeutralBehaviorTime(t);
    // check and give a warning when behavior change time is less then warp in time
    if (pSystem->IsSetNeutralWarpInTime())
    {
      if (t < pSystem->GetNeutralWarpInTime()) Message.Report(WARN_NBEHAVTIME, WARN_HEADER);
    }

  } else
  if (id == IDC_BEHAVIOR2)
  {
    pSystem->SetNeutralBehavior(EnemyAfter); 
    // get the time
    SendDlgItemMessage(IDC_BEHAVIORTIME, WM_GETTEXT,(WORD)MAX_QB2_LINELENGTH,(LONG)(LPSTR)str);
    // time specified ?
    if (str[0] == '\0')
    {
      // no, error
      ErrorDialog.Report(ERR_NBEHAVNOTIMESPEC, ERR_HEADER);
      return;
    }
    t = atol(str);
    if (t<0) t = 0;
    pSystem->SetNeutralBehaviorTime(t);
    // check and give a warning when behavior change time is less then warp in time
    if (pSystem->IsSetNeutralWarpInTime())
    {
      if (t < pSystem->GetNeutralWarpInTime()) Message.Report(WARN_NBEHAVTIME, WARN_HEADER);
    }
  } else
  {
    pSystem->SetNeutralBehavior(EnemyWhenAttacked); 
  }

  EndDialog(IDOK);
}
 
//-----------------------------------
// new OnBehaviorDialogButton method
//-----------------------------------
void CNeutralBehaviorDialog::OnBehaviorDialogButton()
{
  CDoubleDialogDialog DoubleDialogDialog;
  CString             CStr;

  // edit the neutral behavior dialogs
  CStr = pSystem->GetSystemName() + NBEHAVDIA_CAPTION;
  DoubleDialogDialog.Setup(pSystem->GetNeutralToFriendDialogs(), pSystem->GetNeutralToEnemyDialogs(),
                           CStr, NTOFRIENDDIA_CAPTION, NTOENEMYDIA_CAPTION);
  DoubleDialogDialog.DoDialog();
}

//-----------------------------------
// new OnWarpInDialogButton method
//-----------------------------------
void CNeutralBehaviorDialog::OnWarpInDialogButton()
{
  CSingleDialogDialog SingleDialogDialog;
  CString             CStr;

  // edit the neutral warp in dialog
  CStr = pSystem->GetSystemName() + NWARPINDIA_CAPTION;
  SingleDialogDialog.Setup(pSystem->GetNeutralWarpInDialogs(), CStr, NWARPINDIA_CAPTION);
  SingleDialogDialog.DoDialog();
}

//-----------------------------------
// new OnWarpOutDialogButton method
//-----------------------------------
void CNeutralBehaviorDialog::OnWarpOutDialogButton()
{
  CSingleDialogDialog SingleDialogDialog;
  CString             CStr;

  // edit the neutral warp out dialog
  CStr = pSystem->GetSystemName() + NWARPOUTDIA_CAPTION;
  SingleDialogDialog.Setup(pSystem->GetNeutralWarpOutDialogs(), CStr, NWARPOUTDIA_CAPTION);
  SingleDialogDialog.DoDialog();
}

//-----------------------------------
// new OnEnemyFriendAfterButton method
//-----------------------------------
void CNeutralBehaviorDialog::OnEnemyFriendAfterButton()
{
  // enable time box
  GetDlgItem(IDC_BEHAVIORTIME)->EnableWindow(TRUE);
}

//-----------------------------------
// new OnEnemyRandomButton method
//-----------------------------------
void CNeutralBehaviorDialog::OnEnemyRandomButton()
{
  // disable time box
  GetDlgItem(IDC_BEHAVIORTIME)->EnableWindow(FALSE);
}

//-----------------------------------
// new OnWarpInNeverButton method
//-----------------------------------
void CNeutralBehaviorDialog::OnWarpInNeverButton()
{
  // disable time box
  GetDlgItem(IDC_NWARPINTIME)->EnableWindow(FALSE);
}

//-----------------------------------
// new OnWarpInAfterButton method
//-----------------------------------
void CNeutralBehaviorDialog::OnWarpInAfterButton()
{
  // enable time box
  GetDlgItem(IDC_NWARPINTIME)->EnableWindow(TRUE);
}

//-----------------------------------
// new OnWarpOutClearedNeverButton method
//-----------------------------------
void CNeutralBehaviorDialog::OnWarpOutClearedNeverButton()
{
  // disable time box
  GetDlgItem(IDC_NWARPOUTTIME)->EnableWindow(FALSE);
}

//-----------------------------------
// new OnWarpOutAfterButton method
//-----------------------------------
void CNeutralBehaviorDialog::OnWarpOutAfterButton()
{
  // enable time box
  GetDlgItem(IDC_NWARPOUTTIME)->EnableWindow(TRUE);
}


//-----------------------------------
// do the dialog
//-----------------------------------
void CNeutralBehaviorDialog::DoDialog()
{
  // do it
  DoModal();
}


//-----------------------------------
// the message map
//-----------------------------------
BEGIN_MESSAGE_MAP(CNeutralBehaviorDialog, CModalDialog)
  ON_BN_CLICKED(IDC_NBEHAVDIALOG,   OnBehaviorDialogButton)
  ON_BN_CLICKED(IDC_NWARPINDIALOG,  OnWarpInDialogButton)
  ON_BN_CLICKED(IDC_NWARPOUTDIALOG, OnWarpOutDialogButton)
  ON_BN_CLICKED(IDC_BEHAVIOR2,      OnEnemyFriendAfterButton)
  ON_BN_CLICKED(IDC_BEHAVIOR3,      OnEnemyFriendAfterButton)
  ON_BN_CLICKED(IDC_BEHAVIOR1,      OnEnemyRandomButton)
  ON_BN_CLICKED(IDC_BEHAVIOR4,      OnEnemyRandomButton)
  ON_BN_CLICKED(IDC_NWARPIN1,       OnWarpInNeverButton)
  ON_BN_CLICKED(IDC_NWARPIN2,       OnWarpInAfterButton)
  ON_BN_CLICKED(IDC_NWARPOUT1,      OnWarpOutClearedNeverButton)
  ON_BN_CLICKED(IDC_NWARPOUT2,      OnWarpOutAfterButton)
  ON_BN_CLICKED(IDC_NWARPOUT3,      OnWarpOutClearedNeverButton)
END_MESSAGE_MAP()



// ----------- the friend enemy action dialog class ----------------------------------

//-----------------------------------
// overriding the Constructor
//-----------------------------------
CFriendEnemyActionDialog::CFriendEnemyActionDialog(CWnd* pParentWnd) : CModalDialog("ACTIONBOX", pParentWnd)
{
  // init
  pConfiguration = NULL;
  uSystemNum = 0;
  pSystem = NULL;
}

//-----------------------------------
// overriding the OnInitDialog method
//-----------------------------------
BOOL CFriendEnemyActionDialog::OnInitDialog()
{
  CString CStr;
  char    str[MAX_QB2_LINELENGTH + 1];
  
  // some basic checks
  if (pConfiguration == NULL) return TRUE;
  if (pSystem == NULL) return TRUE;

  // the the window caption
  CStr = ACTION_CAPTION;
  CStr = CStr + pSystem->GetSystemName();
  SendMessage(WM_SETTEXT,0,(LONG)(const char *)CStr);

  // get the current actions
  // friend warp in
  if (pSystem->IsSetFriendWarpInTime())
  {
    // warp in time set, set the appropriate radio button
    CheckRadioButton(IDC_FWARPIN1, IDC_FWARPIN2, IDC_FWARPIN2);
    // enable time window
    GetDlgItem(IDC_FWARPINTIME)->EnableWindow(TRUE);
    // set time value
    sprintf(str,"%lu",pSystem->GetFriendWarpInTime());
    SendDlgItemMessage(IDC_FWARPINTIME, WM_SETTEXT,0,(LONG)(LPSTR)str);
  } else
  {
    // no warp in set, set the button and disable time window
    // warp in time set, set the appropriate radio button
    CheckRadioButton(IDC_FWARPIN1, IDC_FWARPIN2, IDC_FWARPIN1);
    // disable time window
    GetDlgItem(IDC_FWARPINTIME)->EnableWindow(FALSE);
  }

  // enemy warp in
  if (pSystem->IsSetEnemyWarpInTime())
  {
    // warp in time set, set the appropriate radio button
    CheckRadioButton(IDC_EWARPIN1, IDC_EWARPIN2, IDC_EWARPIN2);
    // enable time window
    GetDlgItem(IDC_EWARPINTIME)->EnableWindow(TRUE);
    // set time value
    sprintf(str,"%lu",pSystem->GetEnemyWarpInTime());
    SendDlgItemMessage(IDC_EWARPINTIME, WM_SETTEXT,0,(LONG)(LPSTR)str);
  } else
  {
    // no warp in set, set the button and disable time window
    CheckRadioButton(IDC_EWARPIN1, IDC_EWARPIN2, IDC_EWARPIN1);
    // disable time window
    GetDlgItem(IDC_EWARPINTIME)->EnableWindow(FALSE);
  }

  // enemy warp out
  if (pSystem->IsSetEnemyWarpOutTime())
  {
    // time given or warp out when cleared ?
    if (pSystem->GetEnemyWarpOutTime() < 0)
    {
      // when cleared, this is not very useful for enemies.
      // unset the warp out time which is the same (never warp out)
      pSystem->UnsetEnemyWarpOutTime();
      CheckRadioButton(IDC_EWARPOUT1, IDC_EWARPOUT2, IDC_EWARPOUT1);
      GetDlgItem(IDC_EWARPOUTTIME)->EnableWindow(FALSE);
    } else
    {
      // warp out time set, set the appropriate radio button
      CheckRadioButton(IDC_EWARPOUT1, IDC_EWARPOUT2, IDC_EWARPOUT2);
      // enable time window
      GetDlgItem(IDC_EWARPOUTTIME)->EnableWindow(TRUE);
      // set time value
      sprintf(str,"%lu",pSystem->GetEnemyWarpOutTime());
      SendDlgItemMessage(IDC_EWARPOUTTIME, WM_SETTEXT,0,(LONG)(LPSTR)str);
    }
  } else
  {
    // no warp out set, set the button and disable time window
    CheckRadioButton(IDC_EWARPOUT1, IDC_EWARPOUT2, IDC_EWARPOUT1);
    // disable time window
    GetDlgItem(IDC_EWARPOUTTIME)->EnableWindow(FALSE);
  }

  // friend warp out
  if (pSystem->IsSetFriendWarpOutTime())
  {
    // time given or warp out when cleared ?
    if (pSystem->GetFriendWarpOutTime() < 0)
    {
      // when cleared, set the appropriate radio button and disable time windows
      CheckRadioButton(IDC_FWARPOUT1, IDC_FJOINESCORT2, IDC_FWARPOUT3);
      GetDlgItem(IDC_FWARPOUTTIME)->EnableWindow(FALSE);
      GetDlgItem(IDC_FJOINESCORTTIME)->EnableWindow(FALSE);
    } else
    {
      // warp out time set, set the appropriate radio button
      CheckRadioButton(IDC_FWARPOUT1, IDC_FJOINESCORT2, IDC_FWARPOUT2);
      // enable warp out time window
      GetDlgItem(IDC_FWARPOUTTIME)->EnableWindow(TRUE);
      GetDlgItem(IDC_FJOINESCORTTIME)->EnableWindow(FALSE);
      // set time value
      sprintf(str,"%lu",pSystem->GetFriendWarpOutTime());
      SendDlgItemMessage(IDC_FWARPOUTTIME, WM_SETTEXT,0,(LONG)(LPSTR)str);
    }
  } else

  // friend join escort (exclusive to friend warp out
  if (pSystem->IsSetFriendJoinEscortTime())
  {
    // time given or join escort when cleared ?
    if (pSystem->GetFriendJoinEscortTime() < 0)
    {
      // when cleared, set the appropriate radio button and disable time windows
      CheckRadioButton(IDC_FWARPOUT1, IDC_FJOINESCORT2, IDC_FJOINESCORT2);
      GetDlgItem(IDC_FWARPOUTTIME)->EnableWindow(FALSE);
      GetDlgItem(IDC_FJOINESCORTTIME)->EnableWindow(FALSE);
    } else
    {
      // join escort time set, set the appropriate radio button
      CheckRadioButton(IDC_FWARPOUT1, IDC_FJOINESCORT2, IDC_FJOINESCORT1);
      // enable join escort time window
      GetDlgItem(IDC_FWARPOUTTIME)->EnableWindow(FALSE);
      GetDlgItem(IDC_FJOINESCORTTIME)->EnableWindow(TRUE);
      // set time value
      sprintf(str,"%lu",pSystem->GetFriendJoinEscortTime());
      SendDlgItemMessage(IDC_FJOINESCORTTIME, WM_SETTEXT,0,(LONG)(LPSTR)str);
    }
  } else
  {
    // no warp out or join escort set, set the button and disable time window
    CheckRadioButton(IDC_FWARPOUT1, IDC_FJOINESCORT2, IDC_FWARPOUT1);
    // disable time windows
    GetDlgItem(IDC_FWARPOUTTIME)->EnableWindow(FALSE);
    GetDlgItem(IDC_FJOINESCORTTIME)->EnableWindow(FALSE);
  }

  return TRUE;
}

//-----------------------------------
// setup data to edit
//-----------------------------------
void CFriendEnemyActionDialog::Setup(CConfiguration *Configuration, unsigned SystemNum)
{
  POSITION pos;

  // assign the pointer to the configuration
  pConfiguration = Configuration;
  // assign the system number
  uSystemNum = SystemNum;
  // get the system
  pos = (pConfiguration->GetSystems())->FindIndex(uSystemNum - 1);
  if (pos != NULL) pSystem = (CSystemData *)(pConfiguration->GetSystems())->GetAt(pos);
}

//-----------------------------------
// overriding the OnOK method
//-----------------------------------
void CFriendEnemyActionDialog::OnOK()
{
  CErrorDialog ErrorDialog;
  CMessageDialog Message;
  int          id;
  long         t;
  char         str[MAX_QB2_LINELENGTH + 1];

  // check the friend warp in selection
  id = GetCheckedRadioButton(IDC_FWARPIN1, IDC_FWARPIN2);
  if (id == IDC_FWARPIN1)
  {
    // no warp in
    pSystem->UnsetFriendWarpInTime();
  } else
  {
    // get the time
    SendDlgItemMessage(IDC_FWARPINTIME, WM_GETTEXT,(WORD)MAX_QB2_LINELENGTH,(LONG)(LPSTR)str);
    // time specified ?
    if (str[0] == '\0')
    {
      // no, error
      ErrorDialog.Report(ERR_FWARPINNOTIMESPEC, ERR_HEADER);
      return;
    }
    t = atol(str);
    if (t<0) t = 0;
    pSystem->SetFriendWarpInTime(t);
  }

  // check the enemy warp in selection
  id = GetCheckedRadioButton(IDC_EWARPIN1, IDC_EWARPIN2);
  if (id == IDC_EWARPIN1)
  {
    // no warp in
    pSystem->UnsetEnemyWarpInTime();
  } else
  {
    // get the time
    SendDlgItemMessage(IDC_EWARPINTIME, WM_GETTEXT,(WORD)MAX_QB2_LINELENGTH,(LONG)(LPSTR)str);
    // time specified ?
    if (str[0] == '\0')
    {
      // no, error
      ErrorDialog.Report(ERR_EWARPINNOTIMESPEC, ERR_HEADER);
      return;
    }
    t = atol(str);
    if (t<0) t = 0;
    pSystem->SetEnemyWarpInTime(t);
  }

  // check the enemy warp out selection
  id = GetCheckedRadioButton(IDC_EWARPOUT1, IDC_EWARPOUT2);
  if (id == IDC_EWARPOUT1)
  {
    // no warp out
    pSystem->UnsetEnemyWarpOutTime();
  } else
  if (id == IDC_EWARPOUT2)
  {
    // get the time
    SendDlgItemMessage(IDC_EWARPOUTTIME, WM_GETTEXT,(WORD)MAX_QB2_LINELENGTH,(LONG)(LPSTR)str);
    // time specified ?
    if (str[0] == '\0')
    {
      // no, error
      ErrorDialog.Report(ERR_EWARPOUTNOTIMESPEC, ERR_HEADER);
      return;
    }
    t = atol(str);
    if (t<0) t = 0;
    pSystem->SetEnemyWarpOutTime(t);
    // check and give a warning when warp out time is less then warp in time
    if (pSystem->IsSetEnemyWarpInTime())
    {
      if (t < pSystem->GetEnemyWarpInTime()) Message.Report(WARN_EWARPOUTTIME, WARN_HEADER);
    }
  } 

  // check the friend warp out selection
  id = GetCheckedRadioButton(IDC_FWARPOUT1, IDC_FJOINESCORT2);
  if (id == IDC_FWARPOUT1)
  {
    // no warp out or join escort
    pSystem->UnsetFriendWarpOutTime();
    pSystem->UnsetFriendJoinEscortTime();
  } else
  if (id == IDC_FWARPOUT2)
  {
    // no join escort
    pSystem->UnsetFriendJoinEscortTime();
    // get the warp out time
    SendDlgItemMessage(IDC_FWARPOUTTIME, WM_GETTEXT,(WORD)MAX_QB2_LINELENGTH,(LONG)(LPSTR)str);
    // time specified ?
    if (str[0] == '\0')
    {
      // no, error
      ErrorDialog.Report(ERR_FWARPOUTNOTIMESPEC, ERR_HEADER);
      return;
    }
    t = atol(str);
    if (t<0) t = 0;
    pSystem->SetFriendWarpOutTime(t);
    // check and give a warning when warp out time is less then warp in time
    if (pSystem->IsSetFriendWarpInTime())
    {
      if (t < pSystem->GetFriendWarpInTime()) Message.Report(WARN_FWARPOUTTIME, WARN_HEADER);
    }
  } else
  if (id == IDC_FWARPOUT3)
  {
    // no join escort
    pSystem->UnsetFriendJoinEscortTime();
    // warp out when cleared
    pSystem->SetFriendWarpOutTime(-1);
  } else
  if (id == IDC_FJOINESCORT1)
  {
    // no warp out
    pSystem->UnsetFriendWarpOutTime();
    // get the join escort time
    SendDlgItemMessage(IDC_FJOINESCORTTIME, WM_GETTEXT,(WORD)MAX_QB2_LINELENGTH,(LONG)(LPSTR)str);
    // time specified ?
    if (str[0] == '\0')
    {
      // no, error
      ErrorDialog.Report(ERR_JOINESCORTNOTIMESPEC, ERR_HEADER);
      return;
    }
    t = atol(str);
    if (t<0) t = 0;
    pSystem->SetFriendJoinEscortTime(t);
    // check and give a warning when join escort time is less then warp in time
    if (pSystem->IsSetFriendWarpInTime())
    {
      if (t < pSystem->GetFriendWarpInTime()) Message.Report(WARN_JOINESCORTTIME, WARN_HEADER);
    }
  } else
  {
    // no warp out
    pSystem->UnsetFriendWarpOutTime();
    // join escort when cleared
    pSystem->SetFriendJoinEscortTime(-1);
  }


  EndDialog(IDOK);
}
 
//-----------------------------------
// new OnFriendWarpInDialogButton method
//-----------------------------------
void CFriendEnemyActionDialog::OnFriendWarpInDialogButton()
{
  CSingleDialogDialog SingleDialogDialog;
  CString             CStr;

  // edit the friend warp in dialog
  CStr = pSystem->GetSystemName() + FWARPINDIA_CAPTION;
  SingleDialogDialog.Setup(pSystem->GetFriendWarpInDialogs(), CStr, FWARPINDIA_CAPTION);
  SingleDialogDialog.DoDialog();
}

//-----------------------------------
// new OnEnemyWarpInDialogButton method
//-----------------------------------
void CFriendEnemyActionDialog::OnEnemyWarpInDialogButton()
{
  CSingleDialogDialog SingleDialogDialog;
  CString             CStr;

  // edit the enemy warp in dialog
  CStr = pSystem->GetSystemName() + EWARPINDIA_CAPTION;
  SingleDialogDialog.Setup(pSystem->GetEnemyWarpInDialogs(), CStr, EWARPINDIA_CAPTION);
  SingleDialogDialog.DoDialog();
}

//-----------------------------------
// new OnFriendWarpOutDialogButton method
//-----------------------------------
void CFriendEnemyActionDialog::OnFriendWarpOutDialogButton()
{
  CDoubleDialogDialog DoubleDialogDialog;
  CString             CStr;

  // edit the friend warp out and join escort dialogs
  CStr = pSystem->GetSystemName() + FWOJEDIA_CAPTION;
  DoubleDialogDialog.Setup(pSystem->GetFriendWarpOutDialogs(), pSystem->GetFriendJoinEscortDialogs(),
                           CStr, FWARPOUTDIA_CAPTION, FJOINESCORTDIA_CAPTION);
  DoubleDialogDialog.DoDialog();
}

//-----------------------------------
// new OnEnemyWarpOutDialogButton method
//-----------------------------------
void CFriendEnemyActionDialog::OnEnemyWarpOutDialogButton()
{
  CSingleDialogDialog SingleDialogDialog;
  CString             CStr;

  // edit the enemy warp out dialog
  CStr = pSystem->GetSystemName() + EWARPOUTDIA_CAPTION;
  SingleDialogDialog.Setup(pSystem->GetEnemyWarpOutDialogs(), CStr, EWARPOUTDIA_CAPTION);
  SingleDialogDialog.DoDialog();
}


//-----------------------------------
// new OnFriendWarpInNeverButton method
//-----------------------------------
void CFriendEnemyActionDialog::OnFriendWarpInNeverButton()
{
  // disable time box
  GetDlgItem(IDC_FWARPINTIME)->EnableWindow(FALSE);
}

//-----------------------------------
// new OnFriendWarpInAfterButton method
//-----------------------------------
void CFriendEnemyActionDialog::OnFriendWarpInAfterButton()
{
  // enable time box
  GetDlgItem(IDC_FWARPINTIME)->EnableWindow(TRUE);
}

//-----------------------------------
// new OnEnemyWarpInNeverButton method
//-----------------------------------
void CFriendEnemyActionDialog::OnEnemyWarpInNeverButton()
{
  // disable time box
  GetDlgItem(IDC_EWARPINTIME)->EnableWindow(FALSE);
}

//-----------------------------------
// new OnEnemyWarpInAfterButton method
//-----------------------------------
void CFriendEnemyActionDialog::OnEnemyWarpInAfterButton()
{
  // enable time box
  GetDlgItem(IDC_EWARPINTIME)->EnableWindow(TRUE);
}

//-----------------------------------
// new OnEnemyWarpOutClearedNeverButton method
//-----------------------------------
void CFriendEnemyActionDialog::OnEnemyWarpOutClearedNeverButton()
{
  // disable time box
  GetDlgItem(IDC_EWARPOUTTIME)->EnableWindow(FALSE);
}

//-----------------------------------
// new OnEnemyWarpOutAfterButton method
//-----------------------------------
void CFriendEnemyActionDialog::OnEnemyWarpOutAfterButton()
{
  // enable time box
  GetDlgItem(IDC_EWARPOUTTIME)->EnableWindow(TRUE);
}

//-----------------------------------
// new OnFriendWarpOutClearedNeverButton method
//-----------------------------------
void CFriendEnemyActionDialog::OnFriendWarpOutClearedNeverButton()
{
  // disable time boxes
  GetDlgItem(IDC_FWARPOUTTIME)->EnableWindow(FALSE);
  GetDlgItem(IDC_FJOINESCORTTIME)->EnableWindow(FALSE);
}

//-----------------------------------
// new OnFriendWarpOutAfterButton method
//-----------------------------------
void CFriendEnemyActionDialog::OnFriendWarpOutAfterButton()
{
  // enable warp out time box
  GetDlgItem(IDC_FWARPOUTTIME)->EnableWindow(TRUE);
  GetDlgItem(IDC_FJOINESCORTTIME)->EnableWindow(FALSE);
}

//-----------------------------------
// new OnFriendJoinEscortAfterButton method
//-----------------------------------
void CFriendEnemyActionDialog::OnFriendJoinEscortAfterButton()
{
  // enable join escort time box
  GetDlgItem(IDC_FWARPOUTTIME)->EnableWindow(FALSE);
  GetDlgItem(IDC_FJOINESCORTTIME)->EnableWindow(TRUE);
}

//-----------------------------------
// do the dialog
//-----------------------------------
void CFriendEnemyActionDialog::DoDialog()
{
  // do it
  DoModal();
}



//-----------------------------------
// the message map
//-----------------------------------
BEGIN_MESSAGE_MAP(CFriendEnemyActionDialog, CModalDialog)
  ON_BN_CLICKED(IDC_FWARPINDIALOG,  OnFriendWarpInDialogButton)
  ON_BN_CLICKED(IDC_EWARPINDIALOG,  OnEnemyWarpInDialogButton)
  ON_BN_CLICKED(IDC_FWARPOUTDIALOG, OnFriendWarpOutDialogButton)
  ON_BN_CLICKED(IDC_EWARPOUTDIALOG, OnEnemyWarpOutDialogButton)
  ON_BN_CLICKED(IDC_FWARPIN1,       OnFriendWarpInNeverButton)
  ON_BN_CLICKED(IDC_FWARPIN2,       OnFriendWarpInAfterButton)
  ON_BN_CLICKED(IDC_EWARPIN1,       OnEnemyWarpInNeverButton)
  ON_BN_CLICKED(IDC_EWARPIN2,       OnEnemyWarpInAfterButton)
  ON_BN_CLICKED(IDC_EWARPOUT1,      OnEnemyWarpOutClearedNeverButton)
  ON_BN_CLICKED(IDC_EWARPOUT2,      OnEnemyWarpOutAfterButton)
  ON_BN_CLICKED(IDC_EWARPOUT3,      OnEnemyWarpOutClearedNeverButton)
  ON_BN_CLICKED(IDC_FWARPOUT1,      OnFriendWarpOutClearedNeverButton)
  ON_BN_CLICKED(IDC_FWARPOUT2,      OnFriendWarpOutAfterButton)
  ON_BN_CLICKED(IDC_FWARPOUT3,      OnFriendWarpOutClearedNeverButton)
  ON_BN_CLICKED(IDC_FJOINESCORT1,   OnFriendJoinEscortAfterButton)
  ON_BN_CLICKED(IDC_FJOINESCORT2,   OnFriendWarpOutClearedNeverButton)
END_MESSAGE_MAP()



// ----------- the single dialog dialog class ------------------------------------------

//-----------------------------------
// overriding the Constructor
//-----------------------------------
CSingleDialogDialog::CSingleDialogDialog(CWnd* pParentWnd) : CModalDialog("DIALOGSBOX2", pParentWnd)
{
  // init
  pDialogList = NULL;
  pCaption = NULL;
  pSubCaption = NULL;
}

//-----------------------------------
// overriding the OnInitDialog method
//-----------------------------------
BOOL CSingleDialogDialog::OnInitDialog()
{
  CString     CStr;
  CDialogData *pDialog;
  POSITION    pos;
  
  // some basic checks
  if (pDialogList == NULL) return TRUE;
  if (pCaption == NULL) return TRUE;
  if (pSubCaption == NULL) return TRUE;

  // the the window captions
  SendMessage(WM_SETTEXT,0,(LONG)(const char *)pCaption);
  SendDlgItemMessage(IDC_DIALOGHEADER21, WM_SETTEXT ,0,(LONG)(const char *)pSubCaption);

  // set dialogs
  CStr = "";
  for (pos = pDialogList->GetHeadPosition(); pos != NULL; )
  {
    // get the lines, put it together and seperate it by CR LF
    pDialog = (CDialogData *)pDialogList->GetNext(pos);
    if (!pDialog->GetVoiceFile().IsEmpty())
    {
      CStr = CStr + "[" + pDialog->GetVoiceFile() + "] ";
    }
    CStr = CStr + pDialog->GetSpeaker();
    CStr = CStr + ": ";
    CStr = CStr + pDialog->GetDialogLine();
    CStr = CStr + "\015\012";
  }
  // set dialog lines
  SendDlgItemMessage(IDC_DIALOG21, WM_SETTEXT ,0,(LONG)(const char *)CStr);

  return TRUE;
}

//-----------------------------------
// setup data to edit
//-----------------------------------
void CSingleDialogDialog::Setup(CDialogList *DialogList, const char *Caption, const char *SubCaption)
{
  // assig caption and dialog list
  pDialogList = DialogList;
  pCaption = (char *)Caption;
  pSubCaption = (char *)SubCaption;
}

//-----------------------------------
// overriding the OnOK method
//-----------------------------------
void CSingleDialogDialog::OnOK()
{
  CDialogData  *pDialog;
  CErrorDialog ErrorDialog;
  CMessageDialog Message;
  POSITION     pos;
  int          i, len;
  BOOL         bTruncate, bDefaultSpeaker, bEmptyLine, bFisRemoved;
  char         str[LINE_LENGTH], strSpeaker[LINE_LENGTH], strVoice[LINE_LENGTH];

  // init the warning flags
  bTruncate = FALSE;
  bDefaultSpeaker = FALSE;
  bEmptyLine = FALSE;
  bFisRemoved = FALSE;

  // clear the old dialog  lines
  for (pos = pDialogList->GetHeadPosition(); pos != NULL; )
  {
    // clear the objects in the list
    pDialog = (CDialogData *)pDialogList->GetNext(pos);
    delete pDialog;
  }
  //clear the list itself
  pDialogList->RemoveAll();
 
  // read the new dialog lines back from the edit box
  for (i = 0; i < ((CEdit *)GetDlgItem(IDC_DIALOG21))->GetLineCount(); i++)
  {
    // get the line
    len = ((CEdit *)GetDlgItem(IDC_DIALOG21))->GetLine(i, str, LINE_LENGTH - 1);
    // set string terminator
    str[len] = '\0';
    // format it
    if (FormatDialog(str, strSpeaker, strVoice, &bTruncate, &bDefaultSpeaker, &bEmptyLine, &bFisRemoved))
    {
      // all is fine, create the object
      pDialog = new CDialogData;
      if (pDialog == NULL)
      {
        ErrorDialog.Report(ERR_OUTOFMEMORY, ERR_HEADER);
        return;
      }
      pDialog->SetSpeaker(strSpeaker);
      pDialog->SetDialogLine(str);
      pDialog->SetVoiceFile(strVoice);
      // add it to the list
      pDialogList->AddTail(pDialog);
    }
  }

  // a warning if truncated
  if (bTruncate) Message.Report(WARN_DIALOGTRUNC, WARN_HEADER);
  // a warning if no speaker
  if (bDefaultSpeaker) Message.Report(WARN_DIALOGNOSPEAKER, WARN_HEADER);
  // a warning if # removed
  if (bFisRemoved) Message.Report(WARN_DIALOGFISREMOVED, WARN_HEADER);
  // a warning if empty dialog line
  if (bEmptyLine) Message.Report(WARN_DIALOGEMPTY, WARN_HEADER);


  EndDialog(IDOK);
}
 

//-----------------------------------
// do the dialog
//-----------------------------------
void CSingleDialogDialog::DoDialog()
{
  // do it
  DoModal();
}



// ----------- the double dialog dialog class ------------------------------------------

//-----------------------------------
// overriding the Constructor
//-----------------------------------
CDoubleDialogDialog::CDoubleDialogDialog(CWnd* pParentWnd) : CModalDialog("DIALOGSBOX1", pParentWnd)
{
  // init
  pDialogList1 = NULL;
  pDialogList2 = NULL;
  pCaption = NULL;
  pSubCaption1 = NULL;
  pSubCaption2 = NULL;
}

//-----------------------------------
// overriding the OnInitDialog method
//-----------------------------------
BOOL CDoubleDialogDialog::OnInitDialog()
{
  CString     CStr;
  CDialogData *pDialog;
  POSITION    pos;
  
  // some basic checks
  if (pDialogList1 == NULL) return TRUE;
  if (pDialogList2 == NULL) return TRUE;
  if (pCaption == NULL) return TRUE;
  if (pSubCaption1 == NULL) return TRUE;
  if (pSubCaption2 == NULL) return TRUE;

  // the the window captions
  SendMessage(WM_SETTEXT,0,(LONG)(const char *)pCaption);
  SendDlgItemMessage(IDC_DIALOGHEADER11, WM_SETTEXT ,0,(LONG)(const char *)pSubCaption1);
  SendDlgItemMessage(IDC_DIALOGHEADER12, WM_SETTEXT ,0,(LONG)(const char *)pSubCaption2);

  // set dialogs 1
  CStr = "";
  for (pos = pDialogList1->GetHeadPosition(); pos != NULL; )
  {
    // get the lines, put it together and seperate it by CR LF
    pDialog = (CDialogData *)pDialogList1->GetNext(pos);
    if (!pDialog->GetVoiceFile().IsEmpty())
    {
      CStr = CStr + "[" + pDialog->GetVoiceFile() + "] ";
    }
    CStr = CStr + pDialog->GetSpeaker();
    CStr = CStr + ": ";
    CStr = CStr + pDialog->GetDialogLine();
    CStr = CStr + "\015\012";
  }
  // set dialog lines
  SendDlgItemMessage(IDC_DIALOG11, WM_SETTEXT ,0,(LONG)(const char *)CStr);

  // set dialogs 2
  CStr = "";
  for (pos = pDialogList2->GetHeadPosition(); pos != NULL; )
  {
    // get the lines, put it together and seperate it by CR LF
    pDialog = (CDialogData *)pDialogList2->GetNext(pos);
    if (!pDialog->GetVoiceFile().IsEmpty())
    {
      CStr = CStr + "[" + pDialog->GetVoiceFile() + "] ";
    }
    CStr = CStr + pDialog->GetSpeaker();
    CStr = CStr + ": ";
    CStr = CStr + pDialog->GetDialogLine();
    CStr = CStr + "\015\012";
  }
  // set dialog lines
  SendDlgItemMessage(IDC_DIALOG12, WM_SETTEXT ,0,(LONG)(const char *)CStr);

  return TRUE;
}

//-----------------------------------
// setup data to edit
//-----------------------------------
void CDoubleDialogDialog::Setup(CDialogList *DialogList1, CDialogList *DialogList2, 
                                const char *Caption, const char *SubCaption1, const char *SubCaption2)
{
  // assig caption and dialog list
  pDialogList1 = DialogList1;
  pDialogList2 = DialogList2;
  pCaption = (char *)Caption;
  pSubCaption1 = (char *)SubCaption1;
  pSubCaption2 = (char *)SubCaption2;
}

//-----------------------------------
// overriding the OnOK method
//-----------------------------------
void CDoubleDialogDialog::OnOK()
{
  CDialogData  *pDialog;
  CErrorDialog ErrorDialog;
  CMessageDialog Message;
  POSITION     pos;
  int          i, len;
  BOOL         bTruncate, bDefaultSpeaker, bEmptyLine, bFisRemoved;
  char         str[LINE_LENGTH], strSpeaker[LINE_LENGTH], strVoice[LINE_LENGTH];

  // init the warning flags
  bTruncate = FALSE;
  bDefaultSpeaker = FALSE;
  bEmptyLine = FALSE;
  bFisRemoved = FALSE;

  // clear the old dialog 1 lines
  for (pos = pDialogList1->GetHeadPosition(); pos != NULL; )
  {
    // clear the objects in the list
    pDialog = (CDialogData *)pDialogList1->GetNext(pos);
    delete pDialog;
  }
  //clear the list itself
  pDialogList1->RemoveAll();
 
  // read the new dialog 1 lines back from the edit box
  for (i = 0; i < ((CEdit *)GetDlgItem(IDC_DIALOG11))->GetLineCount(); i++)
  {
    // get the line
    len = ((CEdit *)GetDlgItem(IDC_DIALOG11))->GetLine(i, str, LINE_LENGTH - 1);
    // set string terminator
    str[len] = '\0';
    // format it
    if (FormatDialog(str, strSpeaker, strVoice, &bTruncate, &bDefaultSpeaker, &bEmptyLine, &bFisRemoved))
    {
      // all is fine, create the object
      pDialog = new CDialogData;
      if (pDialog == NULL)
      {
        ErrorDialog.Report(ERR_OUTOFMEMORY, ERR_HEADER);
        return;
      }
      pDialog->SetSpeaker(strSpeaker);
      pDialog->SetDialogLine(str);
      pDialog->SetVoiceFile(strVoice);
      // add it to the list
      pDialogList1->AddTail(pDialog);
    }
  }

  // clear the old dialog 2 lines
  for (pos = pDialogList2->GetHeadPosition(); pos != NULL; )
  {
    // clear the objects in the list
    pDialog = (CDialogData *)pDialogList2->GetNext(pos);
    delete pDialog;
  }
  //clear the list itself
  pDialogList2->RemoveAll();
 
  // read the new dialog 1 lines back from the edit box
  for (i = 0; i < ((CEdit *)GetDlgItem(IDC_DIALOG12))->GetLineCount(); i++)
  {
    // get the line
    len = ((CEdit *)GetDlgItem(IDC_DIALOG12))->GetLine(i, str, LINE_LENGTH - 1);
    // set string terminator
    str[len] = '\0';
    // format it
    if (FormatDialog(str, strSpeaker, strVoice, &bTruncate, &bDefaultSpeaker, &bEmptyLine, &bFisRemoved))
    {
      // all is fine, create the object
      pDialog = new CDialogData;
      if (pDialog == NULL)
      {
        ErrorDialog.Report(ERR_OUTOFMEMORY, ERR_HEADER);
        return;
      }
      pDialog->SetSpeaker(strSpeaker);
      pDialog->SetDialogLine(str);
      pDialog->SetVoiceFile(strVoice);
      // add it to the list
      pDialogList2->AddTail(pDialog);
    }
  }

  // a warning if truncated
  if (bTruncate) Message.Report(WARN_DIALOGTRUNC, WARN_HEADER);
  // a warning if no speaker
  if (bDefaultSpeaker) Message.Report(WARN_DIALOGNOSPEAKER, WARN_HEADER);
  // a warning if # removed
  if (bFisRemoved) Message.Report(WARN_DIALOGFISREMOVED, WARN_HEADER);
  // a warning if empty dialog line
  if (bEmptyLine) Message.Report(WARN_DIALOGEMPTY, WARN_HEADER);


  EndDialog(IDOK);
}
 

//-----------------------------------
// do the dialog
//-----------------------------------
void CDoubleDialogDialog::DoDialog()
{
  // do it
  DoModal();
}



// ----------- the check configuration dialog class ------------------------------------------

//-----------------------------------
// overriding the Constructor
//-----------------------------------
CCheckConfigDialog::CCheckConfigDialog(CWnd* pParentWnd) : CModalDialog("CHECKBOX", pParentWnd)
{
  // init
  pConfiguration = NULL;
  pAvailableShips = NULL;
  pAvailableSystemsAndPlanets = NULL;
  bMissingObjectFound = FALSE;
  strCaption = NULL;
}

//-----------------------------------
// overriding the OnInitDialog method
//-----------------------------------
BOOL CCheckConfigDialog::OnInitDialog()
{
  CSystemData       *pSystem;
  CSystemPlanetData *pSystemPlanet;
  CShipList         *pShipList;
  CShipData         *pShipData;
  CString           CStr;
  POSITION          pos, pos1;
  LONG              iItem;

  // some basic test
  if (pConfiguration == NULL) return TRUE;
  if (pAvailableShips == NULL) return TRUE;
  if (pAvailableSystemsAndPlanets == NULL) return TRUE;

  // set the caption if not NULL
  if (strCaption != NULL)
    SendMessage(WM_SETTEXT,0,(LONG)(LPSTR)strCaption);
  
  // do the check
  bMissingObjectFound = FALSE;

  // check the player ship class
  CStr = pConfiguration->GetPlayership()->GetShipClass();
  if (pAvailableShips->FindShip(CStr) == NULL)
  {
    // ship class unkown
    bMissingObjectFound = TRUE;
    // add it to the overview list if not contained already
    if (!ExactFindInListBox((CListBox *)GetDlgItem(IDC_MISSEDSHIPSOVERVIEW), CStr, &iItem))
      SendDlgItemMessage(IDC_MISSEDSHIPSOVERVIEW, LB_ADDSTRING,0,(LONG)(const char *)CStr);
    // add it with extra info to the missing ships detail list
    CStr = CStr + CHECK_MISSINGPLAYERSHIP;
    SendDlgItemMessage(IDC_MISSEDSHIPSDETAILS, LB_ADDSTRING,0,(LONG)(const char *)CStr);
  }

  // check the escort ship classes
  pShipList = pConfiguration->GetEscort();
  for (pos1 = pShipList->GetHeadPosition(); pos1 != NULL; )
  {
    // loop through all ships
    pShipData = (CShipData *)pShipList->GetNext(pos1);
    CStr = pShipData->GetShipClass();
    if (pAvailableShips->FindShip(CStr) == NULL)
    {
      // ship class unkown
      bMissingObjectFound = TRUE;
      // add it to the overview list if not contained already
      if (!ExactFindInListBox((CListBox *)GetDlgItem(IDC_MISSEDSHIPSOVERVIEW), CStr, &iItem))
        SendDlgItemMessage(IDC_MISSEDSHIPSOVERVIEW, LB_ADDSTRING,0,(LONG)(const char *)CStr);
      // add it with extra info to the missing ships detail list
      CStr = CStr + CHECK_MISSINGESCORTSHIP;
      SendDlgItemMessage(IDC_MISSEDSHIPSDETAILS, LB_ADDSTRING,0,(LONG)(const char *)CStr);
    }
  }


  // loop through all systems
  for (pos = pConfiguration->GetSystems()->GetHeadPosition(); pos != NULL; )
  {

    pSystem = (CSystemData *)pConfiguration->GetSystems()->GetNext(pos);

    // check the ship classes in the system

    // the friends
    pShipList = pSystem->GetFriends();
    for (pos1 = pShipList->GetHeadPosition(); pos1 != NULL; )
    {
      // loop through all ships
      pShipData = (CShipData *)pShipList->GetNext(pos1);
      CStr = pShipData->GetShipClass();
      if (pAvailableShips->FindShip(CStr) == NULL)
      {
        // ship class unkown
        bMissingObjectFound = TRUE;
        // add it to the overview list if not contained already
        if (!ExactFindInListBox((CListBox *)GetDlgItem(IDC_MISSEDSHIPSOVERVIEW), CStr, &iItem))
          SendDlgItemMessage(IDC_MISSEDSHIPSOVERVIEW, LB_ADDSTRING,0,(LONG)(const char *)CStr);
        // add it with extra info to the missing ships detail list
        CStr = CStr + CHECK_MISSINGFRIENDSHIP + pSystem->GetSystemName();
        SendDlgItemMessage(IDC_MISSEDSHIPSDETAILS, LB_ADDSTRING,0,(LONG)(const char *)CStr);
      }
    }

    // the enemies
    pShipList = pSystem->GetEnemies();
    for (pos1 = pShipList->GetHeadPosition(); pos1 != NULL; )
    {
      // loop through all ships
      pShipData = (CShipData *)pShipList->GetNext(pos1);
      CStr = pShipData->GetShipClass();
      if (pAvailableShips->FindShip(CStr) == NULL)
      {
        // ship class unkown
        bMissingObjectFound = TRUE;
        // add it to the overview list if not contained already
        if (!ExactFindInListBox((CListBox *)GetDlgItem(IDC_MISSEDSHIPSOVERVIEW), CStr, &iItem))
          SendDlgItemMessage(IDC_MISSEDSHIPSOVERVIEW, LB_ADDSTRING,0,(LONG)(const char *)CStr);
        // add it with extra info to the missing ships detail list
        CStr = CStr + CHECK_MISSINGENEMYSHIP + pSystem->GetSystemName();
        SendDlgItemMessage(IDC_MISSEDSHIPSDETAILS, LB_ADDSTRING,0,(LONG)(const char *)CStr);
      }
    }

    // the neutrals
    pShipList = pSystem->GetNeutrals();
    for (pos1 = pShipList->GetHeadPosition(); pos1 != NULL; )
    {
      // loop through all ships
      pShipData = (CShipData *)pShipList->GetNext(pos1);
      CStr = pShipData->GetShipClass();
      if (pAvailableShips->FindShip(CStr) == NULL)
      {
        // ship class unkown
        bMissingObjectFound = TRUE;
        // add it to the overview list if not contained already
        if (!ExactFindInListBox((CListBox *)GetDlgItem(IDC_MISSEDSHIPSOVERVIEW), CStr, &iItem))
          SendDlgItemMessage(IDC_MISSEDSHIPSOVERVIEW, LB_ADDSTRING,0,(LONG)(const char *)CStr);
        // add it with extra info to the missing ships detail list
        CStr = CStr + CHECK_MISSINGNEUTRALSHIP + pSystem->GetSystemName();
        SendDlgItemMessage(IDC_MISSEDSHIPSDETAILS, LB_ADDSTRING,0,(LONG)(const char *)CStr);
      }
    }

    // the hulks
    pShipList = pSystem->GetHulks();
    for (pos1 = pShipList->GetHeadPosition(); pos1 != NULL; )
    {
      // loop through all ships
      pShipData = (CShipData *)pShipList->GetNext(pos1);
      CStr = pShipData->GetShipClass();
      if (pAvailableShips->FindShip(CStr) == NULL)
      {
        // ship class unkown
        bMissingObjectFound = TRUE;
        // add it to the overview list if not contained already
        if (!ExactFindInListBox((CListBox *)GetDlgItem(IDC_MISSEDSHIPSOVERVIEW), CStr, &iItem))
          SendDlgItemMessage(IDC_MISSEDSHIPSOVERVIEW, LB_ADDSTRING,0,(LONG)(const char *)CStr);
        // add it with extra info to the missing ships detail list
        CStr = CStr + CHECK_MISSINGHULKSHIP + pSystem->GetSystemName();
        SendDlgItemMessage(IDC_MISSEDSHIPSDETAILS, LB_ADDSTRING,0,(LONG)(const char *)CStr);
      }
    }

    // the jump gate
    if (pSystem->GetJumpGate()->IsSetShipClass())
    {
      CStr = pSystem->GetJumpGate()->GetShipClass();
      if (pAvailableShips->FindShip(CStr) == NULL)
      {
        // ship class unkown
        bMissingObjectFound = TRUE;
        // add it to the overview list if not contained already
        if (!ExactFindInListBox((CListBox *)GetDlgItem(IDC_MISSEDSHIPSOVERVIEW), CStr, &iItem))
          SendDlgItemMessage(IDC_MISSEDSHIPSOVERVIEW, LB_ADDSTRING,0,(LONG)(const char *)CStr);
        // add it with extra info to the missing ships detail list
        CStr = CStr + CHECK_MISSINGJUMPGATE + pSystem->GetSystemName();
        SendDlgItemMessage(IDC_MISSEDSHIPSDETAILS, LB_ADDSTRING,0,(LONG)(const char *)CStr);
      }
    }

    // check the system name
    CStr = pSystem->GetSystemName();
    if ((pSystemPlanet = pAvailableSystemsAndPlanets->FindSystem(CStr)) == NULL)
    {
      // system unknown   
      bMissingObjectFound = TRUE;
      // add it to the missing systems list
      SendDlgItemMessage(IDC_MISSEDSYSTEMS, LB_ADDSTRING,0,(LONG)(const char *)CStr);
    }

    // if the system is known, check the planets
    if (pSystemPlanet != NULL)
    {

      // a planet as player position given ?
      if (pSystem->IsSetPlayerPosPlanet())
      {
        // yes, check it
        CStr = pSystem->GetPlayerPosPlanet();
        if (!pAvailableSystemsAndPlanets->FindPlanet(pSystemPlanet, CStr))
        {
          // planet unknown
          bMissingObjectFound = TRUE;
          // add it to the overview list if not contained already
          if (!ExactFindInListBox((CListBox *)GetDlgItem(IDC_MISSEDPLANETSOVERVIEW), CStr, &iItem))
            SendDlgItemMessage(IDC_MISSEDPLANETSOVERVIEW, LB_ADDSTRING,0,(LONG)(const char *)CStr);
          // add it with extra info to the missing planets detail list
          CStr = CStr + CHECK_MISSINGPLAYERPLANET + pSystem->GetSystemName();
          SendDlgItemMessage(IDC_MISSEDPLANETSDETAILS, LB_ADDSTRING,0,(LONG)(const char *)CStr);
        }
      }

      // a planet as friend position given ?
      if (pSystem->IsSetFriendPosPlanet())
      {
        // yes, check it
        CStr = pSystem->GetFriendPosPlanet();
        if (!pAvailableSystemsAndPlanets->FindPlanet(pSystemPlanet, CStr))
        {
          // planet unknown
          bMissingObjectFound = TRUE;
          // add it to the overview list if not contained already
          if (!ExactFindInListBox((CListBox *)GetDlgItem(IDC_MISSEDPLANETSOVERVIEW), CStr, &iItem))
            SendDlgItemMessage(IDC_MISSEDPLANETSOVERVIEW, LB_ADDSTRING,0,(LONG)(const char *)CStr);
          // add it with extra info to the missing planets detail list
          CStr = CStr + CHECK_MISSINGFRIENDPLANET + pSystem->GetSystemName();
          SendDlgItemMessage(IDC_MISSEDPLANETSDETAILS, LB_ADDSTRING,0,(LONG)(const char *)CStr);
        }
      }

      // a planet as enemy position given ?
      if (pSystem->IsSetEnemyPosPlanet())
      {
        // yes, check it
        CStr = pSystem->GetEnemyPosPlanet();
        if (!pAvailableSystemsAndPlanets->FindPlanet(pSystemPlanet, CStr))
        {
          // planet unknown
          bMissingObjectFound = TRUE;
          // add it to the overview list if not contained already
          if (!ExactFindInListBox((CListBox *)GetDlgItem(IDC_MISSEDPLANETSOVERVIEW), CStr, &iItem))
            SendDlgItemMessage(IDC_MISSEDPLANETSOVERVIEW, LB_ADDSTRING,0,(LONG)(const char *)CStr);
          // add it with extra info to the missing planets detail list
          CStr = CStr + CHECK_MISSINGENEMYPLANET + pSystem->GetSystemName();
          SendDlgItemMessage(IDC_MISSEDPLANETSDETAILS, LB_ADDSTRING,0,(LONG)(const char *)CStr);
        }
      }

      // a planet as neutral position given ?
      if (pSystem->IsSetNeutralPosPlanet())
      {
        // yes, check it
        CStr = pSystem->GetNeutralPosPlanet();
        if (!pAvailableSystemsAndPlanets->FindPlanet(pSystemPlanet, CStr))
        {
          // planet unknown
          bMissingObjectFound = TRUE;
          // add it to the overview list if not contained already
          if (!ExactFindInListBox((CListBox *)GetDlgItem(IDC_MISSEDPLANETSOVERVIEW), CStr, &iItem))
            SendDlgItemMessage(IDC_MISSEDPLANETSOVERVIEW, LB_ADDSTRING,0,(LONG)(const char *)CStr);
          // add it with extra info to the missing planets detail list
          CStr = CStr + CHECK_MISSINGNEUTRALPLANET + pSystem->GetSystemName();
          SendDlgItemMessage(IDC_MISSEDPLANETSDETAILS, LB_ADDSTRING,0,(LONG)(const char *)CStr);
        }
      }

      // a planet as hulk position given ?
      if (pSystem->IsSetHulkPosPlanet())
      {
        // yes, check it
        CStr = pSystem->GetHulkPosPlanet();
        if (!pAvailableSystemsAndPlanets->FindPlanet(pSystemPlanet, CStr))
        {
          // planet unknown
          bMissingObjectFound = TRUE;
          // add it to the overview list if not contained already
          if (!ExactFindInListBox((CListBox *)GetDlgItem(IDC_MISSEDPLANETSOVERVIEW), CStr, &iItem))
            SendDlgItemMessage(IDC_MISSEDPLANETSOVERVIEW, LB_ADDSTRING,0,(LONG)(const char *)CStr);
          // add it with extra info to the missing planets detail list
          CStr = CStr + CHECK_MISSINGHULKPLANET + pSystem->GetSystemName();
          SendDlgItemMessage(IDC_MISSEDPLANETSDETAILS, LB_ADDSTRING,0,(LONG)(const char *)CStr);
        }
      }

      // a planet as jump gate position given ?
      if (pSystem->IsSetJumpGatePosPlanet())
      {
        // yes, check it
        CStr = pSystem->GetJumpGatePosPlanet();
        if (!pAvailableSystemsAndPlanets->FindPlanet(pSystemPlanet, CStr))
        {
          // planet unknown
          bMissingObjectFound = TRUE;
          // add it to the overview list if not contained already
          if (!ExactFindInListBox((CListBox *)GetDlgItem(IDC_MISSEDPLANETSOVERVIEW), CStr, &iItem))
            SendDlgItemMessage(IDC_MISSEDPLANETSOVERVIEW, LB_ADDSTRING,0,(LONG)(const char *)CStr);
          // add it with extra info to the missing planets detail list
          CStr = CStr + CHECK_MISSINGJUMPGATEPLANET + pSystem->GetSystemName();
          SendDlgItemMessage(IDC_MISSEDPLANETSDETAILS, LB_ADDSTRING,0,(LONG)(const char *)CStr);
        }
      }

    } 

  }

  // fill empty list boxes with -- none --
  if (((CListBox *)GetDlgItem(IDC_MISSEDSHIPSOVERVIEW))->GetCount() == 0)
    SendDlgItemMessage(IDC_MISSEDSHIPSOVERVIEW, LB_ADDSTRING,0,(LONG)CHECK_NONE);
  if (((CListBox *)GetDlgItem(IDC_MISSEDSHIPSDETAILS))->GetCount() == 0)
    SendDlgItemMessage(IDC_MISSEDSHIPSDETAILS, LB_ADDSTRING,0,(LONG)CHECK_NONE);
  if (((CListBox *)GetDlgItem(IDC_MISSEDSYSTEMS))->GetCount() == 0)
    SendDlgItemMessage(IDC_MISSEDSYSTEMS, LB_ADDSTRING,0,(LONG)CHECK_NONE);
  if (((CListBox *)GetDlgItem(IDC_MISSEDPLANETSOVERVIEW))->GetCount() == 0)
    SendDlgItemMessage(IDC_MISSEDPLANETSOVERVIEW, LB_ADDSTRING,0,(LONG)CHECK_NONE);
  if (((CListBox *)GetDlgItem(IDC_MISSEDPLANETSDETAILS))->GetCount() == 0)
    SendDlgItemMessage(IDC_MISSEDPLANETSDETAILS, LB_ADDSTRING,0,(LONG)CHECK_NONE);

  // some remarks on the check result
  if (bMissingObjectFound) SendDlgItemMessage(IDC_CHECKRESULT, WM_SETTEXT ,0,(LONG)CHECK_RESULTBAD);
                      else SendDlgItemMessage(IDC_CHECKRESULT, WM_SETTEXT ,0,(LONG)CHECK_RESULTGOOD);

  return TRUE;
}


//-----------------------------------
// setup data to edit
//-----------------------------------
void CCheckConfigDialog::Setup(CConfiguration *pConfig, CAvailableShips *pAvShips,
                               CAvailableSystemsAndPlanets *pAvSystemsAndPlanets,
                               const char *Caption)

{
  // setup configuration and other lists
  pConfiguration = pConfig;
  pAvailableShips = pAvShips;
  pAvailableSystemsAndPlanets = pAvSystemsAndPlanets;
  strCaption = (char *)Caption;
}


//-----------------------------------
// do the dialog
//-----------------------------------
BOOL CCheckConfigDialog::DoDialog()
{
  // do it
  DoModal();
  // return the check result
  return bMissingObjectFound;
}


// ----------- the KA setup dialog class ---------------------------------------

//-----------------------------------
// overriding the Constructor
//-----------------------------------
CKASetupDialog::CKASetupDialog(CWnd* pParentWnd)
{
  pParentWindow   = pParentWnd;
}


//-----------------------------------
// new DoDialog method
//-----------------------------------
void CKASetupDialog::DoDialog()
{
  CString      CStr;
  LPMALLOC     lpMalloc;
  BROWSEINFO   browseInfo;
  LPITEMIDLIST lpItemIDList;
  char         strDisplayName[_MAX_PATH];
  char         strPath[_MAX_PATH];
  
  // get a handle to the shell memory manager
  if (::SHGetMalloc(&lpMalloc) != NOERROR) return; 

  // setup the structure for the shell browse for folder function
  CStr = SETUPKA_TEXT;
  browseInfo.hwndOwner = pParentWindow->m_hWnd;
  browseInfo.pidlRoot = NULL;     // set root at Desktop
  browseInfo.pszDisplayName = strDisplayName;
  browseInfo.lpszTitle = CStr + strPathToKA;
  browseInfo.ulFlags = BIF_RETURNONLYFSDIRS; // directories only
  browseInfo.lpfn = NULL;             
  browseInfo.lParam = 0;              

  // ask for the folder
  if ((lpItemIDList = SHBrowseForFolder(&browseInfo)) != NULL)
  {
    // get the path
    if (::SHGetPathFromIDList(lpItemIDList, strPath))
    {
      // set it as the new path to KA
      strPathToKA = strPath;

      // append a backslash if not present
      if ((strPathToKA.GetLength() > 0) && (strPathToKA[strPathToKA.GetLength()-1] != '\\'))
        strPathToKA += "\\";

      // set the path to KA in the registry
      RegSetValue(HKEY_LOCAL_MACHINE,QB2EDIT_REGKEY,REG_SZ,strPathToKA.GetBuffer(PATH_LENGTH),strPathToKA.GetLength());

    }
  }

  // free memeory allocated by the shell
  lpMalloc->Free(lpItemIDList);
  lpMalloc->Release();      
}  


// ----------- the import battle dialog class ---------------------------------------

//-----------------------------------
// overriding the Constructor
//-----------------------------------
CImportBattleDialog::CImportBattleDialog(CWnd* pParentWnd)
{
  pParentWindow   = pParentWnd;
  bBattleImported = FALSE;
}



//-----------------------------------
// new DoImport method
//-----------------------------------
BOOL CImportBattleDialog::DoImport()
{
  BOOL               bSamePathAsKA;
  LONG               iItem;
  CErrorDialog       ErrorDialog;
  CMessageDialog     MessageDialog;
  CCheckConfigDialog CheckConfigDialog;
  CConfiguration     Configuration;
  CCfgFile           SourceFile, DestFile;
  char               strBattleName[LINE_LENGTH], line[LINE_LENGTH];


  // is the selected file a .qb2 file ?
  int l = strCurrentSel.GetLength();
  if (l <= 4) return FALSE;
  if ((strCurrentSel[l-4] != '.') || (toupper(strCurrentSel[l-3]) != 'Q') ||
        (toupper(strCurrentSel[l-2]) != 'B') || (strCurrentSel[l-1] != '2')) return FALSE;

  // remove .qb2 to get the battle name
  GetSubString(strBattleName, strCurrentSel, 0, strCurrentSel.GetLength()-5);

  // first see if the battle is not already imported
  bSamePathAsKA = (strCurrentPath.CompareNoCase(strPathToKA + QB2_PATH) == 0);
  if (bSamePathAsKA)
  {
    if (ExactFindInListBox(pBattleListBox, strBattleName, &iItem))
    {
      ErrorDialog.Report(ERR_BATTLEALREADYPRESENT, ERR_HEADER);
      return FALSE;
    }
  }

  // check if the battle is valid
  // try to open and parse the configuration to check
  if (!Configuration.Parse(strBattleName, strCurrentPath, TRUE)) return FALSE;
  // get the list of the available ships
  if (!AvailableShips.Get()) return FALSE;
  // get the list of the available systems and planets
  if (!AvailableSystemsAndPlanets.Get()) return FALSE;
  // check the configuration
  CheckConfigDialog.Setup(&Configuration, &AvailableShips, &AvailableSystemsAndPlanets, IMPORTCHECK_CAPTION);
  if (CheckConfigDialog.DoDialog())
  {
    // missing objects found, confirm import
    CConfirmDialog confirmDialog(NULL, CONFIRM_IMPORT);
    confirmDialog.DoModal();
    if (!confirmDialog.GetResult()) return FALSE;
  }

  // do we have to copy the battle file ?
  if (!bSamePathAsKA)
  {
    // yes we have, check if a battle file with the same name already exists in our
    // target folder
    if (DestFile.Open(strPathToKA + QB2_PATH + strCurrentSel))
    {
      // confirm to overwrite the battle
      DestFile.Close();
      COverwriteDialog overwriteDialog(NULL, strBattleName);
      overwriteDialog.DoModal();
      if (!overwriteDialog.GetResult()) return FALSE;
    }

    // do the copy
    if (!SourceFile.Open(strCurrentPath + strCurrentSel))
    {
      ErrorDialog.Report(ERR_CFGOPEN, ERR_HEADER);
      return FALSE;
    }
    if (!DestFile.Create(strPathToKA + QB2_PATH + strCurrentSel))
    {
      SourceFile.Close();
      ErrorDialog.Report(ERR_CFGCREATE, ERR_HEADER);
      return FALSE;
    }
    while (SourceFile.ReadLine(line))
    {
      if (!DestFile.WriteLine(line))
      {
        SourceFile.Close();
        DestFile.Close();
        ErrorDialog.Report(ERR_CFGWRITE, ERR_HEADER);
        return FALSE;
      }
    }
    SourceFile.Close();
    DestFile.Close();
  }

  // look if we have to update the configuration descriptor
  if (!ExactFindInListBox(pBattleListBox, strBattleName, &iItem))
  {
    // we have to add the new battle
    pBattleListBox->AddString(strBattleName);
  }

  bBattleImported = TRUE;

  sprintf(line, IMPORT_SUCCESS, strBattleName);
  MessageDialog.Report(line, IMPORT_CAPTION);

  return TRUE;
}


//-----------------------------------
// new DoDialog method
//-----------------------------------
BOOL CImportBattleDialog::DoDialog(CListBox *BattleListBox)
{
  BOOL         result;
  OPENFILENAME ofn;
  char         strFilter[50];
  char         strFilename[LINE_LENGTH];
  char         strFiletitle[LINE_LENGTH];

  // set the battle listbox
  pBattleListBox = BattleListBox;

  // if the global path to the import folder is not set yet, do it now
  if (bImportPathNotSet)
  {
    bImportPathNotSet = FALSE;
    strImportPath = strPathToKA + QB2_PATH;
  }

  // set the current path to the last used path
  strCurrentPath = strImportPath;

  // select the file
  for (;;)
  {
    // set the filter for open file
    memset(strFilter, 0, sizeof(strFilter));
    strcpy(strFilter,IMPORT_FILES);
    strcpy(&strFilter[strlen(strFilter)+1], "*.qb2");
    // set the starting filename
    strFilename[0] = '\0';

    // set all structure members to zero
    memset(&ofn, 0, sizeof(OPENFILENAME));

    // set the get open file descriptor
    ofn.lStructSize = sizeof(OPENFILENAME);
    ofn.hwndOwner = pParentWindow->m_hWnd;
    ofn.lpstrFilter = strFilter;
    ofn.nFilterIndex = 1;
    ofn.lpstrFile= strFilename;
    ofn.nMaxFile = LINE_LENGTH;
    ofn.lpstrFileTitle = strFiletitle;  
    ofn.nMaxFileTitle = LINE_LENGTH;
    ofn.lpstrInitialDir = (const char *)strCurrentPath;
    ofn.lpstrTitle = IMPORT_CAPTION;
    ofn.Flags = OFN_PATHMUSTEXIST | OFN_FILEMUSTEXIST | OFN_HIDEREADONLY | OFN_EXPLORER;

    // get the filename
    result = GetOpenFileName(&ofn);

    // set new path anyhow if aborted or not
    GetSubString(strFilename, strFilename, 0, ofn.nFileOffset-1);
    strCurrentPath = strFilename;

    // stop if aborted
    if (!result) break;

    // set the battle name
    strCurrentSel = strFiletitle;
    // import it
    if (DoImport()) break;
  }

  // update the global path to the last used path
  strImportPath = strCurrentPath;

  return bBattleImported;
}


// ----------- the save dialog class ---------------------------------------

//-----------------------------------
// overriding the Constructor
//-----------------------------------
CSaveDialog::CSaveDialog(CWnd* pParentWnd) : CModalDialog("FINISHBOX", pParentWnd)
{
  bDoSave = FALSE;
  strConfigname = NULL;
}

//-----------------------------------
// overriding the OnInitDialog method
//-----------------------------------
BOOL CSaveDialog::OnInitDialog()
{

  // set the name of the configuration
  if (strConfigname != NULL)
    SendDlgItemMessage(IDC_FINISHOBJECT, WM_SETTEXT,0,(LONG)(LPSTR)strConfigname);
    
  return TRUE;
}

//-----------------------------------
// do the dialog
//-----------------------------------
BOOL CSaveDialog::DoDialog(char *configname)
{
  // set the text
  strConfigname = configname;
  // do the dialog
  DoModal();
  return bDoSave;
}

//-----------------------------------
// overriding the OnOK method
//-----------------------------------
void CSaveDialog::OnOK()
{
  bDoSave = TRUE;
  EndDialog(IDOK);
}
 
//-----------------------------------
// new OnSaveUnder method
//-----------------------------------
void CSaveDialog::OnSaveUnder()
{
  CSaveUnderDialog SaveUnderDialog;
  CCfgFile         ConfigFile;
  CString          CStr;
  char             strNewname[LINE_LENGTH];  

  // ask for the new name
  if (SaveUnderDialog.GetName(strConfigname, strNewname, NULL))
  {

    // check if a valid name
    StripString(strNewname, strNewname);
    if (strNewname[0]!='\0')
    {
      // check if configuration exists already
      CStr = strPathToKA + QB2_PATH + strNewname + ".qb2";
      if (ConfigFile.Open(CStr))
      {
        // print a warning
        ConfigFile.Close();
        // confirm overwrite
        COverwriteDialog overwriteDialog(NULL, strNewname);
        overwriteDialog.DoModal();
        if (!overwriteDialog.GetResult()) return;
      }
      strcpy(strConfigname, strNewname);
      bDoSave = TRUE;
      EndDialog(IDOK);
    }
  }
}

//-----------------------------------
// the message map
//-----------------------------------
BEGIN_MESSAGE_MAP(CSaveDialog, CModalDialog)
  ON_BN_CLICKED(IDC_SAVEUNDER,       OnSaveUnder)
END_MESSAGE_MAP()



// ----------- the save under dialog class ---------------------------------------

//-----------------------------------
// overriding the Constructor
//-----------------------------------
CSaveUnderDialog::CSaveUnderDialog(CWnd* pParentWnd) : CModalDialog("SAVEUNDERBOX", pParentWnd)
{
  bNameSet = FALSE;
}

//-----------------------------------
// overriding the OnInitDialog method
//-----------------------------------
BOOL CSaveUnderDialog::OnInitDialog()
{

  // set the caption if not NULL
  if (strCaption != NULL)
    SendMessage(WM_SETTEXT,0,(LONG)(LPSTR)strCaption);

  // set the edit text in the dialog
  if (strOldname != NULL)
    SendDlgItemMessage(IDC_SAVEUNDER, WM_SETTEXT,0,(LONG)(LPSTR)strOldname);
  SendDlgItemMessage(IDC_SAVEUNDER, EM_SETSEL);
    
  return TRUE;
}

//-----------------------------------
// get the name
//-----------------------------------
BOOL CSaveUnderDialog::GetName(const char *oldname, char *newname, const char *caption)
{
  // set the text
  strOldname = (char *)oldname;
  // set the caption
  strCaption = (char *)caption;
  // do the dialog
  DoModal();
  if (bNameSet)
  {
    // get the name
    strcpy(newname,strNewName);
    return TRUE;
  }
  return FALSE;
}

//-----------------------------------
// overriding the OnOK method
//-----------------------------------
void CSaveUnderDialog::OnOK()
{
  bNameSet = TRUE;
  // get the name
  SendDlgItemMessage(IDC_SAVEUNDER, WM_GETTEXT,(WORD)LINE_LENGTH - 1,(LONG)(LPSTR)strNewName);
  EndDialog(IDOK);
}


// ----------- the ship name dialog class ---------------------------------------

//-----------------------------------
// overriding the Constructor
//-----------------------------------
CShipNameDialog::CShipNameDialog(CWnd* pParentWnd) : CModalDialog("NAMEBOX", pParentWnd)
{
  bNameSet = FALSE;
}

//-----------------------------------
// overriding the OnInitDialog method
//-----------------------------------
BOOL CShipNameDialog::OnInitDialog()
{

  // set the text of the caption in the dialog
  if (strCaption != NULL)
    SendDlgItemMessage(IDC_SNCAPTION, WM_SETTEXT,0,(LONG)(LPSTR)strCaption);
  // set the edit text in the dialog
  if (strOldname != NULL)
    SendDlgItemMessage(IDC_SHIPNAME, WM_SETTEXT,0,(LONG)(LPSTR)strOldname);
  SendDlgItemMessage(IDC_SHIPNAME, EM_SETSEL);
    
  return TRUE;
}

//-----------------------------------
// get the name
//-----------------------------------
BOOL CShipNameDialog::GetName(const char *caption, const char *oldname, char *newname)
{
  // set the text and the captain
  strCaption = (char *)caption;
  strOldname = (char *)oldname;
  // do the dialog
  DoModal();
  if (bNameSet)
  {
    // get the name
    strcpy(newname,strNewName);
    return TRUE;
  }
  return FALSE;
}

//-----------------------------------
// overriding the OnOK method
//-----------------------------------
void CShipNameDialog::OnOK()
{
  bNameSet = TRUE;
  // get the name
  SendDlgItemMessage(IDC_SHIPNAME, WM_GETTEXT,(WORD)MAX_QB2_LINELENGTH - 1,(LONG)(LPSTR)strNewName);
  EndDialog(IDOK);
}
 


// ----------- the create new config dialog class ---------------------------------------

//-----------------------------------
// overriding the Constructor
//-----------------------------------
CCreateNewDialog::CCreateNewDialog(CWnd* pParentWnd) : CModalDialog("CREATEBOX", pParentWnd)
{
  bNameSet = FALSE;
}

//-----------------------------------
// overriding the OnInitDialog method
//-----------------------------------
BOOL CCreateNewDialog::OnInitDialog()
{

  // set the edit text in the dialog
  if (strOldname != NULL)
    SendDlgItemMessage(IDC_NEWCONF, WM_SETTEXT,0,(LONG)(LPSTR)strOldname);
  SendDlgItemMessage(IDC_NEWCONF, EM_SETSEL);
    
  return TRUE;
}

//-----------------------------------
// get the name
//-----------------------------------
BOOL CCreateNewDialog::GetName(const char *oldname, char *newname)
{
  // set the text
  strOldname = (char *)oldname;
  // do the dialog
  DoModal();
  if (bNameSet)
  {
    // get the name
    strcpy(newname,strNewName);
    return TRUE;
  }
  return FALSE;
}

//-----------------------------------
// overriding the OnOK method
//-----------------------------------
void CCreateNewDialog::OnOK()
{
  bNameSet = TRUE;
  // get the name
  SendDlgItemMessage(IDC_NEWCONF, WM_GETTEXT,(WORD)LINE_LENGTH - 1,(LONG)(LPSTR)strNewName);
  EndDialog(IDOK);
}
 


// ----------- the delete dialog class ---------------------------------------

//-----------------------------------
// overriding the Constructor
//-----------------------------------
CDeleteDialog::CDeleteDialog(CWnd* pParentWnd, const char *strDelObj) : CModalDialog("DELETEBOX", pParentWnd)
{
  bDeleteIt = FALSE;
  strDeleteObject = strDelObj;
}

//-----------------------------------
// overriding the OnInitDialog method
//-----------------------------------
BOOL CDeleteDialog::OnInitDialog()
{

  // set the text of the object to delete in the dialog
  if (strDeleteObject != NULL)
    SendDlgItemMessage(IDC_DELOBJECT, WM_SETTEXT,0,(LONG)(LPSTR)strDeleteObject);
  return TRUE;
}

//-----------------------------------
// overriding the OnOK method
//-----------------------------------
void CDeleteDialog::OnOK()
{
  // set delete flag
  bDeleteIt = TRUE;
  EndDialog(IDOK);
}
 

//-----------------------------------
// new GetResult method
//-----------------------------------
BOOL CDeleteDialog::GetResult()
{

  // return result
  return bDeleteIt;
}


 
// ----------- the general confirm dialog class ---------------------------------------

//-----------------------------------
// overriding the Constructor
//-----------------------------------
CConfirmDialog::CConfirmDialog(CWnd* pParentWnd, const char *strConfObj) : CModalDialog("CONFIRMBOX", pParentWnd)
{
  bConfirmed = FALSE;
  strConfirmObject = strConfObj;
}

//-----------------------------------
// overriding the OnInitDialog method
//-----------------------------------
BOOL CConfirmDialog::OnInitDialog()
{

  // set the confirm text 
  if (strConfirmObject != NULL)
    SendDlgItemMessage(IDC_CONFIRMOBJECT, WM_SETTEXT,0,(LONG)(LPSTR)strConfirmObject);
  return TRUE;
}

//-----------------------------------
// overriding the OnOK method
//-----------------------------------
void CConfirmDialog::OnOK()
{
  // set delete flag
  bConfirmed = TRUE;
  EndDialog(IDOK);
}
 

//-----------------------------------
// new GetResult method
//-----------------------------------
BOOL CConfirmDialog::GetResult()
{

  // return result
  return bConfirmed;
}


 
// ----------- the replace dialog class ---------------------------------------

//-----------------------------------
// overriding the Constructor
//-----------------------------------
CReplaceDialog::CReplaceDialog(CWnd* pParentWnd, const char *strReplaceObj1, 
                               const char *strReplaceObj2) : CModalDialog("REPLACEBOX", pParentWnd)
{
  bReplaceIt = FALSE;
  strReplaceObject1 = strReplaceObj1;
  strReplaceObject2 = strReplaceObj2;
}

//-----------------------------------
// overriding the OnInitDialog method
//-----------------------------------
BOOL CReplaceDialog::OnInitDialog()
{

  // set the text of the object to replace in the dialog
  if (strReplaceObject1 != NULL)
    SendDlgItemMessage(IDC_REPLACEOBJECT, WM_SETTEXT,0,(LONG)(LPSTR)strReplaceObject1);
  if (strReplaceObject2 != NULL)
    SendDlgItemMessage(IDC_REPLACEOBJECT1, WM_SETTEXT,0,(LONG)(LPSTR)strReplaceObject2);
  return TRUE;
}

//-----------------------------------
// overriding the OnOK method
//-----------------------------------
void CReplaceDialog::OnOK()
{
  // set delete flag
  bReplaceIt = TRUE;
  EndDialog(IDOK);
}
 

//-----------------------------------
// new GetResult method
//-----------------------------------
BOOL CReplaceDialog::GetResult()
{

  // return result
  return bReplaceIt;
}


// ----------- the overwrite dialog class ---------------------------------------

//-----------------------------------
// overriding the Constructor
//-----------------------------------
COverwriteDialog::COverwriteDialog(CWnd* pParentWnd, char *strOverwrObj) : CModalDialog("OVERWRITEBOX", pParentWnd)
{
  bOverwriteIt = FALSE;
  strOverwriteObject = strOverwrObj;
}

//-----------------------------------
// overriding the OnInitDialog method
//-----------------------------------
BOOL COverwriteDialog::OnInitDialog()
{

  // set the text of the object to delete in the dialog
  if (strOverwriteObject != NULL)
    SendDlgItemMessage(IDC_OVERWRITE, WM_SETTEXT,0,(LONG)(LPSTR)strOverwriteObject);
  return TRUE;
}

//-----------------------------------
// overriding the OnOK method
//-----------------------------------
void COverwriteDialog::OnOK()
{
  // set delete flag
  bOverwriteIt = TRUE;
  EndDialog(IDOK);
}
 

//-----------------------------------
// new GetResult method
//-----------------------------------
BOOL COverwriteDialog::GetResult()
{

  // return result
  return bOverwriteIt;
}


// ----------- the cancel dialog class ---------------------------------------

//-----------------------------------
// overriding the Constructor
//-----------------------------------
CCancelDialog::CCancelDialog(CWnd* pParentWnd) : CModalDialog("CANCELBOX", pParentWnd)
{
  bCancel = FALSE;
}


//-----------------------------------
// overriding the OnOK method
//-----------------------------------
void CCancelDialog::OnOK()
{
  // set delete flag
  bCancel = TRUE;
  EndDialog(IDOK);
}
 

//-----------------------------------
// new GetResult method
//-----------------------------------
BOOL CCancelDialog::GetResult()
{

  // return result
  return bCancel;
}


// ----------- the error dialog class ---------------------------------------

//-----------------------------------
// overriding the Constructor
//-----------------------------------
CErrorDialog::CErrorDialog(CWnd* pParentWnd) : CModalDialog("ERRORBOX", pParentWnd)
{
  // init some values
  strCaption = NULL;
  strMessage = NULL;
}

//-----------------------------------
// overriding the OnInitDialog method
//-----------------------------------
BOOL CErrorDialog::OnInitDialog()
{

  // set the text of the caption and the message
  if (strCaption != NULL) SendMessage(WM_SETTEXT,0,(LONG)(LPSTR)strCaption);
  if (strMessage != NULL) SendDlgItemMessage(IDC_ERRMESSAGE, WM_SETTEXT,0,(LONG)(LPSTR)strMessage);
  return TRUE;
}

//-----------------------------------
// reporting the error
//-----------------------------------
void CErrorDialog::Report(const char *message, const char *caption)
{

  // set the text and the captain
  strCaption = (char *)caption;
  strMessage = (char *)message;
  // do the dialog
  DoModal();
}



// ----------- the message dialog class ---------------------------------------

//-----------------------------------
// overriding the Constructor
//-----------------------------------
CMessageDialog::CMessageDialog(CWnd* pParentWnd) : CModalDialog("MESSAGBOX", pParentWnd)
{
  // init some values
  strCaption = NULL;
  strMessage = NULL;
}

//-----------------------------------
// overriding the OnInitDialog method
//-----------------------------------
BOOL CMessageDialog::OnInitDialog()
{

  // set the text of the caption and the message
  if (strCaption != NULL) SendMessage(WM_SETTEXT,0,(LONG)(LPSTR)strCaption);
  if (strMessage != NULL) SendDlgItemMessage(IDC_USERMESSAGE, WM_SETTEXT,0,(LONG)(LPSTR)strMessage);
  return TRUE;
}

//-----------------------------------
// reporting the error
//-----------------------------------
void CMessageDialog::Report(const char *message, const char *caption)
{

  // set the text and the captain
  strCaption = (char *)caption;
  strMessage = (char *)message;
  // do the dialog
  DoModal();
}


// ----------- the main editor class ---------------------------------------

//-----------------------------------
// the constructor
//-----------------------------------
CMainQB2Edit::CMainQB2Edit()
{
}

//-----------------------------------
// the destructor
//-----------------------------------
CMainQB2Edit::~CMainQB2Edit()
{
}

//-----------------------------------
// do edit
//-----------------------------------
BOOL CMainQB2Edit::DoEdit(char *ConfigName, BOOL CreateNewConfig)
{
  CSaveDialog       SaveDialog;
  CMessageDialog    Message;
  CString           CStr;
  EDialogResultType res;
  unsigned short    dialognum;
  unsigned          uSystem;

  // create a new configuration ?
  if (CreateNewConfig)
  {
    // yes, set the name
    Configuration.SetConfigName(ConfigName);
    // setup the empty config
    if (!Configuration.Setup()) return FALSE;
  } else
  {
    // no, load an existing configuration
    if (!Configuration.Parse(ConfigName, NULL, FALSE)) return FALSE;
  }

  // get the list of the available ships
  if (!AvailableShips.Get()) return FALSE;

  // get the list of the available systems and planets
  if (!AvailableSystemsAndPlanets.Get()) return FALSE;

  // start with the first dialog
  dialognum = 1;

  // do the dialogs
  for (;;)
  {

    // the first dialog : the general dialog
    if (dialognum == 1)
    {
      // show the general edit dialog
      GeneralDialog.Setup(&Configuration);
      res = GeneralDialog.DoDialog();
    } else

    // the second dialog : the system dialog
    if (dialognum == 2)
    {
      // show the system friend enemy edit dialog
      SystemDialog.Setup(&Configuration);
      res = SystemDialog.DoDialog(&uSystem);
    } else

    // the third dialog : the system friend enemy dialog 
    if (dialognum == 3)
    {
      // show the system friend enemy edit dialog
      SystemFriendEnemyDialog.Setup(&Configuration, uSystem);
      res = SystemFriendEnemyDialog.DoDialog();
    } else

    // the fourth dialog : the system neutral hulk dialog
    if (dialognum == 4)
    {
      // show the system neutral hulk edit dialog
      SystemNeutralHulkDialog.Setup(&Configuration, uSystem);
      res = SystemNeutralHulkDialog.DoDialog();
    } else

    // the fifth dialog : the jump gate dialog
    if (dialognum == 5)
    {
      // show the jump gate dialog
      SystemJumpGateDialog.Setup(&Configuration, uSystem);
      res = SystemJumpGateDialog.DoDialog();
    }

    // evaluate the result
    if (res == NextDialog) dialognum += 1; else
    if (res == PreviousDialog) dialognum -= 1; else
    if (res == CancelDialog) break; else
    if (res == FinishDialog)
    {
      if (SaveDialog.DoDialog(ConfigName)) break;
    }

  }

  // write the updated configuration
  if (res == FinishDialog)
  {
    if (Configuration.Write(ConfigName))
    {
      // tell the user about success
      CStr = SAVE_TEXT1;
      CStr = CStr + ConfigName;
      CStr = CStr + SAVE_TEXT2;
      Message.Report(CStr, SAVE_CAPTION);
      return TRUE;
    }
  }

  return FALSE;
}

//-----------------------------------
// do check
//-----------------------------------
BOOL CMainQB2Edit::DoCheck(char *ConfigName)
{
  CCheckConfigDialog CheckConfigDialog;

  // open and parse the configuration to check
  if (!Configuration.Parse(ConfigName, NULL, TRUE)) return FALSE;

  // get the list of the available ships
  if (!AvailableShips.Get()) return FALSE;

  // get the list of the available systems and planets
  if (!AvailableSystemsAndPlanets.Get()) return FALSE;

  // check the configuration
  CheckConfigDialog.Setup(&Configuration, &AvailableShips, &AvailableSystemsAndPlanets, NULL);
  CheckConfigDialog.DoDialog();

  return TRUE;
}
